CalendarEditor_cMaxItems = 15;

gCalendarEditor_SelectedDate = -1;
gCalendarEditor_CompiledSchedule = nil;

function CalendarEditor_ShowCompiledSchedule(pDate, pCompiledSchedule)
	-- Make sure the event editor and viewer is closed
	
	CalendarEventEditor_DoneEditing();
	CalendarEventViewer_DoneViewing();
	
	--
	
	gCalendarEditor_SelectedDate = pDate;
	gCalendarEditor_CompiledSchedule = pCompiledSchedule;
	
	-- Set the date being displayed
	
	CalendarEditorFrameSubTitle:SetText(Calendar_GetLongDateString(pDate));
	
	CalendarEditor_BuildCompiledScheduleList(pCompiledSchedule);
	
	ShowUIPanel(CalendarEditorFrame);
end

function CalendarEditor_BuildCompiledScheduleList(pCompiledSchedule)
	local	vItemIndex = 1;
	
	if pCompiledSchedule ~= nil then
		-- Populate the schedule items
		
		for vEventIndex, vCompiledEvent in pCompiledSchedule do
			local	vEventItemName = "CalendarEditorEvent"..vItemIndex;
			
			local	vEventItemFrame = getglobal(vEventItemName);
			local	vEventItemTime = getglobal(vEventItemName.."Time");
			local	vEventItemTitle = getglobal(vEventItemName.."Title");
			local	vEventItemOwner = getglobal(vEventItemName.."Owner");
			
			if vCompiledEvent.mEvent.mType == "Birth" then
				vEventItemTime:SetText(GroupCalendar_cBirthdayEventName);
			else
				local	vTime;
				
				if gGroupCalendar_Settings.ShowEventsInLocalTime then
					vTime = vCompiledEvent.mEvent.mTime + gGroupCalendar_ServerTimeZoneOffset;
				else
					vTime = vCompiledEvent.mEvent.mTime;
				end
				
				vEventItemTime:SetText(Calendar_GetShortTimeString(vTime));
			end
			
			vEventItemTitle:SetText(EventDatabase_GetEventDisplayName(vCompiledEvent.mEvent));
			vEventItemOwner:SetText(vCompiledEvent.mOwner);
			
			vEventItemFrame:Show();
			
			vItemIndex = vItemIndex + 1;
			
			if vItemIndex > CalendarEditor_cMaxItems then
				break;
			end
		end
	end
	
	for vIndex = vItemIndex, CalendarEditor_cMaxItems do
		local	vEventItemFrame = getglobal("CalendarEditorEvent"..vIndex);
		
		vEventItemFrame:Hide();
	end
end

function CalendarEditor_ScheduleChanged(pDate, pSchedule)
	if pDate == gCalendarEditor_SelectedDate then
		local	vCompiledSchedule = EventDatabase_GetCompiledSchedule(pDate);
		
		gCalendarEditor_CompiledSchedule = vCompiledSchedule;
		
		CalendarEditor_BuildCompiledScheduleList(vCompiledSchedule);
	end
end

function CalendarEditor_MajorDatabaseChange()
	local	vCompiledSchedule = EventDatabase_GetCompiledSchedule(gCalendarEditor_SelectedDate);

	gCalendarEditor_CompiledSchedule = vCompiledSchedule;

	CalendarEditor_BuildCompiledScheduleList(vCompiledSchedule);
end

function CalendarEditor_Close()
	HideUIPanel(CalendarEditorFrame);
end

function CalendarEditor_OnShow()
	PlaySound("igCharacterInfoOpen");
end

function CalendarEditor_OnHide()
	PlaySound("igCharacterInfoClose");
	GroupCalendar_EditorClosed();
end

function CalendarEditor_NewEvent()
	local	vDatabase = EventDatabase_GetDatabase(gGroupCalendar_PlayerName, true);
	local	vEvent = EventDatabase_NewEvent(vDatabase, gCalendarEditor_SelectedDate);
	
	CalendarEventEditor_EditEvent(vDatabase, vEvent, true);
end

function CalendarEditor_EditIndexedEvent(pIndex)
	local		vCompiledEvent = gCalendarEditor_CompiledSchedule[pIndex];
	local		vDatabase = EventDatabase_GetDatabase(vCompiledEvent.mOwner);
	
	if vDatabase then
		if vDatabase.IsPlayerOwned then
			CalendarEventEditor_EditEvent(vDatabase, vCompiledEvent.mEvent, false);
		else
			CalendarEventViewer_ViewEvent(vDatabase, vCompiledEvent.mEvent);
		end
	end
end
