gGroupCalendar_MessagePrefix0 = "GC1";
gGroupCalendar_MessagePrefix = gGroupCalendar_MessagePrefix0.."/";

gGroupCalendar_MessagePrefixLength = string.len(gGroupCalendar_MessagePrefix);

gGroupCalendar_Channel =
{
	Name         = nil,
	NameLower    = nil,
	ID           = nil,
	Password     = nil,
	AutoPlayer   = nil,
	Disconnected = false,
};

gGroupCalendar_Queue =
{
	RequestsDelay    = 0,
	Requests         = {},
	
	InboundDelay     = 0,
	InboundMessages  = {},
	
	OutboundDelay    = 0,
	OutboundMessages = {},
	
	DatabaseUpdates  = {},
	RSVPUpdates      = {},
};

gCalendarNetwork_UserTrustCache =
{
};

gCalendarNetwork_RequestDelay =
{
	Init = 3,
	Notices = 5,
	JoinChannel2 = 2,
	OwnedUpdate = 5,
	ProxyUpdateMin = 10,
	ProxyUpdateRange = 15,
	
	RFUMin = 1,
	RFURange = 5,
	
	OwnedNOURange = 2,
	ProxyNOUMin = 6,
	ProxyNOURange = 4,
	
	InboundQueue = 1,
	RequestQueue = 1.5,
	OutboundQueue = 2.0,
	
	GuildUpdateAutoConfig = 2,
	
	CheckDatabaseTrust = 10,
	GuildRosterUpdate = 1,
};

function CalendarNetwork_GetChannelStatus()
	if gGroupCalendar_Channel.ID then
		return "Connected";
	elseif not gGroupCalendar_RealmSettings then
		return "Initializing";
	else
		return "Disconnected";
	end
end

function CalendarNetwork_SetChannel(pChannel, pPassword)
	-- Leave the channel and return if the new channel is nil
	
	if not pChannel then
		CalendarNetwork_LeaveChannel();
		return true;
	end
	
	-- Just return if nothing is actually changing
	
	if gGroupCalendar_Channel.Name
	and strupper(gGroupCalendar_Channel.Name) == strupper(pChannel)
	and gGroupCalendar_Channel.Password == pPassword then
		return false; -- return false to indicate that no change was made
	end
	
	-- Leave the old channel and join the new one
	
	CalendarNetwork_LeaveChannel();
	
	CalendarNetwork_JoinChannel(pChannel, pPassword);
	
	return true;
end

function CalendarNetwork_JoinChannel(pChannelName, pPassword)
	if not pChannelName
	or pChannelName == ""
	or gGroupCalendar_Channel.Disconnected then
		return false;
	end
	
	-- There seems to be some sort of bug with passworded channels getting
	-- screwed up if the UI is reloaded and the solution appears to be to
	-- leave and re-join the channel.  Therefore, the code below...
	
	LeaveChannelByName(pChannelName);
	
	CalendarNetwork_QueueUniqueOpcodeRequest({mOpcode = "JOINCHANNEL2", mChannelName = pChannelName, mPassword = pPassword}, gCalendarNetwork_RequestDelay.JoinChannel2);
end

function CalendarNetwork_JoinChannel2(pChannelName, pPassword)
	--
	
	if gGroupCalendar_Settings.Debug then
		GroupCalendar_DebugMessage("CalendarNetwork_JoinChannel: "..pChannelName);
	end
	
	local	vChannelName = GetChannelName(pChannelName);
	local	vZoneChannel;
	
	if vChannelName and vChannelName ~= 0 then
		if gGroupCalendar_Settings.Debug then
			GroupCalendar_DebugMessage("Found existing channel "..pChannelName.." ("..vChannelName..")");
		end
		
		vChannelName = pChannelName;
	else
		if gGroupCalendar_Settings.Debug then
			GroupCalendar_DebugMessage("Channel "..pChannelName.." not found ("..vChannelName..") joining channel...");
		end
		
		vZoneChannel, vChannelName = JoinChannelByName(pChannelName, pPassword, DEFAULT_CHAT_FRAME:GetID());
		
		if not vZoneChannel then
			GroupCalendar_DebugMessage("GroupCalendar failed to join channel "..pChannelName);
			return false;
		end
		
		if not vChannelName then
			vChannelName = pChannelName;
		end
	end
	
	-- Remove the channel from the chat frame so the user doesn't
	-- have to watch all that data comm. (if we're running debug
	-- code then leave the setting alone)
	
	ChatFrame_RemoveChannel(DEFAULT_CHAT_FRAME, pChannelName);
	
	gGroupCalendar_Channel.Name = pChannelName;
	gGroupCalendar_Channel.Password = pPassword;
	gGroupCalendar_Channel.NameLower = strlower(pChannelName);
	gGroupCalendar_Channel.ID = GetChannelName(gGroupCalendar_Channel.Name);
	
	if gGroupCalendar_Channel.ID
	and gGroupCalendar_Channel.ID ~= 0 then
		if gGroupCalendar_Settings.Debug then
			GroupCalendar_DebugMessage("CalendarNetwork_JoinChannel succeeded channel "..vChannelName.." ID "..gGroupCalendar_Channel.ID);
		end
		
		-- Send update requests/notices
		
		CalendarNetwork_QueueUniqueOpcodeRequest({mOpcode = "NOTICES"}, gCalendarNetwork_RequestDelay.Notices);
	else
		GroupCalendar_DebugMessage("GroupCalendar failed to join channel "..pChannelName);
		gGroupCalendar_Channel.ID = nil;
	end
	
	GroupCalendar_ChannelChanged();
	
	return true;
end

function CalendarNetwork_LeaveChannel()
	if gGroupCalendar_Channel.Name
	and gGroupCalendar_Channel.ID
	and not gGroupCalendar_Channel.Suspended then
		LeaveChannelByName(gGroupCalendar_Channel.Name);
	end

	gGroupCalendar_Channel.Name = nil;
	gGroupCalendar_Channel.Password = nil;
	gGroupCalendar_Channel.NameLower = nil;
	gGroupCalendar_Channel.ID = nil;
	
	GroupCalendar_ChannelChanged();
end

function CalendarNetwork_SuspendChannel()
	if gGroupCalendar_Channel.Suspended
	or not gGroupCalendar_Channel.Name
	or not gGroupCalendar_Channel.ID then
		return;
	end
	
	LeaveChannelByName(gGroupCalendar_Channel.Name);
	gGroupCalendar_Channel.ID = nil;
	
	gGroupCalendar_Channel.Suspended = true;
end

function CalendarNetwork_ResumeChannel()
	if not gGroupCalendar_Channel.Suspended then
		return;
	end
	
	JoinChannelByName(gGroupCalendar_Channel.Name, gGroupCalendar_Channel.Password, DEFAULT_CHAT_FRAME:GetID());
	gGroupCalendar_Channel.ID = GetChannelName(gGroupCalendar_Channel.Name);
	
	ChatFrame_RemoveChannel(DEFAULT_CHAT_FRAME, gGroupCalendar_Channel.Name);
	
	gGroupCalendar_Channel.Suspended = false;
end

function CalendarNetwork_CalendarLoaded()
	-- Set up a deferred initialization
	
	CalendarNetwork_QueueUniqueOpcodeRequest({mOpcode = "INIT"}, gCalendarNetwork_RequestDelay.Init);
end

function CalendarNetwork_ProcessCommandString(pSender, pTrustLevel, pCommandString)
	local	vCommand = CalendarNetwork_ParseCommandString(pCommandString);
	
	if not vCommand then
		if gGroupCalendar_Settings.Debug then
			GroupCalendar_DebugMessage("ProcessCommandString: Couldn't parse ["..pSender.."]:"..pCommandString);
		end
		
		return false;
	end
	
	return CalendarNetwork_ProcessCommand(pSender, pTrustLevel, vCommand);
end

function CalendarNetwork_ProcessCommand(pSender, pTrustLevel, pCommand)
	local	vOpcode = pCommand[1].opcode;
	local	vOperands = pCommand[1].operands;
	
	table.remove(pCommand, 1);
	
	if vOpcode == "DB" then
		local	vUserName = vOperands[1];
		local	vDatabaseID = tonumber(vOperands[2]);
		local	vRevision = tonumber(vOperands[3]);
		
		if not vRevision then
			vRevision = 0;
		end
		
		if CalendarTrust_UserIsTrusted(vUserName) then -- only accept databases for trusted users (or our own)
			CalendarNetwork_ProcessDatabaseCommand(pSender, pTrustLevel, vUserName, vDatabaseID, vRevision, pCommand);
		else
			if gGroupCalendar_Settings.Debug then
				GroupCalendar_DebugMessage("ChannelMessageReceived: User "..vUserName.." is not trusted for DB command");
			end
		end
	elseif vOpcode == "RAT" then
		local	vUserName = vOperands[1];
		local	vDatabaseID = tonumber(vOperands[2]);
		local	vRevision = tonumber(vOperands[3]);
		
		if not vRevision then
			vRevision = 0;
		end
		
		if CalendarTrust_UserIsTrustedForRSVPs(vUserName) then -- only accept RSVP for trusted users (or our own)
			CalendarNetwork_ProcessRSVPCommand(pSender, vUserName, vDatabaseID, vRevision, pCommand);
		else
			if gGroupCalendar_Settings.Debug then
				GroupCalendar_DebugMessage("ChannelMessageReceived: User "..vUserName.." is not trusted for RAT command");
			end
		end
	elseif vOpcode == "GLD" then
		local	vGuildName = vOperands[1];
		local	vMinRank = tonumber(vOperands[2]);
		
		CalendarNetwork_ProcessGuildCommand(pSender, pTrustLevel, vGuildName, vMinRank, pCommand);
	elseif vOpcode == "ALL" then
		CalendarNetwork_ProcessAllCommand(pSender, pTrustLevel, pCommand);
	else
		if gGroupCalendar_Settings.Debug then
			GroupCalendar_DebugMessage("ProcessCommand: Unknown opcode "..vOpcode);
		end
	end
end

function CalendarNetwork_ProcessChangesRFU(pChanges, pDatabaseTag, pPlayerOwned, pUserName, pDatabaseID, pRevision)
	if gGroupCalendar_Settings.Debug then
		GroupCalendar_DebugMessage(pDatabaseTag.." changes update requested for: "..pUserName..","..pRevision);
	end
	
	-- Cancel a queued RFU for the same database if it's redundant
	
	CalendarNetwork_CancelRedundantRFURequest(pDatabaseTag, pUserName, pDatabaseID, pRevision)
	
	-- Just bail out if we don't have the requested database
	
	if not pChanges then
		return;
	end
	
	-- Create the request

	local		vRequest;
	local		vFromRevision;
	local		vForceUpdate = false;
	
	if pChanges.ID ~= pDatabaseID then
		if pPlayerOwned
		or pChanges.ID > pDatabaseID then
			vFromRevision = 0;
			vForceUpdate = true;
		else
			if gGroupCalendar_Settings.Debug then
				GroupCalendar_DebugMessage("Not sending "..pDatabaseTag.." update: Requested database isn't available for "..pUserName..","..pDatabaseID);
			end
			
			return;
		end
	else
		vFromRevision = pRevision;
	end
	
	vRequest = CalendarNetwork_FindUPDRequest(pDatabaseTag, pUserName);
	
	if vRequest then
		if vFromRevision < vRequest.mRevision then
			if gGroupCalendar_Settings.Debug then
				GroupCalendar_DebugMessage("Changing existing "..pDatabaseTag.." request to revision "..vFromRevision.." for "..pUserName);
			end
			
			vRequest.mRevision = vFromRevision;
		else
			if gGroupCalendar_Settings.Debug then
				GroupCalendar_DebugMessage("Existing request for "..pDatabaseTag.." "..pUserName..","..pRevision);
			end
		end
	elseif pChanges.Revision > vFromRevision
	or vForceUpdate then
		vRequest = {};
		
		vRequest.mOpcode = pDatabaseTag.."_UPD";
		vRequest.mUserName = pUserName;
		vRequest.mDatabaseID = pChanges.ID;
		vRequest.mRevision = vFromRevision;
		
		-- Determine a delay
		
		local	vDelay;
		
		if pPlayerOwned then
			vDelay = gCalendarNetwork_RequestDelay.OwnedUpdate;
		else
			vDelay = gCalendarNetwork_RequestDelay.ProxyUpdateMin + math.random() * gCalendarNetwork_RequestDelay.ProxyUpdateRange;
		end
		
		CalendarNetwork_QueueRequest(vRequest, vDelay);
	end
end

function CalendarNetwork_GetDatabaseChanges(pUserName, pCreate)
	local	vDatabase = EventDatabase_GetDatabase(pUserName, pCreate);
	
	if vDatabase then
		return vDatabase, vDatabase.Changes, vDatabase.IsPlayerOwned;
	else
		return nil, nil, nil;
	end
end

function CalendarNetwork_GetDatabaseRSVPChanges(pUserName, pCreate)
	local	vDatabase = EventDatabase_GetDatabase(pUserName, pCreate);
	
	if vDatabase then
		return vDatabase, vDatabase.RSVPs, vDatabase.IsPlayerOwned;
	else
		return nil, nil, nil;
	end
end

function CalendarNetwork_ProcessDatabaseCommand(pSender, pTrustLevel, pUserName, pDatabaseID, pRevision, pCommand)
	local	vOpcode = pCommand[1].opcode;
	local	vOperands = pCommand[1].operands;

	table.remove(pCommand, 1);
	
	local	vDatabase, vChanges, vIsPlayerOwned = CalendarNetwork_GetDatabaseChanges(pUserName, false);
	
	if vOpcode ~= "RFU"
	and vOpcode ~= "RFV"
	and pTrustLevel < 2 then
		-- Players with moderate trust levels (ie, fellow guild members) are
		-- only allowed to requeust updates, not provide updates
		
		return;
	end
	
	if vOpcode == "RFU" then
		CalendarNetwork_ProcessChangesRFU(vChanges, "DB", vIsPlayerOwned, pUserName, pDatabaseID, pRevision);
	elseif vOpcode == "RFV" then
		if vIsPlayerOwned then
			CalendarNetwork_QueueOutboundMessage(gGroupCalendar_MessagePrefix.."VER:"..gGroupCalendar_VersionString);
		end
	elseif vOpcode == "NOU" then
		local		vCurrentRevision = 0;
		local		vDatabaseIDChanged = false;
		
		if not vIsPlayerOwned then
			CalendarNetwork_CancelRedundantNOURequests(vChanges, "DB", pSender, pUserName, pDatabaseID, pRevision);
		end
		
		if vDatabase then
			-- Ignore external updates to our own databases
			
			if vDatabase.IsPlayerOwned then
				-- If someone sent out a NOU for one of our databases and it's older than
				-- ours and ours is now empty, send out a DEL to let them know that it's newer
				-- and empty now
				
				if CalendarChanges_IsEmpty(vChanges) then
					CalendarNetwork_SendEmptyChanges(vChanges, "DB", pUserName);
				end
				
				return;
			end
			
			-- Ignore the update if the ID is older than the one we have
			-- unless it's from the owner
			
			if pDatabaseID < vChanges.ID
			and pSender ~= vDatabase.UserName then
				return;
			end
			
			-- Purge the existing database if the ID is changing
			
			if vChanges.ID ~= pDatabaseID then
				EventDatabase_PurgeDatabase(vDatabase, pDatabaseID);
				
				vCurrentRevision = 0;
				vDatabaseIDChanged = true;
			else
				vCurrentRevision = vChanges.Revision;
			end
		end
		
		if vDatabaseIDChanged
		or vCurrentRevision < pRevision then
			CalendarNetwork_QueueRFURequest(pUserName, "DB", pDatabaseID, pRevision);
		end
	elseif vOpcode == "UPD" then
		-- Begin a database update
		
		local	vSinceRevision = tonumber(vOperands[1]);
		
		if not vIsPlayerOwned then
			-- If we're waiting to request this same update, cancel the request
			
			CalendarNetwork_CancelRedundantRFURequest("DB", pUserName, pDatabaseID, vSinceRevision);

			-- If we're waiting to send this same update, cancel the request
			
			CalendarNetwork_CancelRedundantUPDRequests(vChanges, "DB", pUserName, pDatabaseID, pRevision, vSinceRevision);
		end
		
		CalendarNetwork_BeginDatabaseUpdate(pSender, pUserName, pDatabaseID, pRevision, vSinceRevision);

	elseif vOpcode == "DEL" then
		-- Delete the database since it's empty
		
		if not vIsPlayerOwned then
			CalendarNetwork_CancelRedundantUPDRequests(vChanges, "DB", pUserName, pDatabaseID, pRevision, 0);
		end
	
		CalendarNetwork_DeleteDatabase(pSender, pUserName, pDatabaseID, pRevision);
		
	elseif vOpcode == "EVT" then
		-- Event data
		
		local	vEventID = tonumber(vOperands[1]);
		
		CalendarNetwork_InsertEventUpdate(pSender, pUserName, pDatabaseID, pRevision, vEventID, pCommand);
		
	elseif vOpcode == "END" then
		-- End a database update
		
		local	vSinceRevision = tonumber(vOperands[1]);
		
		CalendarNetwork_EndDatabaseUpdate(pSender, pUserName, pDatabaseID, pRevision, vSinceRevision);
	end
end

function CalendarNetwork_ProcessRSVPCommand(pSender, pUserName, pDatabaseID, pRevision, pCommand)
	local	vOpcode = pCommand[1].opcode;
	local	vOperands = pCommand[1].operands;

	table.remove(pCommand, 1);
	
	local	vDatabase, vChanges, vIsPlayerOwned = CalendarNetwork_GetDatabaseRSVPChanges(pUserName, false);
	
	if vOpcode == "RFU" then
		CalendarNetwork_ProcessChangesRFU(vChanges, "RAT", vIsPlayerOwned, pUserName, pDatabaseID, pRevision);
	elseif vOpcode == "NOU" then
		local		vCurrentRevision = 0;
		local		vDatabaseIDChanged = false;
		
		if not vIsPlayerOwned then
			CalendarNetwork_CancelRedundantNOURequests(vChanges, "RAT", pSender, pUserName, pDatabaseID, pRevision);
		end
		
		if vDatabase then
			-- Ignore external updates to our own databases
			
			if vDatabase.IsPlayerOwned then
				-- If someone sent out a NOU for one of our databases and it's older than
				-- ours and ours is now empty, send out a DEL to let them know that it's newer
				-- and empty now
				
				if CalendarChanges_IsEmpty(vChanges) then
					CalendarNetwork_SendEmptyChanges(vChanges, "RAT", pUserName);
				end
				
				return;
			end

			-- Ignore the update if the ID is older than the one we have
			-- unless it's from the owner
			
			if pDatabaseID < vChanges.ID
			and pSender ~= vDatabase.UserName then
				return;
			end
			
			-- Purge the existing database if the ID is changing
			
			if vChanges.ID ~= pDatabaseID then
				EventDatabase_PurgeRSVPs(vDatabase, pDatabaseID);
				vCurrentRevision = 0;
				
				vDatabaseIDChanged = true;
			else
				vCurrentRevision = vChanges.Revision;
			end
		end
		
		if vCurrentRevision < pRevision
		or vDatabaseIDChanged then
			CalendarNetwork_QueueRFURequest(pUserName, "RAT", pDatabaseID, pRevision);
		end
	elseif vOpcode == "UPD" then
		-- Begin a RSVP database update
		
		local	vSinceRevision = tonumber(vOperands[1]);
		
		if not vIsPlayerOwned then
			-- If we're waiting to ask for an update to this database, cancel the request
			
			CalendarNetwork_CancelRedundantRFURequest("RAT", pUserName, pDatabaseID, vSinceRevision);
			
			-- If we're waiting to send this same update, cancel the request
			
			CalendarNetwork_CancelRedundantUPDRequests(vChanges, "RAT", pUserName, pDatabaseID, pRevision, vSinceRevision);
		end
		
		CalendarNetwork_BeginRSVPUpdate(pSender, pUserName, pDatabaseID, pRevision, vSinceRevision);
		
	elseif vOpcode == "DEL" then
		-- Delete the database since it's empty
		
		if not vIsPlayerOwned then
			CalendarNetwork_CancelRedundantUPDRequests(vChanges, "RAT", pUserName, pDatabaseID, pRevision, 0);
		end
	
		CalendarNetwork_DeleteRSVPs(pSender, pUserName, pDatabaseID, pRevision);
		
	elseif vOpcode == "EVT" then
		-- Event data
		
		CalendarNetwork_InsertRSVPUpdate(pSender, pUserName, pDatabaseID, pRevision, vOperands);
		
	elseif vOpcode == "END" then
		-- End a RSVP update
		
		local	vSinceRevision = tonumber(vOperands[1]);
		
		CalendarNetwork_EndRSVPUpdate(pSender, pUserName, pDatabaseID, pRevision, vSinceRevision);
	end
end

function CalendarNetwork_ProcessGuildCommand(pSender, pTrustLevel, pGuildName, pMinRank, pCommand)
	-- Ignore guild commands if they're not directed at the player's guild
	-- or not at the player's rank
	
	if pGuildName ~= gGroupCalendar_PlayerGuild
	or gGroupCalendar_PlayerGuildRank > pMinRank then
		return;
	end
	
	-- Pass it on to the ALL handler for further processing
	
	CalendarNetwork_ProcessAllCommand(pSender, pTrustLevel, pCommand);
end

function CalendarNetwork_ProcessAllCommand(pSender, pTrustLevel, pCommand)
	local	vOpcode = pCommand[1].opcode;
	local	vOperands = pCommand[1].operands;

	table.remove(pCommand, 1);
	
	if vOpcode == "RFU" then
		CalendarNetwork_SendAllRevisionNotices();
	elseif vOpcode == "RFV" then
		CalendarNetwork_QueueOutboundMessage(gGroupCalendar_MessagePrefix.."VER:"..gGroupCalendar_VersionString);
	end
end

function CalendarNetwork_DatabaseIsNewer(pDatabaseID1, pRevision1, pDatabaseID2, pRevision2)
	if pDatabaseID2 > pDatabaseID1 then
		return true;
	elseif pDatabaseID2 < pDatabaseID1 then
		return false;
	else
		return pRevision2 > pRevision1;
	end
end

function CalendarNetwork_UpdateIsBetterThanOurs(pSender, pUserName, pDatabaseID, pRevision, pDatabase, pChanges)
	-- Updates for our own databases are always worse
	
	if pDatabase.IsPlayerOwned then
		return false;
	end
	
	-- Updates from the owner are always better than ours
	
	if string.lower(pSender) == string.lower(pUserName) then
		return true;
	end
	
	-- If the revision is higher than what we have it's better
	
	return CalendarNetwork_DatabaseIsNewer(pChanges.ID, pChanges.Revision, pDatabaseID, pRevision);
end

function CalendarNetwork_DeleteDatabase(pSender, pUserName, pDatabaseID, pRevision)
	-- Get the database
	
	local		vDatabase = EventDatabase_GetDatabase(pUserName, false);
	
	-- Nothing to do if we don't even have the database
	
	if not vDatabase then
		return;
	end
	
	-- Bail out if our stuff is better
	
	if not CalendarNetwork_UpdateIsBetterThanOurs(pSender, pUserName, pDatabaseID, pRevision, vDatabase, vDatabase.Changes) then
		return;
	end
	
	-- Empty the specified changelist
	
	EventDatabase_PurgeDatabase(vDatabase, pDatabaseID);
	vDatabase.Changes.Revision = pRevision;
end

function CalendarNetwork_BeginDatabaseUpdate(pSender, pUserName, pDatabaseID, pRevision, pSinceRevision)
	-- If the same sender has a pending update already in progress, kill it
	
	CalendarNetwork_KillSendersDatabaseUpdates(pSender);
	
	-- Get the database
	
	local		vDatabase = EventDatabase_GetDatabase(pUserName, true);
	
	-- Ignore updates for our own databases
	
	if vDatabase.IsPlayerOwned then
		return;
	end
	
	--
	
	local	vChanges = vDatabase.Changes;
	
	-- Ignore the update if it isn't from the owner and
	-- our database ID is newer
	
	if string.lower(pSender) ~= string.lower(pUserName)
	and vChanges.ID > pDatabaseID then
		return;
	end
	
	-- If it's from the owner and isn't the same revision as ours,
	-- then delete the database.  If the update is from revision
	-- zero go ahead with the update otherwise ignore it and request
	-- and update from zero
	
	if string.lower(pSender) == string.lower(pUserName)
	and vChanges.ID ~= pDatabaseID then
		EventDatabase_PurgeDatabase(vDatabase, pDatabaseID);
		
		if pSinceRevision ~= 0 then
			-- Request an update and exit
			
			CalendarNetwork_QueueRFURequest(pUserName, "DB", pDatabaseID, 0)
			return;
		end
	end
	
	-- See if there's another update for the same database already in progress
	
	local	vDatabaseUpdate = CalendarNetwork_FindDatabaseUpdate(nil, pUserName, nil);
	
	if vDatabaseUpdate then
		-- If the new update is from the owner or
		-- is a higher revision, then cancel the old update
		
		if string.lower(pSender) == string.lower(pUserName)
		or CalendarNetwork_DatabaseIsNewer(vDatabaseUpdate.mDatabaseID, vDatabaseUpdate.mRevision, pDatabaseID, pRevision)  then
			CalendarNetwork_CancelDatabaseUpdate(nil, pUserName, nil);
		
		-- Otherwise cancel this one
		
		else
			return;
		end
	end
	
	if gGroupCalendar_Settings.Debug then
		GroupCalendar_DebugMessage("CalendarNetwork_BeginDatabaseUpdate: "..pUserName..","..pRevision.." since revision "..pSinceRevision.." from "..pSender);
	end

	-- Can't accept updates which don't cover the last revision we received
	
	if pSinceRevision > vChanges.Revision then
		-- Ask for another copy of the update starting with the revision we need
		
		CalendarNetwork_QueueRFURequest(pUserName, "DB", pDatabaseID, vChanges.Revision)
		return;
	end
	
	-- Create the database update record
	
	vDatabaseUpdate = {};
	
	vDatabaseUpdate.mSender = pSender;
	vDatabaseUpdate.mUserName = pUserName;
	vDatabaseUpdate.mDatabaseID = pDatabaseID;
	vDatabaseUpdate.mRevision = pRevision;
	vDatabaseUpdate.mSinceRevision = pSinceRevision;
	vDatabaseUpdate.mChanges = {};
	
	table.insert(gGroupCalendar_Queue.DatabaseUpdates, vDatabaseUpdate);
end

function CalendarNetwork_InsertEventUpdate(pSender, pUserName, pDatabaseID, pRevision, pEventID, pCommand)
	local	vDatabaseUpdate = CalendarNetwork_FindDatabaseUpdate(pSender, pUserName, pDatabaseID);
	
	if not vDatabaseUpdate then
		return;
	end
	
	local	vChanges = vDatabaseUpdate.mChanges[pRevision];
	
	if not vChanges then
		vChanges = {};
		vDatabaseUpdate.mChanges[pRevision] = vChanges;
	end
	
	-- Process the event command
	
	local	vChange = nil;
	
	local	vOpcode = pCommand[1].opcode;
	local	vOperands = pCommand[1].operands;
	
	if vOpcode == "NEW"
	or vOpcode == "UPD" then
		-- Create the event record
		
		local		vEvent = {};
		
		vChange = {};
		
		vChange.mOpen = true;
		vChange.mCommand = vOpcode;
		vChange.mEventID = pEventID;
		vChange.mEvent = vEvent;
		
		-- Decode the basic fields
		
		vEvent.mID = pEventID;
		vEvent.mType = vOperands[1];
		vEvent.mDate = tonumber(vOperands[2]);
		vEvent.mTime = tonumber(vOperands[3]);
		vEvent.mDuration = tonumber(vOperands[4]);
		vEvent.mMinLevel = tonumber(vOperands[5]);
		vEvent.mMaxLevel = tonumber(vOperands[6]);
		
		table.insert(vChanges, vChange);
	elseif vOpcode == "DEL" then
		
		vChange = {};
		
		vChange.mCommand = vOpcode;
		vChange.mEventID = pEventID;
		
		table.insert(vChanges, vChange);
	else
		local	vIsQuickUpdate = false;
		
		-- Find the change record for this event
		
		vChange = CalendarNetwork_FindOpenEventChangeRecord(vChanges, pEventID);
		
		if not vChange then
			-- Fail if it isn't an opcode which allows for quick update
			-- events
			
			if vOpcode == "END" then
				if gGroupCalendar_Settings.Debug then
					GroupCalendar_DebugMessage("CalendarNetwork_InsertEventUpdate: Open change record not found for event "..pEventID);
				end
				
				return;
			end
			
			vIsQuickUpdate = true;
			
			-- Create the event record
			
			vChange = {};
			
			vChange.mOpen = false; -- Don't open the change since it's only
			                       -- going to be close immediately anyhow
			vChange.mCommand = "UPD";
			vChange.mEventID = pEventID;
			vChange.mEvent =
			{
				mID = pEventID,
			};
			
			table.insert(vChanges, vChange);
		end
		
		if vOpcode == "TIT" then
			vChange.mEvent.mTitle = vOperands[1];
			
		elseif vOpcode == "DSC" then
			vChange.mEvent.mDescription = vOperands[1];
		
		elseif vOpcode == "ATT" then
			-- Attendance information
			
			local	vUserName = vOperands[1];
			
			-- Remove the specified user from the attendence list
			
			if not vChange.mEvent.mAttendance then
				vChange.mEvent.mAttendance = {};
			end
			
			local	vAttendanceString = nil;
			
			if vOperands[2] == nil then
				if vChange.mCommand == "UPD" then
					vAttendanceString = "-";
				end
			else
				local	vNumOperands = table.getn(vOperands);
				
				if vOperands[2] ~= nil then
					vAttendanceString = vOperands[2];
				else
					vAttendanceString = "";
				end
				
				for vOperandIndex = 3, vNumOperands do
					local	vOperand = vOperands[vOperandIndex];
					
					if vOperand ~= nil then
						vAttendanceString = vAttendanceString..","..vOperand;
					else
						vAttendanceString = vAttendanceString..",";
					end
				end
			end
			
			vChange.mEvent.mAttendance[vUserName] = vAttendanceString;
		
		elseif vOpcode == "END" then
			-- Close the event record
			
			vChange.mOpen = nil;
		else
			if gGroupCalendar_Settings.Debug then
				GroupCalendar_DebugMessage("CalendarNetwork_InsertEventUpdate: Unknown opcode "..vOpcode);
			end
		end
	end
end

function CalendarNetwork_EndDatabaseUpdate(pSender, pUserName, pDatabaseID, pRevision, pSinceRevision)
	local	vDatabaseUpdate = CalendarNetwork_FindDatabaseUpdate(pSender, pUserName, pDatabaseID, true);
	
	if not vDatabaseUpdate then
		return;
	end
	
	-- Sanity check: make sure the sinceRevision field matches the original UPD message
	
	if vDatabaseUpdate.mSinceRevision ~= pSinceRevision then
		return;
	end
	
	-- The update was received successfully.
	-- Process the commands in the update
	
	if gGroupCalendar_Settings.Debug then
		GroupCalendar_DebugMessage("CalendarNetwork_EndDatabaseUpdate: Process update for "..pUserName..","..pRevision.." since revision "..pSinceRevision.." from "..pSender);
	end
	
	local	vDatabase = EventDatabase_GetDatabase(pUserName, true);
	local	vDatabaseChanges = vDatabase.Changes;
	
	for vRevision, vChanges in vDatabaseUpdate.mChanges do
		if vRevision > vDatabase.Changes.Revision then
			local	vChangeList = CalendarChanges_Open(vDatabase.Changes, vRevision);
			
			for vChangeIndex, vChange in vChanges do
				if vChange.mCommand == "NEW" then
					if gGroupCalendar_Settings.Debug then
						GroupCalendar_DebugMessage("Adding event "..vChange.mEventID.." for "..pUserName);
					end
					
					if not EventDatabase_FindEventByID(vDatabase, vChange.mEventID) then
						EventDatabase_AddEvent(vDatabase, vChange.mEvent);
					end
				
				elseif vChange.mCommand == "UPD" then
					local	vEvent = EventDatabase_FindEventByID(vDatabase, vChange.mEventID);
					local	vChangedFields = {};
					
					if vEvent then
						for vFieldIndex, vFieldValue in vChange.mEvent do
							if vFieldIndex == "mAttendance" then
								if vEvent.mAttendance == nil then
									vEvent.mAttendance = {};
								end
								
								for vAttendeeName, vAttendanceValue in vFieldValue do
									if vAttendanceValue == "-" then
										vEvent.mAttendance[vAttendeeName] = nil;
									else
										vEvent.mAttendance[vAttendeeName] = vAttendanceValue;
									end
								end
								
								vChangedFields[vFieldIndex] = {op = "UPD", val = vFieldValue};
							else
								vEvent[vFieldIndex] = vFieldValue;
								vChangedFields[vFieldIndex] = {op = "UPD", val = vFieldValue};
							end
						end
						
						EventDatabase_EventChanged(vDatabase, vEvent, vChangedFields);
					end
					
				elseif vChange.mCommand == "DEL" then
					local	vEvent = EventDatabase_FindEventByID(vDatabase, vChange.mEventID);
					
					if vEvent then
						EventDatabase_DeleteEvent(vDatabase, vEvent);
					end
				else
					if gGroupCalendar_Settings.Debug then
						Calendar_DumpArray("Unknown change request: ", vChange);
					end
				end
			end
			
			CalendarChanges_Close(vDatabase.Changes, vRevision);
		end
	end
	
	vDatabaseChanges.ID = pDatabaseID;
	vDatabaseChanges.Revision = pRevision;
end

function CalendarNetwork_CancelDatabaseUpdate(pSender, pUserName, pDatabaseID)
	CalendarNetwork_FindDatabaseUpdate(pSender, pUserName, pDatabaseID, true);
end

function CalendarNetwork_FindOpenEventChangeRecord(pChanges, pEventID)
	for vIndex, vChange in pChanges do
		if vChange.mEventID == pEventID
		and vChange.mOpen then
			return vChange;
		end
	end
	
	return nil;
end

function CalendarNetwork_KillSendersDatabaseUpdates(pSender)
	CalendarNetwork_FindDatabaseUpdate(pSender, nil, nil, true);
end

function CalendarNetwork_FindDatabaseUpdate(pSender, pUserName, pDatabaseID, pDelete)
	for vIndex, vDatabaseUpdate in gGroupCalendar_Queue.DatabaseUpdates do
		if (pSender == nil or vDatabaseUpdate.mSender == pSender)
		and (pUserName == nil or vDatabaseUpdate.mUserName == pUserName)
		and (pDatabaseID == nil or vDatabaseUpdate.mDatabaseID == pDatabaseID) then
			-- Delete the update if requested
			
			if pDelete then
				table.remove(gGroupCalendar_Queue.DatabaseUpdates, vIndex);
			end
			
			return vDatabaseUpdate;
		end
	end
	
	return nil;
end

function CalendarNetwork_DeleteRSVPs(pSender, pUserName, pDatabaseID, pRevision)
	-- Get the database
	
	local		vDatabase = EventDatabase_GetDatabase(pUserName, false);
	
	-- Nothing to do if we don't even have the database
	
	if not vDatabase then
		return;
	end
	
	-- Bail out if our stuff is better
	
	if not CalendarNetwork_UpdateIsBetterThanOurs(pSender, pUserName, pDatabaseID, pRevision, vDatabase, vDatabase.RSVPs) then
		return;
	end
	
	-- Empty the specified changelist
	
	EventDatabase_PurgeRSVPs(vDatabase, pDatabaseID);
	vDatabase.RSVPs.Revision = pRevision;
end

function CalendarNetwork_BeginRSVPUpdate(pSender, pUserName, pDatabaseID, pRevision, pSinceRevision)
	-- If the same sender has a pending update already in progress, kill it
	
	CalendarNetwork_KillSendersRSVPUpdates(pSender);
	
	-- Get the database
	
	local		vDatabase = EventDatabase_GetDatabase(pUserName, true);
	
	-- Don't listen to updates for our own databases
	
	if vDatabase.IsPlayerOwned then
		return;
	end
	
	-- 
	
	local		vChanges = vDatabase.RSVPs;
	
	-- Ignore the update if it isn't from the owner and
	-- our ID is newer
	
	if string.lower(pSender) ~= string.lower(pUserName)
	and vChanges.ID > pDatabaseID then
		return;
	end
	
	-- If it's from the owner and isn't the same revision as ours,
	-- then delete the database.  If the update is from revision
	-- zero go ahead with the update otherwise ignore it and request
	-- and update from zero
	
	if string.lower(pSender) == string.lower(pUserName)
	and vChanges.ID ~= pDatabaseID then
		EventDatabase_PurgeRSVPs(vDatabase, pDatabaseID);
		
		if pSinceRevision ~= 0 then
			-- Request an update and exit
			
			CalendarNetwork_QueueRFURequest(pUserName, "RAT", pDatabaseID, 0)
			return;
		end
	end
	
	-- See if there's another update for the same database already in progress
	
	local	vRSVPUpdate = CalendarNetwork_FindRSVPUpdate(nil, pUserName, nil);
	
	if vRSVPUpdate then
		-- If the new update is from the owner or
		-- is a higher revision, then cancel the old update
		
		if strlower(pSender) == strlower(pUserName)
		or CalendarNetwork_DatabaseIsNewer(vRSVPUpdate.mDatabaseID, vRSVPUpdate.mRevision, pDatabaseID, pRevision)  then
			CalendarNetwork_CancelRSVPUpdate(nil, pUserName, nil);
		
		-- Otherwise cancel this one
		
		else
			return;
		end
	end
	
	if gGroupCalendar_Settings.Debug then
		GroupCalendar_DebugMessage("CalendarNetwork_BeginRSVPUpdate: "..pUserName..","..pRevision.." since revision "..pSinceRevision.." from "..pSender);
	end

	-- Can't accept updates which don't cover the last revision we received
	
	if pSinceRevision > vChanges.Revision then
		-- Ask for another copy of the update starting with the revision we need
		
		CalendarNetwork_QueueRFURequest(pUserName, "RAT", pDatabaseID, vChanges.Revision);
		return;
	end
	
	-- Create the update record
	
	vRSVPUpdate = {};
	
	vRSVPUpdate.mSender = pSender;
	vRSVPUpdate.mOrganizerName = pUserName;
	vRSVPUpdate.mDatabaseID = pDatabaseID;
	vRSVPUpdate.mRevision = pRevision;
	vRSVPUpdate.mSinceRevision = pSinceRevision;
	vRSVPUpdate.mChanges = {};
	
	table.insert(gGroupCalendar_Queue.RSVPUpdates, vRSVPUpdate);
end

function CalendarNetwork_InsertRSVPUpdate(pSender, pUserName, pDatabaseID, pRevision, pEventFields)
	local	vRSVPUpdate = CalendarNetwork_FindRSVPUpdate(pSender, pUserName, pDatabaseID);
	
	if not vRSVPUpdate then
		if gGroupCalendar_Settings.Debug then
			GroupCalendar_DebugMessage("CalendarNetwork_InsertRSVPUpdate: Ignoring update for "..pUserName..","..pDatabaseID..": Update not found");
		end
		return;
	end
	
	local	vChanges = vRSVPUpdate.mChanges[pRevision];
	
	if not vChanges then
		vChanges = {};
		vRSVPUpdate.mChanges[pRevision] = vChanges;
	end
	
	-- Process the event command
	
	local	vChange = EventDatabase_UnpackRSVPFieldArray(pEventFields, pUserName);
	
	table.insert(vChanges, vChange);
end

function CalendarNetwork_EndRSVPUpdate(pSender, pUserName, pDatabaseID, pRevision, pSinceRevision)
	local	vRSVPUpdate = CalendarNetwork_FindRSVPUpdate(pSender, pUserName, pDatabaseID);
	
	if not vRSVPUpdate then
		return;
	end
	
	if vRSVPUpdate.mSinceRevision ~= pSinceRevision then
		if gGroupCalendar_Settings.Debug then
			GroupCalendar_DebugMessage("CalendarNetwork_EndRSVPUpdate: Ignoring update for "..pUserName..","..pDatabaseID..": Revisions don't match ("..vRSVPUpdate.mSinceRevision.." and "..pSinceRevision..")" );
		end
		return;
	end
	
	-- The update was received successfully.
	-- Process the commands in the update
	
	if gGroupCalendar_Settings.Debug then
		GroupCalendar_DebugMessage("CalendarNetwork_EndRSVPUpdate: Process update for "..pUserName..","..pRevision.." since revision "..pSinceRevision.." from "..pSender);
	end
	
	local	vDatabase = EventDatabase_GetDatabase(pUserName, true);
	local	vRSVPChanges = vDatabase.RSVPs;
	
	for vRevision, vChanges in vRSVPUpdate.mChanges do
		if vRevision > vDatabase.RSVPs.Revision then
			if gGroupCalendar_Settings.Debug then
				GroupCalendar_DebugMessage("CalendarNetwork_EndRSVPUpdate: Processing revision "..vRevision);
			end
			
			local	vChangeList = CalendarChanges_Open(vDatabase.RSVPs, vRevision);
			
			for vChangeIndex, vChange in vChanges do
				if gGroupCalendar_Settings.Debug then
					GroupCalendar_DebugMessage("CalendarNetwork_EndRSVPUpdate: Adding change number "..vChangeIndex);
				end
				EventDatabase_AddRSVP(vDatabase, vChange);
			end
			
			CalendarChanges_Close(vDatabase.RSVPs, vRevision);
		else
			if gGroupCalendar_Settings.Debug then
				GroupCalendar_DebugMessage("CalendarNetwork_EndRSVPUpdate: Ignoring revision "..vRevision..": Already exists");
			end
		end
	end
	
	vRSVPChanges.ID = pDatabaseID;
	vRSVPChanges.Revision = pRevision;
end

function CalendarNetwork_CancelRSVPUpdate(pSender, pUserName, pDatabaseID)
	CalendarNetwork_FindRSVPUpdate(pSender, pUserName, pDatabaseID, true);
end

function CalendarNetwork_KillSendersRSVPUpdates(pSender)
	CalendarNetwork_FindRSVPUpdate(pSender, nil, nil, pDelete);
end

function CalendarNetwork_FindRSVPUpdate(pSender, pUserName, pDatabaseID, pDelete)
	for vIndex, vRSVPUpdate in gGroupCalendar_Queue.RSVPUpdates do
		if (pSender == nil or vRSVPUpdate.mSender == pSender)
		and (pUserName == nil or vRSVPUpdate.mOrganizerName == pUserName)
		and (pDatabaseID == nil or vRSVPUpdate.mDatabaseID == pDatabaseID) then
			-- Delete the update if requested
			
			if pDelete then
				table.remove(gGroupCalendar_Queue.RSVPUpdates,vIndex);
			end
			
			return vRSVPUpdate;
		end
	end
	
	return nil;
end

function CalendarNetwork_ParseCommandString(pCommandString)
	
	-- Verify the command begins with the message prefix
	
	if strsub(pCommandString, 1, gGroupCalendar_MessagePrefixLength) ~= gGroupCalendar_MessagePrefix then
		return nil;
	end
	
	local	vCommandString = strsub(pCommandString, gGroupCalendar_MessagePrefixLength);
	
	return CalendarNetwork_ParseCommandSubString(vCommandString)
end

function CalendarNetwork_ParseCommandSubString(pCommandString)
	-- Break the command into parts
	
	local	vCommand = {};
	
	for vOpcode, vOperands in string.gfind(pCommandString, "/(%w+):*([^/]*)") do
		local	vOperation = {};
		
		vOperation.opcode = vOpcode;
		vOperation.operands = CalendarNetwork_ParseParameterString(vOperands);
		
		table.insert(vCommand, vOperation);
	end
	
	return vCommand;
end

function CalendarNetwork_ParseParameterString(pParameterString)
	local	vParameters = {};
	
	for vParameter in string.gfind(pParameterString, "([^,]+)") do
		table.insert(vParameters, vParameter);
	end
	
	return vParameters;
end

function CalendarNetwork_NewEvent(pDatabase, pEvent)
	local	vChangeList = EventDatabase_GetCurrentChangeList(pDatabase);
	
	EventDatabase_AppendNewEvent(vChangeList, pEvent, EventDatabase_GetEventPath(pEvent));
end

function CalendarNetwork_EventChanged(pDatabase, pEvent, pChangedFields)
	local	vChangeList = EventDatabase_GetCurrentChangeList(pDatabase);
	
	EventDatabase_AppendEventUpdate(
			vChangeList,
			pEvent,
			EventDatabase_GetEventPath(pEvent),
			pChangedFields);
end

function CalendarNetwork_RemovingEvent(pDatabase, pEvent)
	EventDatabase_RemoveEventChanges(pDatabase, pEvent);
	
	-- Insert a delete event
	
	local	vChangeList = EventDatabase_GetCurrentChangeList(pDatabase);
	
	table.insert(vChangeList, EventDatabase_GetEventPath(pEvent).."DEL");
end

function CalendarNetwork_SendRevisionChanged(pChanges, pLabel, pUserName)
	-- Just leave if there's no channel to communicate on
	
	if not gGroupCalendar_Channel.ID then
		return;
	end
	
	CalendarNetwork_QueueOutboundMessage(gGroupCalendar_MessagePrefix..CalendarChanges_GetRevisionPath(pLabel, pUserName, pChanges.ID, pChanges.Revision).."NOU");
end

function CalendarNetwork_DBRevisionChanged(pDatabase)
	CalendarNetwork_SendRevisionChanged(pDatabase.Changes, "DB", pDatabase.UserName);
end

function CalendarNetwork_RSVPRevisionChanged(pDatabase)
	CalendarNetwork_SendRevisionChanged(pDatabase.RSVPs, "RAT", pDatabase.UserName);
end

function CalendarNetwork_RequestAllUpdate()
	-- Just leave if there's no channel to communicate on
	
	if not gGroupCalendar_Channel.ID then
		return;
	end
	
	CalendarNetwork_QueueOutboundMessage(gGroupCalendar_MessagePrefix.."ALL/RFU");
end

function CalendarNetwork_RequestGuildUpdate(pGuildName, pMinRank)
	-- Just leave if there's no channel to communicate on
	
	if not gGroupCalendar_Channel.ID then
		return;
	end
	
	if pMinRank then
		CalendarNetwork_QueueOutboundMessage(gGroupCalendar_MessagePrefix.."GLD:"..pGuildName..","..pMinRank.."/RFU");
	else
		CalendarNetwork_QueueOutboundMessage(gGroupCalendar_MessagePrefix.."GLD:"..pGuildName.."/RFU");
	end
end

function CalendarNetwork_RequestUpdate(pDatabase)
	-- Just leave if there's no channel to communicate on
	
	if not gGroupCalendar_Channel.ID then
		return;
	end
	
	CalendarNetwork_QueueRFURequest(pDatabase.UserName, "DB", pDatabase.Changes.ID, pDatabase.Changes.Revision);
end

function CalendarNetwork_SendEmptyChanges(pChanges, pLabel, pUserName)
	CalendarNetwork_QueueOutboundMessage(gGroupCalendar_MessagePrefix..CalendarChanges_GetRevisionPath(pLabel, pUserName, pChanges.ID, pChanges.Revision).."DEL");
end

function CalendarNetwork_SendChanges(pChanges, pLabel, pUserName, pLockdown, pSinceRevision)
	if pLockdown then
		CalendarChanges_LockdownCurrentChangeList(pChanges);
	end
	
	if CalendarChanges_IsEmpty(pChanges) then
		CalendarNetwork_SendEmptyChanges(pChanges, pLabel, pUserName);
		return;
	end
	
	CalendarNetwork_QueueOutboundMessage(gGroupCalendar_MessagePrefix..CalendarChanges_GetRevisionPath(pLabel, pUserName, pChanges.ID, pChanges.Revision).."UPD:"..pSinceRevision);
	
	for vRevision = pSinceRevision + 1, pChanges.Revision do
		local	vRevisionPath = CalendarChanges_GetRevisionPath(pLabel, pUserName, pChanges.ID, vRevision);
		local	vChangeList = pChanges.ChangeList[vRevision];
		
		if vChangeList then
			vChangeList.IsOpen = nil; -- Make sure IsOpen is cleared, a bug may have caused it to remain open
			
			for vIndex, vChange in vChangeList do
				CalendarNetwork_QueueOutboundMessage(gGroupCalendar_MessagePrefix..vRevisionPath..vChange);
			end
		end
	end
	
	CalendarNetwork_QueueOutboundMessage(gGroupCalendar_MessagePrefix..CalendarChanges_GetRevisionPath(pLabel, pUserName, pChanges.ID, pChanges.Revision).."END:"..pSinceRevision);
end

function CalendarNetwork_QueueOutboundMessage(pMessage)
	table.insert(gGroupCalendar_Queue.OutboundMessages, pMessage);
	GroupCalendar_StartUpdateTimer();
end

function CalendarNetwork_QueueInboundMessage(pSender, pTrustLevel, pMessage)
	table.insert(gGroupCalendar_Queue.InboundMessages, {mSender = pSender, mTrustLevel = pTrustLevel, mMessage = pMessage});
	GroupCalendar_StartUpdateTimer();
end

function CalendarNetwork_QueueRequest(pRequest, pDelay)
	pRequest.mDelay = pDelay;
	
	if gGroupCalendar_Settings.Debug then
		GroupCalendar_DebugMessage("CalendarNetwork_QueueRequest: "..pRequest.mOpcode.." in "..pDelay.." seconds");
	end
	
	table.insert(gGroupCalendar_Queue.Requests, pRequest);
	GroupCalendar_StartUpdateTimer();
	
	return true;
end

function CalendarNetwork_QueueUniqueOpcodeRequest(pRequest, pDelay)
	-- Remove an existing request with the same opcode
	
	for vIndex, vRequest in gGroupCalendar_Queue.Requests do
		if vRequest.mOpcode == pRequest.mOpcode then
			table.remove(gGroupCalendar_Queue.Requests, vIndex);
			break;
		end
	end
	
	return CalendarNetwork_QueueRequest(pRequest, pDelay);
end

function CalendarNetwork_QueueUniqueUserRequest(pRequest, pDelay)
	-- Remove an existing request with the same opcode and user name
	
	for vIndex, vRequest in gGroupCalendar_Queue.Requests do
		if vRequest.mOpcode == pRequest.mOpcode
		and vRequest.mUserName == pRequest.mUserName then
			table.remove(gGroupCalendar_Queue.Requests, vIndex);
			break;
		end
	end
	
	return CalendarNetwork_QueueRequest(pRequest, pDelay);
end

function CalendarNetwork_QueueUniqueDatabaseRequest(pRequest, pDelay)
	-- Remove an existing request with the same opcode and user name
	
	for vIndex, vRequest in gGroupCalendar_Queue.Requests do
		if vRequest.mOpcode == pRequest.mOpcode
		and vRequest.mUserName == pRequest.mUserName then
			-- See if the request database ID and revision are better
			-- and update the existing request if it is
			
			if pRequest.mDatabaseID > vRequest.mDatabaseID
			or (pRequest.mDatabaseID == vRequest.mDatabaseID
			and pRequest.mRevision < vRequest.mRevision) then
				vRequest.mDatabaseID = pRequest.mDatabaseID;
				vRequest.mRevision = pRequest.mRevision;
			end
			
			return;
		end
	end
	
	return CalendarNetwork_QueueRequest(pRequest, pDelay);
end

function CalendarNetwork_ArrayContainsArray(pArray, pSubArray)
	for vFieldName, vFieldValue in pSubArray do
		if pArray[vFieldName] ~= vFieldValue then
			return false;
		end
	end
	
	return true;
end

function CalendarNetwork_FindRequest(pRequest)
	for vIndex, vRequest in gGroupCalendar_Queue.Requests do
		if CalendarNetwork_ArrayContainsArray(vRequest, pRequest) then
			return vRequest;
		end
	end
	
	return nil;
end

function CalendarNetwork_FindUPDRequest(pDatabaseTag, pUserName)
	return CalendarNetwork_FindRequest({mOpcode = pDatabaseTag.."_UPD", mUserName = pUserName});
end

function CalendarNetwork_CancelRedundantRFURequest(pDatabaseTag, pUserName, pDatabaseID, pFromRevision)
	local	vOpcode = pDatabaseTag.."_RFU";
	
	for vIndex, vRequest in gGroupCalendar_Queue.Requests do
		if vRequest.mOpcode == vOpcode
		and vRequest.mUserName == pUserName then
			if vRequest.mDatabaseID == pDatabaseID
			and pFromRevision <= vRequest.mRevision then
				if gGroupCalendar_Settings.Debug then
					GroupCalendar_DebugMessage("Removing redundant RFU for "..pDatabaseTag.." "..pUserName..","..pDatabaseID..","..pFromRevision);
				end
				
				table.remove(gGroupCalendar_Queue.Requests, vIndex);
				return false; -- Return false to indicate there is no longer an RFU for this user
			else
				if gGroupCalendar_Settings.Debug then
					GroupCalendar_DebugMessage("Keeping RFU request for "..pDatabaseTag.." "..pUserName..","..vRequest.mDatabaseID..","..vRequest.mRevision);
					GroupCalendar_DebugMessage("Better thean "..pDatabaseTag.." "..pUserName..","..pDatabaseID..","..pFromRevision);
				end
			end
			
			if gGroupCalendar_Settings.Debug then
				GroupCalendar_DebugMessage("RFU for "..pDatabaseTag.." "..pUserName..","..pDatabaseID..","..pFromRevision.." already exists");
			end
			
			return true; -- Return true to indicate there is an RFU for this user
		end
	end
	
	return false; -- Return false to indicate no RFU for this user was found
end

function CalendarNetwork_NeedsUpdateTimer()
	local	vNumRequests = table.getn(gGroupCalendar_Queue.Requests);
	
	if vNumRequests > 0 then
		return true;
	end

	local	vNumInboundMessages = table.getn(gGroupCalendar_Queue.InboundMessages);
	
	if vNumInboundMessages > 0 then
		return true;
	end

	local	vNumOutboundMessages = table.getn(gGroupCalendar_Queue.OutboundMessages);
	
	if vNumOutboundMessages > 0 then
		return true;
	end
end

function CalendarNetwork_ProcessQueues(pElapsed)
	-- Process inbound messages
	
	local	vNumInboundMessages = table.getn(gGroupCalendar_Queue.InboundMessages);
	
	if vNumInboundMessages > 0 then
		gGroupCalendar_Queue.InboundDelay = gGroupCalendar_Queue.InboundDelay - pElapsed;
		
		if gGroupCalendar_Queue.InboundDelay <= 0 then
			
			gGroupCalendar_Queue.InboundDelay = 0;
			
			-- Process up to four messages at a time
			
			local	vNumMessages = 4;
			
			if vNumMessages > vNumInboundMessages then
				vNumMessages = vNumInboundMessages;
			end
			
			if gGroupCalendar_Settings.Debug then
				GroupCalendar_DebugMessage("CalendarNetwork_ProcessQueues: Processing "..vNumMessages.." messages");
			end

			-- Process the messages
			
			for vIndex = 1, vNumMessages do
				local	vMessage = gGroupCalendar_Queue.InboundMessages[1];
				
				CalendarNetwork_ProcessCommandString(vMessage.mSender, vMessage.mTrustLevel, vMessage.mMessage);

				table.remove(gGroupCalendar_Queue.InboundMessages, 1);
			end
			
			gGroupCalendar_Queue.InboundDelay = gCalendarNetwork_RequestDelay.InboundQueue;
		end
		
		-- Terminate processing while there are any pending inbound messages and delay further
		-- processing of the outbound and request queues
		
		
		gGroupCalendar_Queue.OutboundDelay = gCalendarNetwork_RequestDelay.OutboundQueue;
		gGroupCalendar_Queue.RequestsDelay = gCalendarNetwork_RequestDelay.RequestQueue;
		
		return;
	end
	
	-- Process pending requests
	
	local	vNumRequests = table.getn(gGroupCalendar_Queue.Requests);
	
	if vNumRequests > 0 then
		gGroupCalendar_Queue.RequestsDelay = gGroupCalendar_Queue.RequestsDelay - pElapsed;
		
		if gGroupCalendar_Queue.RequestsDelay <= 0 then
			local	vIndex = 1;
			
			while vIndex <= vNumRequests do
				local	vRequest = gGroupCalendar_Queue.Requests[vIndex];
				
				vRequest.mDelay = vRequest.mDelay - pElapsed;
				
				if vRequest.mDelay <= 0 then
					table.remove(gGroupCalendar_Queue.Requests, vIndex);
					vNumRequests = vNumRequests - 1;
					
					CalendarNetwork_ProcessRequest(vRequest);
				else
					vIndex = vIndex + 1;
				end
			end
			
			gGroupCalendar_Queue.RequestsDelay = gCalendarNetwork_RequestDelay.RequestQueue;
		end
	end
	
	-- Process outbound messages
	
	local	vNumOutboundMessages = table.getn(gGroupCalendar_Queue.OutboundMessages);
	
	if vNumOutboundMessages > 0 then
		gGroupCalendar_Queue.OutboundDelay = gGroupCalendar_Queue.OutboundDelay - pElapsed;
		
		if gGroupCalendar_Queue.OutboundDelay <= 0 then
			-- Send up to four messages at a time
			
			local	vNumMessages = 4;
			
			if vNumMessages > vNumOutboundMessages then
				vNumMessages = vNumOutboundMessages;
			end
			
			if gGroupCalendar_Settings.Debug then
				GroupCalendar_DebugMessage("CalendarNetwork_ProcessQueues: Sending "..vNumMessages.." messages");
			end

			-- Send the messages
			
			for vIndex = 1, vNumMessages do
				CalendarNetwork_SendMessage(gGroupCalendar_Queue.OutboundMessages[1]);
				table.remove(gGroupCalendar_Queue.OutboundMessages, 1);
			end
			
			gGroupCalendar_Queue.OutboundDelay = gCalendarNetwork_RequestDelay.OutboundQueue;
		end
	end
end

function CalendarNetwork_ProcessRequest(pRequest)
	if gGroupCalendar_Settings.Debug then
		GroupCalendar_DebugMessage("CalendarNetwork_ProcessRequest: "..pRequest.mOpcode);
	end
	
	local	vDatabase;
	
	if pRequest.mUserName then
		vDatabase = EventDatabase_GetDatabase(pRequest.mUserName, false);
	else
		vDatabase = nil;
	end
	
	if pRequest.mOpcode == "DB_UPD" then
		CalendarNetwork_SendChanges(vDatabase.Changes, "DB", vDatabase.UserName, vDatabase.IsPlayerOwned, pRequest.mRevision);

	elseif pRequest.mOpcode == "RAT_UPD" then
		CalendarNetwork_SendChanges(vDatabase.RSVPs, "RAT", vDatabase.UserName, vDatabase.IsPlayerOwned, pRequest.mRevision);

	elseif pRequest.mOpcode == "DB_NOU" then
		CalendarNetwork_DBRevisionChanged(vDatabase);

	elseif pRequest.mOpcode == "RAT_NOU" then
		CalendarNetwork_RSVPRevisionChanged(vDatabase);

	elseif pRequest.mOpcode == "DB_RFU" then
		local	vCurrentRevision;
		
		if vDatabase then
			vCurrentRevision = vDatabase.Changes.Revision;
		else
			vCurrentRevision = 0;
		end
		
		if vCurrentRevision < pRequest.mRevision
		or vCurrentRevision == 0 then
			CalendarNetwork_QueueOutboundMessage(gGroupCalendar_MessagePrefix..EventDatabase_GetDBRevisionPath(pRequest.mUserName, pRequest.mDatabaseID, vCurrentRevision).."RFU");
		end

	elseif pRequest.mOpcode == "RAT_RFU" then
		local	vCurrentRevision;
		
		if vDatabase then
			vCurrentRevision = vDatabase.RSVPs.Revision;
		else
			vCurrentRevision = 0;
		end
		
		if vCurrentRevision < pRequest.mRevision
		or vCurrentRevision == 0 then
			CalendarNetwork_QueueOutboundMessage(gGroupCalendar_MessagePrefix..EventDatabase_GetRSVPRevisionPath(pRequest.mUserName, pRequest.mDatabaseID, vCurrentRevision).."RFU");
		end

	elseif pRequest.mOpcode == "INIT" then
		gGroupCalendar_RealmSettings = GroupCalendar_GetSettings(gGroupCalendar_PlayerName, GetRealmName());
		
		if gGroupCalendar_Settings.DebugInit then
			GroupCalendar_DebugMessage("GroupCalendar Initializing: Starting initialization");
		end
		
		CalendarNetwork_CheckPlayerGuild();
		
		-- Go ahead and do manual channel configuration now, automatic
		-- config will be handled once the player guild gets set and
		-- the roster gets loaded
		
		if not gGroupCalendar_RealmSettings.Channel.AutoConfig then
			if gGroupCalendar_Settings.DebugInit then
				GroupCalendar_DebugMessage("GroupCalendar INIT: Starting up data channel (manual configuration)");
			end
			
			if gGroupCalendar_RealmSettings.Channel.Name then
				CalendarNetwork_SetChannel(
						gGroupCalendar_RealmSettings.Channel.Name,
						gGroupCalendar_RealmSettings.Channel.Password);
			else
				GroupCalendar_DebugMessage("GroupCalendar channel is not set");
			end
		end

	elseif pRequest.mOpcode == "AUTOCONFIG" then
		CalendarNetwork_DoAutoConfig(pRequest.mCheckDatabaseTrust);
	
	elseif pRequest.mOpcode == "JOINCHANNEL2" then
		CalendarNetwork_JoinChannel2(pRequest.mChannelName, pRequest.mPassword);
	
	elseif pRequest.mOpcode == "DBTRUST" then
		EventDatabase_CheckDatabaseTrust();
	
	elseif pRequest.mOpcode == "NOTICES" then
		if gGroupCalendar_Settings.Debug then
			GroupCalendar_DebugMessage("Sending notices");
		end
		
		if gGroupCalendar_RealmSettings.Security.TrustAnyone then
			if gGroupCalendar_Settings.Debug then
				GroupCalendar_DebugMessage("Requesting updates from anyone");
			end
			CalendarNetwork_RequestAllUpdate();
		elseif gGroupCalendar_RealmSettings.Security.TrustGuildies
		and gGroupCalendar_PlayerGuild then
			if gGroupCalendar_Settings.Debug then
				GroupCalendar_DebugMessage("Requesting guildie updates");
			end
			CalendarNetwork_RequestGuildUpdate(gGroupCalendar_PlayerGuild, gGroupCalendar_RealmSettings.Security.MinTrustedRank);
		else
			if gGroupCalendar_Settings.Debug then
				GroupCalendar_DebugMessage("Not requesting group updates - no trusted groups");
			end
		end
		
		for vRealmName, vDatabase in gGroupCalendar_Database.Databases do
			if vDatabase.Realm == gGroupCalendar_RealmName then
				if vDatabase.IsPlayerOwned then
					if not CalendarChanges_IsEmpty(vDatabase.Changes) then
						CalendarNetwork_DBRevisionChanged(vDatabase);
					end
					
					if vDatabase.RSVPs
					and not CalendarChanges_IsEmpty(vDatabase.RSVPs) then
						CalendarNetwork_RSVPRevisionChanged(vDatabase);
					end
				elseif not gGroupCalendar_RealmSettings.Security.TrustAnyone
				and (not gGroupCalendar_RealmSettings.Security.TrustGuildies
				or vDatabase.Guild ~= gGroupCalendar_PlayerGuild) then
					CalendarNetwork_RequestUpdate(vDatabase);
				end
			end
		end
		
		-- Request databases for trusted players who haven't sent one yet
		
		if not gGroupCalendar_RealmSettings.Security.TrustAnyone then
			CalendarNetwork_RequestMissingDatabases();
		end
	end
end

gCalendarNetwork_GuildMemberRankCache = {};

function CalendarNetwork_BuildGuildRosterCache()
	-- Clear the cache
	
	gCalendarNetwork_GuildMemberRankCache = {};
	
	-- Scan the roster and collect the info
	
	local		vNumGuildMembers = GetNumGuildMembers(true);
	
	for vIndex = 1, vNumGuildMembers do
		local	vName, vRank, vRankIndex = GetGuildRosterInfo(vIndex);
		
		gCalendarNetwork_GuildMemberRankCache[vName] = vRankIndex;
	end
	
	-- Dump any cached trust info
	
	gCalendarNetwork_UserTrustCache = {};
end

local	gGroupCalendar_SentLoadGuildRoster = false;

function CalendarNetwork_LoadGuildRoster()
	if not IsInGuild()
	or GetNumGuildMembers() > 0
	or gGroupCalendar_SentLoadGuildRoster then
		return;
	end
	
	gGroupCalendar_SentLoadGuildRoster = true;
	
	if gGroupCalendar_Settings.DebugInit then
		GroupCalendar_DebugMessage("CalendarNetwork_LoadGuildRoster: Loading");
	end
	
	GuildRoster();
end

function CalendarNetwork_UserIsInSameGuild(pUserName)
	if not IsInGuild() then
		return false, nil;
	end
	
	-- Build the roster
	
	if GetNumGuildMembers() == 0 then
		CalendarNetwork_LoadGuildRoster();
		return false, nil; -- have to return false for now since we don't really know
	end
	
	-- Search for the member
	
	local		vUpperUserName = strupper(pUserName);
	local		vNumGuildMembers = GetNumGuildMembers(true);
	
	for vIndex = 1, vNumGuildMembers do
		local	vName, vRank, vRankIndex = GetGuildRosterInfo(vIndex);
		
		if strupper(vName) == vUpperUserName then
			return true, vRankIndex;
		end
	end
	
	return false, nil;
end

function CalendarNetwork_SendAllRevisionNotices()
	for vRealmName, vDatabase in gGroupCalendar_Database.Databases do
		if vDatabase.Realm == gGroupCalendar_RealmName then
			if not CalendarChanges_IsEmpty(vDatabase.Changes) then
				local		vRequest = {mOpcode = "DB_NOU", mUserName = vDatabase.UserName};
				
				if vDatabase.IsPlayerOwned then
					CalendarNetwork_QueueUniqueUserRequest(vRequest, math.random() * gCalendarNetwork_RequestDelay.OwnedNOURange);
				else
					CalendarNetwork_QueueUniqueUserRequest(vRequest, gCalendarNetwork_RequestDelay.ProxyNOUMin + math.random() * gCalendarNetwork_RequestDelay.ProxyNOURange);
				end
			end
			
			if not CalendarChanges_IsEmpty(vDatabase.RSVPs) then
				local		vRequest = {mOpcode = "RAT_NOU", mUserName = vDatabase.UserName};
				
				if vDatabase.IsPlayerOwned then
					CalendarNetwork_QueueUniqueUserRequest(vRequest, math.random() * gCalendarNetwork_RequestDelay.OwnedNOURange);
				else
					CalendarNetwork_QueueUniqueUserRequest(vRequest, gCalendarNetwork_RequestDelay.ProxyNOUMin + math.random() * gCalendarNetwork_RequestDelay.ProxyNOURange);
				end
			end
		end
	end
end

function CalendarNetwork_CancelRedundantUPDRequests(pChanges, pDatabaseTag, pUserName, pDatabaseID, pRevision, pSinceRevision)
	local	vOpcode = pDatabaseTag.."_UPD";
	
	for vIndex, vRequest in gGroupCalendar_Queue.Requests do
		if vRequest.mOpcode == vOpcode
		and vRequest.mUserName == pUserName then
			-- Theirs is better if:
			--   - Their database ID is higher
			--   - Their database ID is the same and
			--         - Their fromRevision is lower or the same
			--         - Their toRevision is higher or the same
			
			if pDatabaseID > vRequest.mDatabaseID
			or (pDatabaseID == vRequest.mDatabaseID
			and pSinceRevision <= vRequest.mRevision
			and pRevision  >= pChanges.Revision) then
				if gGroupCalendar_Settings.Debug then
					GroupCalendar_DebugMessage("Removing UPD request for "..pDatabaseTag.." "..pUserName..","..pDatabaseID..","..pSinceRevision.." to "..pRevision);
				end
				
				table.remove(gGroupCalendar_Queue.Requests, vIndex);
			else
				if gGroupCalendar_Settings.Debug then
					GroupCalendar_DebugMessage("Keeping UPD request for "..pDatabaseTag.." "..pUserName..","..vRequest.mDatabaseID..","..vRequest.mRevision.." to "..pChanges.Revision);
					GroupCalendar_DebugMessage("Better thean "..pDatabaseTag.." "..pUserName..","..pDatabaseID..","..pSinceRevision.." to "..pRevision);
				end
			end
			
			return;
		end
	end
end

function CalendarNetwork_CancelRedundantNOURequests(pChanges, pDatabaseTag, pSender, pUserName, pDatabaseID, pRevision)
	local	vOpcode = pDatabaseTag.."_NOU";
	
	for vIndex, vRequest in gGroupCalendar_Queue.Requests do
		if vRequest.mOpcode == vOpcode
		and vRequest.mUserName == pUserName then
			-- Cancel our update if the revision they're advertising is equal
			-- to or better than ours or if they're the owner
			
			local		vTheirsIsBetter;
			
			if pSender == pUserName then
				vTheirsIsBetter = true;
			else
				vTheirsIsBetter = pDatabaseID > pChanges.ID
						            or (pDatabaseID == pChanges.ID and pRevision >= pChanges.Revision);
			end
			
			if vTheirsIsBetter then
				if gGroupCalendar_Settings.Debug then
					GroupCalendar_DebugMessage("Removing NOU request for "..pDatabaseTag.." "..pUserName..","..pDatabaseID..","..pRevision);
				end
				
				table.remove(gGroupCalendar_Queue.Requests, vIndex);
			else
				if gGroupCalendar_Settings.Debug then
					GroupCalendar_DebugMessage("Keeping NOU request for "..pDatabaseTag.." "..pUserName..","..pChanges.ID..","..pChanges.Revision);
					GroupCalendar_DebugMessage("Better than "..pDatabaseTag.." "..pUserName..","..pDatabaseID..","..pRevision);
				end
			end
			
			return;
		end
	end
end

function CalendarNetwork_SendMessage(pMessage)
	-- Just leave if there's no channel to communicate on
	
	if not gGroupCalendar_Channel.ID then
		return;
	end
	
	local	vSavedAutoClearAFK = GetCVar("autoClearAFK");
	SetCVar("autoClearAFK", 0);

	SendChatMessage(pMessage, "CHANNEL", nil, gGroupCalendar_Channel.ID);
	
	SetCVar("autoClearAFK", vSavedAutoClearAFK);
end

function CalendarNetwork_ChannelMessageReceived(pSender, pMessage)
	local	vTrustLevel = CalendarTrust_GetUserTrustLevel(pSender);
	
	if vTrustLevel  > 0 then
		CalendarNetwork_QueueInboundMessage(pSender, vTrustLevel, pMessage);
	else
		if gGroupCalendar_Settings.Debug then
			GroupCalendar_DebugMessage("ChannelMessageReceived: "..pSender.." is not trusted");
		end
	end
end

function CalendarNetwork_GetGuildMemberIndex(pPlayerName)
	local		vUpperUserName = strupper(pPlayerName);
	local		vNumGuildMembers = GetNumGuildMembers(true);
	
	for vIndex = 1, vNumGuildMembers do
		local	vName = GetGuildRosterInfo(vIndex);
		
		if strupper(vName) == vUpperUserName then
			return vIndex;
		end
	end
	
	return nil;
end

function CalendarNetwork_SetAutoConfigData(pPlayerName)
	if not CanEditPublicNote()
	or not gGroupCalendar_RealmSettings.Channel.Name then
		return false;
	end
	
	local		vMemberIndex = CalendarNetwork_GetGuildMemberIndex(pPlayerName);
	
	if not vMemberIndex then
		return false;
	end
	
	CalendarNetwork_RemoveAllAutoConfigData(pPlayerName);
	
	local		vNote = "["..gGroupCalendar_MessagePrefix.."C:"..gGroupCalendar_RealmSettings.Channel.Name;
	
	if gGroupCalendar_RealmSettings.Channel.Password then
		vNote = vNote..","..gGroupCalendar_RealmSettings.Channel.Password
	end
	
	-- Add the trust group
	
	local		vTrustGroup = 0;
	
	if gGroupCalendar_RealmSettings.Security.TrustGuildies then
		vTrustGroup = 1;
	elseif gGroupCalendar_RealmSettings.Security.TrustAnyone then
		vTrustGroup = 2;
	end
	
	vNote = vNote.."/T:"..vTrustGroup;
	
	-- Add the trust rank
	
	if gGroupCalendar_RealmSettings.Security.TrustGuildies
	and gGroupCalendar_RealmSettings.Security.MinTrustedRank then
		vNote = vNote..","..gGroupCalendar_RealmSettings.Security.MinTrustedRank;
	end
	
	vNote = vNote.."]";
	
	if gGroupCalendar_Settings.Debug then
		GroupCalendar_DebugMessage("Setting auto-config data on player "..pPlayerName);
	end
	
	GuildRosterSetPublicNote(vMemberIndex, vNote);
	
	return true;
end

function CalendarNetwork_RemoveAllAutoConfigData(pExcludePlayerName)
	local	vIndex = CalendarNetwork_FindAutoConfigData(nil, pExcludePlayerName);
	
	while vIndex do
		CalendarNetwork_RemoveAutoConfigData(vIndex);
		vIndex = CalendarNetwork_FindAutoConfigData(vIndex + 1, pExcludePlayerName);
	end
end

function CalendarNetwork_RemoveAutoConfigData(pMemberIndex)
	if not pMemberIndex then
		GroupCalendar_DebugMessage("CalendarNetwork_RemoveAutoConfigData: nil index");
		return;
	end
	
	local	vName, vNote = CalendarNetwork_GetGuildPublicNote(pMemberIndex);
	
	if not vNote then
		return false;
	end
	
	if gGroupCalendar_Settings.Debug then
		GroupCalendar_DebugMessage("Removing auto-config data from player "..vName);
	end
	
	local	vPatternString = "%["..gGroupCalendar_MessagePrefix0.."[^%]]+%]";
	
	vNote = string.gsub(vNote, vPatternString, "");
	
	GuildRosterSetPublicNote(pMemberIndex, vNote);
	
	return true;
end

function CalendarNetwork_GetGuildPublicNote(pMemberIndex)
	local	vName, vRank, vRankIndex, vLevel, vClass, vZone, vGroup, vNote, vOfficerNote, vOnline = GetGuildRosterInfo(pMemberIndex);
	
	return vName, vNote;
end

function CalendarNetwork_FindAutoConfigData(pStartingIndex, pExcludePlayerName)
	-- Build the roster
	
	if GetNumGuildMembers() == 0 then
		CalendarNetwork_LoadGuildRoster();
		return nil;
	end
	
	-- Search for the member
	
	local		vNumGuildMembers = GetNumGuildMembers(true);
	local		vStartingIndex;
	
	if pStartingIndex then
		vStartingIndex = pStartingIndex;
	else
		vStartingIndex = 1;
	end
	
	for vIndex = vStartingIndex, vNumGuildMembers do
		local	vName, vNote = CalendarNetwork_GetGuildPublicNote(vIndex);
		
		if vNote
		and (not pExcludePlayerName or string.lower(vName) ~= string.lower(pExcludePlayerName))
		and strfind(vNote, "%["..gGroupCalendar_MessagePrefix0) then
			if gGroupCalendar_Settings.Debug then
				GroupCalendar_DebugMessage("Found auto-config data on player "..vName);
			end
			
			return vIndex, vName;
		end
	end
	
	return nil;
end

function CalendarNetwork_ScheduleCheckDatabaseTrust(pDelay)
	CalendarNetwork_QueueUniqueOpcodeRequest({mOpcode = "DBTRUST"}, pDelay);
end

function CalendarNetwork_ScheduleAutoConfig(pDelay, pCheckDatabaseTrust)
	if gGroupCalendar_Channel.Disconnected then
		return;
	end
	
	local	vRequest = CalendarNetwork_FindRequest({mOpcode = "AUTOCONFIG"});
	
	if vRequest then
		if gGroupCalendar_Settings.DebugInit then
			GroupCalendar_DebugMessage("CalendarNetwork_ScheduleAutoConfig: Rescheduling existing request");
		end
		
		if pDelay < vRequest.mDelay then
			vRequest.mDelay = pDelay;
		end
		
		if pCheckDatabaseTrust then
			vRequest.mCheckDatabaseTrust = true;
		end
	else
		if gGroupCalendar_Settings.DebugInit then
			GroupCalendar_DebugMessage("CalendarNetwork_ScheduleAutoConfig: Scheduling request");
		end
		
		CalendarNetwork_QueueRequest({mOpcode = "AUTOCONFIG", mCheckDatabaseTrust = pCheckDatabaseTrust}, pDelay);
	end
end

function CalendarNetwork_DoAutoConfig(pCheckDatabaseTrust)
	if gGroupCalendar_Settings.DebugInit then
		GroupCalendar_DebugMessage("CalendarNetwork_DoAutoConfig: Performing auto-config");
	end
	
	local	vIndex = CalendarNetwork_FindAutoConfigData();
	
	if not vIndex then
		return false;
	end
	
	local	vName, vNote = CalendarNetwork_GetGuildPublicNote(vIndex);
	
	if not vNote then
		return false;
	end
	
	for vConfigString in string.gfind(vNote, "%[([^%]]+)%]") do
		local		vCommand = CalendarNetwork_ParseCommandString(vConfigString);
		
		local		vChannel = nil;
		local		vPassword = nil;
		local		vMinTrustedRank = nil;
		local		vTrustGroup = nil;
		
		while vCommand[1] ~= nil do
			local	vOpcode = vCommand[1].opcode;
			local	vOperands = vCommand[1].operands;
			
			table.remove(vCommand, 1);
			
			if vOpcode == "CHN"
			or vOpcode == "C" then
				vChannel = vOperands[1];
				vPassword = vOperands[2];
				
			elseif vOpcode == "RNK" then
				vMinTrustedRank = tonumber(vOperands[1]);
			elseif vOpcode == "T" then
				vTrustGroup = tonumber(vOperands[1]);
				vMinTrustedRank = tonumber(vOperands[2]);
			end
		end
		
		local	vAutoPlayerChanged = gGroupCalendar_Channel.AutoPlayer ~= vName;
		
		gGroupCalendar_Channel.AutoPlayer = vName;
		
		if not CalendarNetwork_SetChannel(vChannel, vPassword)
		and vAutoPlayerChanged then
			GroupCalendar_ChannelChanged(); -- Send out a change notice just so the calendar knows the player changed
		end
		
		-- Update the trust settings
		
		local	vTrustChanged = false;
		local	vCurrentTrustGroup;
		
		if gGroupCalendar_RealmSettings.Security.TrustAnyone then
			vCurrentTrustGroup = 2;
		elseif gGroupCalendar_RealmSettings.Security.TrustGuildies then
			vCurrentTrustGroup = 1;
		else
			vCurrentTrustGroup = 0;
		end
		
		if vTrustGroup ~= nil
		and vCurrentTrustGroup ~= vTrustGroup then
			if vTrustGroup == 2 then
				gGroupCalendar_RealmSettings.Security.TrustAnyone = true;
				gGroupCalendar_RealmSettings.Security.TrustGuildies = false;
			elseif vTrustGroup == 1 then
				gGroupCalendar_RealmSettings.Security.TrustAnyone = false;
				gGroupCalendar_RealmSettings.Security.TrustGuildies = true;
			else
				gGroupCalendar_RealmSettings.Security.TrustAnyone = false;
				gGroupCalendar_RealmSettings.Security.TrustGuildies = false;
			end
			
			vTrustChanged = true;
		end
		
		if vMinTrustedRank ~= nil
		and gGroupCalendar_RealmSettings.Security.MinTrustedRank ~= vMinTrustedRank then
			gGroupCalendar_RealmSettings.Security.MinTrustedRank =  vMinTrustedRank;
			vTrustChanged = true;
		end
		
		if vTrustChanged then
			CalendarTrust_TrustSettingsChanged();
		elseif pCheckDatabaseTrust then
			EventDatabase_CheckDatabaseTrust();
		end
		
		return true;
	end
	
	return false;
end

function CalendarTrust_TrustSettingsChanged()
	gCalendarNetwork_UserTrustCache = {}; -- Flush the trust cache
	
	EventDatabase_CheckDatabaseTrust(); -- Delete databases owned by players we no longer trust
	CalendarNetwork_RequestMissingDatabases(); -- Request databases for newly trusted players
	
	if gGroupCalendar_RealmSettings.Security.TrustAnyone then
		CalendarNetwork_RequestAllUpdate();
	else
		if gGroupCalendar_RealmSettings.Security.TrustGuildies
		and gGroupCalendar_PlayerGuild then
			CalendarNetwork_RequestGuildUpdate(gGroupCalendar_PlayerGuild, gGroupCalendar_RealmSettings.Security.MinTrustedRank);
		end
	end
	
	CalendarNetwork_SendAllRevisionNotices(); -- Send out revision notices since trusted players may want to know now
end

function CalendarTrust_GetUserTrustLevel(pUserName)
	local	vUserTrustInfo = gCalendarNetwork_UserTrustCache[pUserName];
	
	if not vUserTrustInfo then
		vUserTrustInfo = {};
		vUserTrustInfo.mTrustLevel = CalendarTrust_CalcUserTrust(pUserName);
		gCalendarNetwork_UserTrustCache[pUserName] = vUserTrustInfo;
	end
	
	return vUserTrustInfo.mTrustLevel;
end

function CalendarTrust_UserIsTrusted(pUserName)
	return CalendarTrust_GetUserTrustLevel(pUserName) == 2;
end

function CalendarTrust_UserIsTrustedForRSVPs(pUserName)
	return CalendarTrust_GetUserTrustLevel(pUserName) >= 1;
end

function CalendarTrust_CalcUserTrust(pUserName)
	-- If the user is one of our own characters, then trust them completely
	
	local	vDatabase = EventDatabase_GetDatabase(pUserName, false);
	
	if vDatabase
	and vDatabase.IsPlayerOwned then
		return 2;
	end

	local	vPlayerSecurity = gGroupCalendar_RealmSettings.Security.Player[pUserName];
	
	-- See if they're explicity allowed/forbidden
	
	if vPlayerSecurity ~= nil then
		if vPlayerSecurity == 1 then
			-- Trusted
			
			return 2;
		elseif vPlayerSecurity == 2 then
			-- Excluded
			
			return 0;
		else
			GroupCalendar_DebugMessage("GroupCalendar: Unknown player security setting of "..vPlayerSecurity.." for "..pUserName);
		end
	end
	
	-- Return true if we'll allow anyone in the channel
	
	if gGroupCalendar_RealmSettings.Security.TrustAnyone then
		return 2;
	end
	
	-- Return true if they're in the same guild and of sufficient rank
	
	if gGroupCalendar_RealmSettings.Security.TrustGuildies then
		local	vIsInGuild, vGuildRank = CalendarNetwork_UserIsInSameGuild(pUserName);
		
		if vIsInGuild then
			if vGuildRank <= gGroupCalendar_RealmSettings.Security.MinTrustedRank then
				return 2;
			else
				return 1;
			end
		end
	end
	
	-- Failed all tests
		
	return 0;
end

function CalendarTrust_GetNumTrustedPlayers(pTrustSetting)
	local	vNumPlayers = 0;
	
	for vPlayerName, vPlayerSecurity in gGroupCalendar_RealmSettings.Security.Player do
		if vPlayerSecurity == pTrustSetting then
			vNumPlayers = vNumPlayers + 1;
		end
	end
	
	return vNumPlayers;
end

function CalendarTrust_GetIndexedTrustedPlayers(pTrustSetting, pIndex)
	local	vPlayerIndex = 1;
	
	for vPlayerName, vPlayerSecurity in gGroupCalendar_RealmSettings.Security.Player do
		if vPlayerSecurity == pTrustSetting then
			if vPlayerIndex == pIndex then
				return vPlayerName;
			end
			
			vPlayerIndex = vPlayerIndex + 1;
		end
	end
	
	return nil;
end

function CalendarNetwork_RequestMissingDatabases()
	-- For each player we explicitly trust, see if there's a database for
	-- them yet.  If not, request one
	
	for vPlayerName, vPlayerSecurity in gGroupCalendar_RealmSettings.Security.Player do
		if vPlayerSecurity == 1 then
			-- Found a trusted player, see if they have a database
			
			local	vDatabase = EventDatabase_GetDatabase(vPlayerName, false);
			
			if not vDatabase then
				CalendarNetwork_QueueRFURequest(vPlayerName, "DB", 0, 0);
				CalendarNetwork_QueueRFURequest(vPlayerName, "RAT", 0, 0);
			end
		end
	end
	
	return nil;
end

function CalendarNetwork_QueueRFURequest(pUserName, pDatabaseTag, pDatabaseID, pRevision)
	if CalendarNetwork_CancelRedundantRFURequest(pDatabaseTag, pUserName, pDatabaseID, pRevision) then
		return;
	end
	
	local	vRequest =
	{
		mOpcode = pDatabaseTag.."_RFU",
		mUserName = pUserName,
		mDatabaseID = pDatabaseID,
		mRevision = pRevision,
	}
	
	CalendarNetwork_QueueRequest(vRequest, gCalendarNetwork_RequestDelay.RFUMin + math.random() * gCalendarNetwork_RequestDelay.RFURange);
end

function CalendarNetwork_PlayerGuildChanged()
	gGroupCalendar_SentLoadGuildRoster = false; -- Clear the roster load flag
	
	gCalendarNetwork_UserTrustCache = {}; -- Flush the trust cache
	gCalendarNetwork_GuildMemberRankCache = {}; -- Flush the guild roster cache
	
	-- If the player is unguilded then simply leave the data
	-- channel if it was auto-configured, flush any databases
	-- which are no longer trusted and exit
	
	if not IsInGuild() then
		if gGroupCalendar_Settings.DebugInit then
			GroupCalendar_DebugMessage("PlayerGuildChanged: Player is now unguilded");
		end
		
		if gGroupCalendar_RealmSettings.Channel.AutoConfig then
			CalendarNetwork_LeaveChannel();
		end
		
		EventDatabase_CheckDatabaseTrust();
		
		return;
	end
	
	-- The player is in a new guild or has changed guilds, so
	-- schedule a roster update if necessary
	
	if GetNumGuildMembers() == 0 then
		if gGroupCalendar_Settings.DebugInit then
			GroupCalendar_DebugMessage("PlayerGuildChanged: Loading new roster");
		end
		
		CalendarNetwork_LoadGuildRoster();
	
	-- The roster is already loaded so just call GuildRosterChanged()
	-- to perform any roster-related tasks
	
	else
		if gGroupCalendar_Settings.DebugInit then
			GroupCalendar_DebugMessage("PlayerGuildChanged: Roster is already loaded, calling GuildRosterChanged()");
		end
		
		CalendarNetwork_GuildRosterChanged();
	end
end

function CalendarNetwork_GuildRosterChanged()
	if gGroupCalendar_Settings.DebugInit then
		GroupCalendar_DebugMessage("CalendarNetwork_GuildRosterChanged: Checking changes");
	end
	
	CalendarNetwork_BuildGuildRosterCache();

	if gGroupCalendar_RealmSettings.Channel.AutoConfig then
		CalendarNetwork_ScheduleAutoConfig(gCalendarNetwork_RequestDelay.GuildUpdateAutoConfig, true);
	else
		if gGroupCalendar_Settings.DebugInit then
			GroupCalendar_DebugMessage("PlayerGuildChanged: Channel is set for manual config, verifying database trust");
		end
		
		EventDatabase_CheckDatabaseTrust();
	end
end

function CalendarNetwork_CheckPlayerGuild()
	local	vPlayerGuild;

	if IsInGuild() then
		vPlayerGuild, _, gGroupCalendar_PlayerGuildRank = GetGuildInfo("player");
	else
		vPlayerGuild = nil;
		gGroupCalendar_PlayerGuildRank = nil;
	end

	if gGroupCalendar_PlayerGuild ~= vPlayerGuild then
		gGroupCalendar_PlayerGuild = vPlayerGuild;
		
		CalendarNetwork_PlayerGuildChanged();
	end
end
