gGroupCalendar_Database =
{
	Format = 5,
	Databases = {},
};

gGroupCalendar_UserDatabase = nil;

gGroupCalendar_Events =
{
	{id="Meet", name=GroupCalendar_cMeetingEventName},
	{id="Birth", name=GroupCalendar_cBirthdayEventName}
};

gGroupCalendar_Dungeons =
{
	{id="AQR", name=GroupCalendar_cAQREventName},
	{id="AQT", name=GroupCalendar_cAQTEventName},
	{id="BFD", name=GroupCalendar_cBFDEventName},
	{id="BRD", name=GroupCalendar_cBRDEventName},
	{id="UBRS", name=GroupCalendar_cUBRSEventName},
	{id="LBRS", name=GroupCalendar_cLBRSEventName},
	{id="BWL", name=GroupCalendar_cBWLEventName},
	{id="Deadmines", name=GroupCalendar_cDeadminesEventName},
	{id="DM", name=GroupCalendar_cDMEventName},
	{id="Gnomer", name=GroupCalendar_cGnomerEventName},
	{id="Mara", name=GroupCalendar_cMaraEventName},
	{id="MC", name=GroupCalendar_cMCEventName},
	{id="Onyxia", name=GroupCalendar_cOnyxiaEventName},
	{id="RFC", name=GroupCalendar_cRFCEventName},
	{id="RFD", name=GroupCalendar_cRFDEventName},
	{id="RFK", name=GroupCalendar_cRFKEventName},
	{id="SM", name=GroupCalendar_cSMEventName},
	{id="Scholo", name=GroupCalendar_cScholoEventName},
	{id="SFK", name=GroupCalendar_cSFKEventName},
	{id="Stockades", name=GroupCalendar_cStockadesEventName},
	{id="Strath", name=GroupCalendar_cStrathEventName},
	{id="ST", name=GroupCalendar_cSTEventName},
	{id="Uld", name=GroupCalendar_cUldEventName},
	{id="WC", name=GroupCalendar_cWCEventName},
	{id="ZF", name=GroupCalendar_cZFEventName},
	{id="ZG", name=GroupCalendar_cZGEventName},
};

gGroupCalendar_ClassInfoByClassCode =
{
	D = {name = GroupCalendar_cDruidClassName, color = GroupCalendar_cDruidClassColorName, element = "Druid"},
	H = {name = GroupCalendar_cHunterClassName, color = GroupCalendar_cHunterClassColorName, element = "Hunter"},
	M = {name = GroupCalendar_cMageClassName, color = GroupCalendar_cMageClassColorName, element = "Mage"},
	L = {name = GroupCalendar_cPaladinClassName, color = GroupCalendar_cPaladinClassColorName, element = "Paladin"},
	P = {name = GroupCalendar_cPriestClassName, color = GroupCalendar_cPriestClassColorName, element = "Priest"},
	R = {name = GroupCalendar_cRogueClassName, color = GroupCalendar_cRogueClassColorName, element = "Rogue"},
	S = {name = GroupCalendar_cShamanClassName, color = GroupCalendar_cShamanClassColorName, element = "Shaman"},
	K = {name = GroupCalendar_cWarlockClassName, color = GroupCalendar_cWarlockClassColorName, element = "Warlock"},
	W = {name = GroupCalendar_cWarriorClassName, color = GroupCalendar_cWarriorClassColorName, element = "Warrior"},
};

gGroupCalendar_RaceNamesByRaceCode =
{
	B = GroupCalendar_cBloodElfRaceName,
	A = GroupCalendar_cDraeneiRaceName,
	D = GroupCalendar_cDwarfRaceName,
	G = GroupCalendar_cGnomeRaceName,
	H = GroupCalendar_cHumanRaceName,
	N = GroupCalendar_cNightElfRaceName,
	O = GroupCalendar_cOrcRaceName,
	T = GroupCalendar_cTaurenRaceName,
	R = GroupCalendar_cTrollRaceName,
	U = GroupCalendar_cUndeadRaceName,
};
	
function EventDatabase_GetDatabase(pUserName, pCreate)
	if not pUserName then
		GroupCalendar_DebugMessage("EventDatabase_GetDatabase: pUserName is nil");
		return;
	end
	
	local	vDatabase = gGroupCalendar_Database.Databases[gGroupCalendar_RealmName.."_"..pUserName];
	
	if not vDatabase and pCreate then
		vDatabase = {};
		
		vDatabase.UserName = pUserName;
		vDatabase.IsPlayerOwned = pUserName == gGroupCalendar_PlayerName;
		vDatabase.CurrentEventID = 0;
		vDatabase.Realm = gGroupCalendar_RealmName;
		vDatabase.Events = {};
		vDatabase.Changes = CalendarChanges_New();
		vDatabase.RSVPs = CalendarChanges_New();
		
		gGroupCalendar_Database.Databases[gGroupCalendar_RealmName.."_"..pUserName] = vDatabase;
	end
	
	if vDatabase
	and not vDatabase.IsPlayerOwned
	and pUserName == gGroupCalendar_PlayerName then
		vDatabase.IsPlayerOwned = true;
	end
	
	return vDatabase;
end

function EventDatabase_DeleteDatabase(pUserName)
	if not gGroupCalendar_Database.Databases[gGroupCalendar_RealmName.."_"..pUserName] then
		return;
	end
	
	gGroupCalendar_Database.Databases[gGroupCalendar_RealmName.."_"..pUserName] = nil;
	
	GroupCalendar_MajorDatabaseChange(nil);
end

function EventDatabase_GetChanges(pDatabase)
	return pDatabase.Changes;
end

function EventDatabase_SetUserName(pUserName)
	gGroupCalendar_UserDatabase = EventDatabase_GetDatabase(gGroupCalendar_PlayerName, true);
end

function EventDatabase_NewEvent(pDatabase, pDate)
	local	vEvent = {};
	
	vEvent.mType = nil;
	vEvent.mTitle = nil;
	
	vEvent.mTime = 1140;
	vEvent.mDate = pDate;
	vEvent.mDuration = 120;
	
	vEvent.mDescription = nil;
	
	vEvent.mMinLevel = 0;
	vEvent.mAttendance = nil;
	
	pDatabase.CurrentEventID = pDatabase.CurrentEventID + 1;
	vEvent.mID = pDatabase.CurrentEventID;
	
	return vEvent;
end

function EventDatabase_AddEvent(pDatabase, pEvent)
	local	vSchedule = pDatabase.Events[pEvent.mDate];
	
	if vSchedule == nil then
		vSchedule = {};
		pDatabase.Events[pEvent.mDate] = vSchedule;
	end
	
	-- append the event
	
	table.insert(vSchedule, pEvent);
	
	EventDatabase_EventAdded(pDatabase, pEvent);
end

function EventDatabase_GetDateSchedule(pDate)
	return gGroupCalendar_UserDatabase.Events[pDate];
end

function EventDatabase_GetCompiledSchedule(pDate)
	local		vCompiledSchedule = {};
	
	for vRealmUser, vDatabase in gGroupCalendar_Database.Databases do
		if vDatabase.Realm == gGroupCalendar_RealmName then
			local	vSchedule = vDatabase.Events[pDate];
			
			if vSchedule then
				for vIndex, vEvent in vSchedule do
					local	vCompiledEvent = {mOwner = vDatabase.UserName, mEvent = vEvent};
					
					table.insert(vCompiledSchedule, vCompiledEvent);
				end
			end
		end
	end
	
	table.sort(vCompiledSchedule, EventDatabase_CompareCompiledEvents);
	
	return vCompiledSchedule;
end

function EventDatabase_CompareCompiledEvents(pCompiledEvent1, pCompiledEvent2)
	return EventDatabase_CompareEvents(pCompiledEvent1.mEvent, pCompiledEvent2.mEvent);
end

function EventDatabase_GetEventDisplayName(pEvent)
	if pEvent.mTitle and pEvent.mTitle ~= "" then
		return Calendar_UnescapeString(pEvent.mTitle);
	else
		local	vName = EventDatabase_GetEventNameByID(pEvent.mType);
		
		if vName ~= nil then
			return vName;
		else
			return "";
		end
	end
end

function EventDatabase_CompareEvents(pEvent1, pEvent2)
	if pEvent1.mTime == pEvent2.mTime then
		return EventDatabase_GetEventDisplayName(pEvent1) < EventDatabase_GetEventDisplayName(pEvent2);
	elseif pEvent1.mTime == nil then
		return true;
	elseif pEvent2.mTime == nil then
		return false;
	else
		return pEvent1.mTime < pEvent2.mTime;
	end
end

function EventDatabase_GetEventIndex(pSchedule, pEvent)
	for vIndex, vEvent in pSchedule do
		if vEvent == pEvent then
			return vIndex;
		end
	end
	
	return 0;
end

function EventDatabase_ScheduleIsEmpty(pSchedule)
	for vIndex, vEvent in pSchedule do
		return false;
	end
	
	return true;
end

function EventDatabase_FindEventByID(pDatabase, pEventID)
	for vDate, vSchedule in pDatabase.Events do
		for vEventIndex, vEvent in vSchedule do
			if vEvent.mID == pEventID then
				return vEvent;
			end
		end
	end
	
	return nil;
end

function EventDatabase_DeleteEvent(pDatabase, pEvent)
	-- Get the event's schedule
	
	local	vSchedule = pDatabase.Events[pEvent.mDate];
	
	if vSchedule == nil then
		return;
	end
	
	-- Find the event index
	
	local	vEventIndex = EventDatabase_GetEventIndex(vSchedule, pEvent);
	
	if vEventIndex == 0 then
		return;
	end
	
	-- Notify that the event is being removed
	
	CalendarNetwork_RemovingEvent(pDatabase, pEvent);
	
	-- Remove the event
	
	table.remove(vSchedule, vEventIndex);
	
	if EventDatabase_ScheduleIsEmpty(vSchedule) then
		pDatabase.Events[pEvent.mDate] = nil;
		vSchedule = nil;
	end
	
	-- Notify that the schedule changed
	
	GroupCalendar_ScheduleChanged(pDatabase, pEvent.mDate, vSchedule);
end

function EventDatabase_GetEventNameByID(pID)
	for vIndex, vEventType in gGroupCalendar_Events do
		if vEventType.id == pID then
			return vEventType.name;
		end
	end
	
	for vIndex, vEventType in gGroupCalendar_Dungeons do
		if vEventType.id == pID then
			return vEventType.name;
		end
	end
	
	return nil;
end

function EventDatabase_EventAdded(pDatabase, pEvent)
	local	vSchedule = pDatabase.Events[pEvent.mDate];
	
	-- Notify the calendar
	
	GroupCalendar_ScheduleChanged(pDatabase, pEvent.mDate, vSchedule);
	
	-- Notify the network
	
	CalendarNetwork_NewEvent(pDatabase, pEvent);
end

function EventDatabase_EventChanged(pDatabase, pEvent, pChangedFields)
	-- Update pending RSVPs based on event contents
	
	if pChangedFields.mAttendance then
		if pChangedFields.mAttendance.op == "UPD" then
			for vAttendeeName, vRSVPString in pChangedFields.mAttendance.val do
				local	vDatabase = EventDatabase_GetDatabase(vAttendeeName, false);
				
				if vDatabase then
					-- Remove any older (or same) RSVP for this person
					
					local	vRSVP = EventDatabase_UnpackEventRSVP(pDatabase.UserName, vAttendeeName, pEvent.mID, vRSVPString);
					
					EventDatabase_RemoveOlderRSVP(vDatabase, vRSVP)
				end
			end
		else
			Calendar_DumpArray("EventDatabase_EventChanged: ", pChangedFields);
			GroupCalendar_DebugMessage("EventDatabase_EventChanged: Attendance op "..pChangedFields.mAttendance.op.." not recognized");
		end
	end
	
	-- Notify the calendar
	
	GroupCalendar_EventChanged(pDatabase, pEvent, pChangedFields);
	
	-- Notify the network
	
	CalendarNetwork_EventChanged(pDatabase, pEvent, pChangedFields);
end

function EventDatabase_GetCurrentChangeList(pDatabase)
	local	vChangeList, vRevisionChanged = CalendarChanges_GetCurrentChangeList(pDatabase.Changes);
	
	if vRevisionChanged and pDatabase.IsPlayerOwned then
		CalendarNetwork_DBRevisionChanged(pDatabase);
	end
	
	return vChangeList;
end

function EventDatabase_GetDBRevisionPath(pUserName, pDatabaseID, pRevision)
	return CalendarChanges_GetRevisionPath("DB", pUserName, pDatabaseID, pRevision);
end

function EventDatabase_GetRSVPRevisionPath(pUserName, pDatabaseID, pRevision)
	return CalendarChanges_GetRevisionPath("RAT", pUserName, pDatabaseID, pRevision);
end

function EventDatabase_GetEventPath(pEvent)
	return "EVT:"..pEvent.mID.."/";
end

function EventDatabase_GenerateEventChangeString(pOpcode, pEvent, pEventPath)
	local		vChange;
	
	-- Basic fields: type, date, time, duration, minLevel, maxLevel

	vChange = pEventPath..pOpcode..":";
	
	if pEvent.mType ~= nil then
		vChange = vChange..pEvent.mType..",";
	else
		vChange = vChange..",";
	end

	if pEvent.mDate ~= nil then
		vChange = vChange..pEvent.mDate..",";
	else
		vChange = vChange..",";
	end

	if pEvent.mTime ~= nil then
		vChange = vChange..pEvent.mTime..",";
	else
		vChange = vChange..",";
	end

	if pEvent.mDuration ~= nil then
		vChange = vChange..pEvent.mDuration..",";
	else
		vChange = vChange..",";
	end

	if pEvent.mMinLevel ~= nil then
		vChange = vChange..pEvent.mMinLevel..",";
	else
		vChange = vChange..",";
	end

	if pEvent.mMaxLevel ~= nil then
		vChange = vChange..pEvent.mMaxLevel;
	end
	
	return vChange;
end

function EventDatabase_AppendNewEvent(pChangeList, pEvent, pEventPath)
	-- Basic fields: type, date, time, duration, minLevel, maxLevel
	
	table.insert(pChangeList, EventDatabase_GenerateEventChangeString("NEW", pEvent, pEventPath));
	
	-- Title
	
	if pEvent.mTitle then
		table.insert(pChangeList, pEventPath.."TIT:"..pEvent.mTitle);
	end

	if pEvent.mDescription ~= nil then
		table.insert(pChangeList, pEventPath.."DSC:"..pEvent.mDescription);
	end

	table.insert(pChangeList, pEventPath.."END");
end

function EventDatabase_AppendEventUpdate(pChangeList, pEvent, pEventPath, pChangedFields)
	-- Basic fields: type, date, time, duration, minLevel, maxLevel

	table.insert(pChangeList, EventDatabase_GenerateEventChangeString("UPD", pEvent, pEventPath));
	
	-- Title
	
	if pChangedFields.mTitle ~= nil then
		if pEvent.mTitle then
			table.insert(pChangeList, pEventPath.."TIT:"..pEvent.mTitle);
		end
	end
	
	if pChangedFields.mDescription ~= nil then
		if pEvent.mDescription ~= nil then
			table.insert(pChangeList, pEventPath.."DSC:"..pEvent.mDescription);
		end
	end
	
	if pChangedFields.mAttendance ~= nil then
		if pChangedFields.mAttendance.op == "UPD" then
			for vAttendeeName, vEventRSVPString in pChangedFields.mAttendance.val do
				local	vAttendeeRSVPString = pEvent.mAttendance[vAttendeeName];
				local	vAttendeePath = pEventPath.."ATT:"..vAttendeeName;
				
				if not vAttendeeRSVPString then
					table.insert(pChangeList, vAttendeePath);
				else
					table.insert(pChangeList, vAttendeePath..","..vAttendeeRSVPString);
				end
			end
		else
			GroupCalendar_DebugMessage("EventDatabase_AppendEventUpdate: Unknown attendance opcode "..pChangedFields.mAttendance.op);
		end
	end
	
	table.insert(pChangeList, pEventPath.."END");
end

function EventDatabase_RemoveEventChanges(pDatabase, pEvent)
	-- Remove all prior occurances for this event
	
	for vRevision, vChangeList in pDatabase.Changes.ChangeList do
		local	vEventPath = EventDatabase_GetEventPath(pEvent);
		local	vPathLength = string.len(vEventPath);
		
		local	vNumChanges = table.getn(vChangeList);
		local	vChangeIndex = 1;
		
		while vChangeIndex <= vNumChanges do
			vChange = vChangeList[vChangeIndex];
			
			if vChange ~= nil
			and string.sub(vChange, 1, vPathLength) == vEventPath then
				table.remove(vChangeList, vIndex);
				vNumChanges = vNumChanges - 1;
			else
				vChangeIndex = vChangeIndex + 1;
			end
		end
		
		if vNumChanges == 0 then
			pDatabase.Changes.ChangeList[vRevision] = nil;
		end
	end
end

function EventDatabase_RebuildPlayerDatabases()
	for vRealmUser, vDatabase in gGroupCalendar_Database.Databases do
		if vDatabase.IsPlayerOwned
		and vDatabase.Realm == gGroupCalendar_RealmName then
			EventDatabase_RebuildDatabase(vDatabase);
		end
	end
end

function EventDatabase_RebuildDatabase(pDatabase)
	-- Clear the revisions
	
	CalendarChanges_Rebuild(pDatabase.Changes);
	
	-- Start a new revision
	
	local	vChangeList = nil;
	
	-- Add each event to the revision
	
	for vDate, vSchedule in pDatabase.Events do
		for vEventIndex, vEvent in vSchedule do
			if not vChangeList then
				vChangeList = EventDatabase_GetCurrentChangeList(pDatabase);
			end
			
			EventDatabase_AppendNewEvent(
					vChangeList,
					vEvent,
					EventDatabase_GetEventPath(vEvent));
		end
	end
	
	if not vChangeList then
		CalendarNetwork_SendEmptyChanges(pDatabase.Changes, "DB", pDatabase.UserName)
	end
	
	-- Compact the RSVP list and notify that they're updated
	
	CalendarChanges_Compact(pDatabase.RSVPs);
	
	if not CalendarChanges_IsEmpty(pDatabase.RSVPs) then
		CalendarNetwork_RSVPRevisionChanged(pDatabase);
	else
		CalendarNetwork_SendEmptyChanges(pDatabase.RSVPs, "RAT", pDatabase.UserName)
	end
	
	-- Notify the calendar that there was a major change
	
	GroupCalendar_MajorDatabaseChange(pDatabase);
end

function EventDatabase_PurgeDatabase(pDatabase, pDatabaseID)
	pDatabase.CurrentEventID = 0;
	pDatabase.Events = {};
	
	pDatabase.Changes = CalendarChanges_New();
	pDatabase.Changes.ID = pDatabaseID;
	
	GroupCalendar_MajorDatabaseChange(pDatabase);
end

function EventDatabase_PurgeRSVPs(pDatabase, pDatabaseID)
	pDatabase.RSVPs = CalendarChanges_New();
	pDatabase.RSVPs.ID = pDatabaseID;
	
	GroupCalendar_MajorDatabaseChange(pDatabase);
end

function EventDatabase_CheckDatabase(pDatabase)
	-- Remove empty RSVP changelists
	
	for vRevision, vChangeList in pDatabase.RSVPs.ChangeList do
		if table.getn(vChangeList) == 0 then
			pDatabase.RSVPs.ChangeList[vRevision] = nil;
		end
	end
	
	-- Remove events with duplicate IDs
	
	for vDate, vEvents in pDatabase.Events do
		local	vEventIndex = 1;
		local	vNumEvents = table.getn(vEvents);
		
		while vEventIndex <= vNumEvents do
			local	vEvent = vEvents[vEventIndex];
			
			if not vEvent
			or EventDatabase_FindEventByID(pDatabase, vEvent.mID) ~= vEvent then
				GroupCalendar_DebugMessage("EventDatabase_CheckDatabase: Removing extra event ID "..vEvent.mID.." from database for "..pDatabase.UserName);
				
				table.remove(vEvents, vEventIndex);
				vNumEvents = vNumEvents - 1;
			else
				vEventIndex = vEventIndex + 1;
			end
		end
	end
end

function EventDatabase_ScanForNewlines(pDatabase)
	for vDate, vEvents in pDatabase.Events do
		for vEventID, vEvent in vEvents do
			if vEvent.mDescription then
				vEvent.mDescription = string.gsub(vEvent.mDescription, "\n", "&n;");
			end
		end
	end

	if pDatabase.Changes and pDatabase.Changes.ChangeList then
		for vRevision, vChanges in pDatabase.Changes.ChangeList do
			for vIndex, vChange in vChanges do
				if type(vIndex) == "number" then
					vChanges[vIndex] = string.gsub(vChange, "\n", "&n;");
				end
			end
		end
	end
end

function EventDatabase_CheckDatabases()
	-- Upgrade the database to format 4 (just purge all non-owned databases
	-- and rebuild the owned ones)
	
	if gGroupCalendar_Database.Format < 4 then
		for vRealmUser, vDatabase in gGroupCalendar_Database.Databases do
			if vDatabase.IsPlayerOwned then
				EventDatabase_RebuildDatabase(vDatabase);
			else
				gGroupCalendar_Database.Databases[vRealmUser] = nil;
			end
		end
		
		gGroupCalendar_Database.Format = 4;
	end
	
	-- Upgrade the database to format 5 (scan for newlines in event fields and escape them)
	
	if gGroupCalendar_Database.Format < 5 then
		for vRealmUser, vSettings in gGroupCalendar_Settings do
			if type(vSettings) == "array" and vSettings.EventTemplates then
				for vEventID, vEventTemplate in vSettings.EventTemplates do
					if vEventTemplate.mDescription then
						vEventTemplate.mDescription = string.gsub(vEventTemplate.mDescription, "\n", "&n;");
					end
				end
			end
		end
		
		for vRealmUser, vDatabase in gGroupCalendar_Database.Databases do
			EventDatabase_ScanForNewlines(vDatabase);
		end
		
		gGroupCalendar_Database.Format = 5;
	end

	--
	
	for vRealmUser, vDatabase in gGroupCalendar_Database.Databases do
		if not vDatabase.Realm then
			local	vStartIndex, vEndIndex, vRealmName = string.find(vRealmUser, "([^_]+)");
			
			if vStartIndex ~= nil then
				vDatabase.Realm = vRealmName;
			end
		end
		
		EventDatabase_CheckDatabase(vDatabase);
	end
end

function EventDatabase_CheckDatabaseTrust()
	-- Return if they're in a guild but the roster isn't loaded
	-- so that we don't go and delete a bunch of guildie calendars
	-- by mistake
	
	if IsInGuild() and GetNumGuildMembers() == 0 then
		if gGroupCalendar_Settings.DebugInit then
			GroupCalendar_DebugMessage("CheckDatabaseTrust: Roster isn't loaded, scheduling a load");
		end
		
		CalendarNetwork_LoadGuildRoster();
		return;
	end
	
	if gGroupCalendar_Settings.DebugInit then
		GroupCalendar_DebugMessage("CheckDatabaseTrust: Verifying trust");
	end
	
	-- Verify that each database is still trusted
	
	for vRealmUser, vDatabase in gGroupCalendar_Database.Databases do
		-- Only check databases for the current realm
		
		if vDatabase.Realm == gGroupCalendar_RealmName then
			
			-- See if they're still trusted
			
			if not vDatabase.IsPlayerOwned
			and not CalendarTrust_UserIsTrusted(vDatabase.UserName) then
				-- not trusted anymore, remove their database
				
				gGroupCalendar_Database.Databases[vRealmUser] = nil;
				
				GroupCalendar_MajorDatabaseChange(vDatabase);
			end
		end
	end
end

function EventDatabase_GetDateTime60Stamp()
	local	vYear, vMonth, vDay, vHour, vMinute, vSecond = Calendar_GetCurrentYearMonthDayHourMinute();
	
	local	vDate = Calendar_ConvertMDYToDate(vMonth, vDay, vYear);
	local	vTime60 = Calendar_ConvertHMSToTime60(vHour, vMinute, vSecond);
	
	return vDate, vTime60;
end

function EventDatabase_GetRSVPs(pDatabase)
	local	vRSVPs = pDatabase.RSVPs;
	
	if not vRSVPs then
		vRSVPs = {ID = 1, Revision = 0, ProxyRevision = 0, ChangeList = {}};
		pDatabase.RSVPs = vRSVPs;
	end
	
	return vRSVPs;
end

function EventDatabase_GetCurrentRSVPChangeList(pDatabase)
	local	vRSVPs = EventDatabase_GetRSVPs(pDatabase);
	local	vChangeList, vRevisionChanged = CalendarChanges_GetCurrentChangeList(vRSVPs);
	
	if vRevisionChanged and pDatabase.IsPlayerOwned then
		CalendarNetwork_RSVPRevisionChanged(pDatabase);
	end
	
	return vChangeList;
end

function EventDatabase_AddEventRSVP(pDatabase, pEvent, pAttendeeName, pRSVP)
	-- Verify that the attendance request is newer than the existing one

	local	vExistingRSVPString;

	if pEvent.mAttendance then
		vExistingRSVPString = pEvent.mAttendance[pAttendeeName];
	else
		vExistingRSVPString = nil;
	end

	if vExistingRSVP then
		local	vExistingRSVP = EventDatabase_UnpackEventRSVP(pAttendeeName, pEvent, vExistingRSVPString);
		
		if vExistingRSVP.mDate > pRSVP.mDate
		or (vExistingRSVP.mDate == pRSVP.mDate
		and vExistingRSVP.mTime > pRSVP.mTime) then
			if gGroupCalendar_Settings.Debug then
				GroupCalendar_DebugMessage("EventDatabase_AddRSVP: Ignore event attendance request for "..pAttendeeName.." for ".." event "..pRSVP.mEventID..": Request is out-of-date");
			end
			
			return;
		end
	end

	-- Update the event attendance list

	if gGroupCalendar_Settings.Debug then
		GroupCalendar_DebugMessage("EventDatabase_AddRSVP: Updating event attendance for "..pAttendeeName.." for ".." event "..pRSVP.mEventID);
	end

	if not pEvent.mAttendance then
		pEvent.mAttendance = {};
	end
	
	local	vEventRSVPString = EventDatabase_PackEventRSVP(pRSVP);
	
	pEvent.mAttendance[pAttendeeName] = vEventRSVPString;

	-- Notify the network of the change

	local	vChangedFields =
	{
		mAttendance =
		{
			op = "UPD",
			val =
			{
			}
		}
	};
	
	vChangedFields.mAttendance.val[pAttendeeName] = vEventRSVPString;

	EventDatabase_EventChanged(pDatabase, pEvent, vChangedFields);
end

function EventDatabase_RemoveEventRSVP(pDatabase, pEvent, pAttendeeName)
	if not pEvent.mAttendance then
		return;
	end
	
	pEvent.mAttendance[pAttendeeName] = nil;

	-- Notify the network of the change

	local	vChangedFields =
	{
		mAttendance =
		{
			op = "UPD",
			val =
			{
			}
		}
	};
	
	vChangedFields.mAttendance.val[pAttendeeName] = "-";

	EventDatabase_EventChanged(pDatabase, pEvent, vChangedFields);
end

function EventDatabase_AddRSVPRequest(pDatabase, pRSVP)
	local	vChangeList = EventDatabase_GetCurrentRSVPChangeList(pDatabase);

	-- Remove any existing RSVP for the same event
	
	if not EventDatabase_RemoveOlderRSVP(pDatabase, pRSVP) then
		if gGroupCalendar_Settings.Debug then
			GroupCalendar_DebugMessage("EventDatabase_AddRSVP: Ignoring "..pDatabase.UserName..","..pEventID..": "..pStatus.."/"..pComment);
		end
		
		return; -- A newer request already exists so disregard this one
	end

	-- Add the new RSVP

	local	vRSVPString = EventDatabase_PackRSVPRequest(pRSVP);

	if gGroupCalendar_Settings.Debug then
		GroupCalendar_DebugMessage("EventDatabase_AddRSVP: "..vRSVPString);
	end

	table.insert(vChangeList, vRSVPString);
end

function EventDatabase_AddRSVP(pDatabase, pRSVP)
	if gGroupCalendar_Settings.Debug then
		if not pRSVP.mAttendeeName then
			Calendar_DumpArray("Missing attendee: ", pRSVP);
			return;
		end
	end
	
	-- If the database is one that we own, then process the request instead of queueing it
	
	local	vEventDatabase = EventDatabase_GetDatabase(pRSVP.mOrganizerName, false);
	
	if vEventDatabase and vEventDatabase.IsPlayerOwned then
		local	vEvent = EventDatabase_FindEventByID(vEventDatabase, pRSVP.mEventID);
		
		if not vEvent then
			if gGroupCalendar_Settings.Debug then
				GroupCalendar_DebugMessage("EventDatabase_AddRSVP: Discarding request from "..pDatabase.UserName.." for ".." event "..pRSVP.mEventID..": Event no longer exists");
			end
			
			return;
		end
		
		EventDatabase_AddEventRSVP(vEventDatabase, vEvent, pDatabase.UserName, pRSVP);
	else
		EventDatabase_AddRSVPRequest(pDatabase, pRSVP);
	end
end

function EventDatabase_RemoveOlderRSVP(pDatabase, pRSVP)
	if gGroupCalendar_Settings.Debug then
		GroupCalendar_DebugMessage("RemoveOlderRSVP: Removing RSVP for "..pRSVP.mAttendeeName.." from "..pDatabase.UserName..","..pRSVP.mEventID);
	end
	
	if not pRSVP.mOrganizerName then
		Calendar_DumpArray("EventDatabase_RemoveOlderRSVP: ", pRSVP);
	end
	
	local	vRSVPString, vIndex, vRevision = EventDatabase_FindRSVPRequestString(pDatabase, pRSVP.mOrganizerName, pRSVP.mEventID);
	
	while vRSVPString ~= nil do
		if gGroupCalendar_Settings.Debug then
			GroupCalendar_DebugMessage("EventDatabase_RemoveRSVP: "..pRSVP.mOrganizerName..","..pRSVP.mEventID.." from position "..vRevision..","..vIndex);
		end
		
		-- If the existing RSVP is newer than the specified date/time then disregard the request
		
		if pRSVP.mDate ~= nil then
			local	vRSVP = EventDatabase_UnpackRSVPRequest(vRSVPString, pRSVP.mAttendeeName);
			
			if vRSVP.mDate > pRSVP.mDate
			or (vRSVP.mDate == pRSVP.mDate and vRSVP.mTime > pRSVP.mTime) then
				return false; -- Fail to indicate that a newer request is already in the database
			end
		end
		
		-- Remove the old one
		
		local	vChangeList = pDatabase.RSVPs.ChangeList[vRevision];
		table.remove(vChangeList, vIndex);
		
		if table.getn(vChangeList) == 0 then
			pDatabase.RSVPs.ChangeList[vRevision] = nil;
		end
		
		-- Keep removing in case the database has become corrupted with multiple copies
		
		vRSVPString, vIndex, vRevision = EventDatabase_FindRSVPRequestString(pDatabase, pRSVP.mOrganizerName, pRSVP.mEventID);
	end
	
	return true; -- Everything's ok, no newer RSVP was found
end

function EventDatabase_FindRSVPRequestString(pDatabase, pOrganizerName, pEventID)
	if not pEventID then
		GroupCalendar_DebugMessage("EventDatabase_FindRSVPRequestString: pEventID IS NIL!");
	end
	
	if not pOrganizerName then
		GroupCalendar_DebugMessage("EventDatabase_FindRSVPRequestString: pOrganizerName IS NIL!");
	end
	
	local	vRSVPs = EventDatabase_GetRSVPs(pDatabase);
	local	vRSVPPrefix = "EVT:"..pOrganizerName..","..pEventID..",";
	local	vRSVPPrefixLength = string.len(vRSVPPrefix);
	
	for vRevision, vChangeList in vRSVPs.ChangeList do
		local	vNumChanges = table.getn(vChangeList);
		
		for vIndex = 1, vNumChanges do
			local	vRSVP = vChangeList[vIndex];
			
			if string.sub(vRSVP, 1, vRSVPPrefixLength) == vRSVPPrefix then
				return vRSVP, vIndex, vRevision;
			end
		end
	end
	
	return nil, nil, nil;
end

function EventDatabase_FindRSVPRequestData(pDatabase, pOrganizerName, pEventID)
	if not pEventID then
		GroupCalendar_DebugMessage("EventDatabase_FindRSVPRequestData: pEventID is nil!");
	end
	
	if not pOrganizerName then
		GroupCalendar_DebugMessage("EventDatabase_FindRSVPRequestData: pOrganizerName IS NIL!");
	end
	
	local	vRSVPString = EventDatabase_FindRSVPRequestString(pDatabase, pOrganizerName, pEventID);
	
	if not vRSVPString then
		return nil;
	end
	
	return EventDatabase_UnpackRSVPRequest(vRSVPString, pDatabase.UserName);
end

function EventDatabase_FindEventRSVPString(pEvent, pAttendeeName)
	if not pEvent.mAttendance then
		return nil;
	end
	
	return pEvent.mAttendance[pAttendeeName];
end

function EventDatabase_FindEventRSVP(pEvent, pAttendeeName)
	local	vEventRSVPString = EventDatabase_FindEventRSVPString(pEvent, pAttendeeName);
	
	if not vEventRSVPString then
		return nil;
	end
	
	return EventDatabase_UnpackEventRSVP(gGroupCalendar_PlayerName, pAttendeeName, pEvent.ID, vEventRSVPString);
end

function EventDatabase_PackRSVPRequest(pRSVP)
	local	vRequest = "EVT:"..pRSVP.mOrganizerName..","..pRSVP.mEventID..","..pRSVP.mDate..","..pRSVP.mTime..","..pRSVP.mStatus..","..EventDatabase_PackCharInfo(pRSVP)..",";
	
	if pRSVP.mComment then
		vRequest = vRequest..pRSVP.mComment;
	end
	
	return vRequest;
end

function EventDatabase_UnpackRSVPRequest(pRSVPString, pAttendee)
	vCommands = CalendarNetwork_ParseCommandSubString("/"..pRSVPString);
	
	local	vOpcode = vCommands[1].opcode;
	
	if vOpcode ~= "EVT" then
		return false;
	end
	
	local	vOperands = vCommands[1].operands;
	
	return EventDatabase_UnpackRSVPFieldArray(vOperands, pAttendee);
end

function EventDatabase_UnpackRSVPFieldArray(pArray, pAttendee)
	local	vRSVP =
	{
		mOrganizerName = pArray[1],
		mAttendeeName = pAttendee;
		mEventID = tonumber(pArray[2]),
		mDate = tonumber(pArray[3]),
		mTime = tonumber(pArray[4]),
		mStatus = pArray[5],
		mComment = pArray[7],
	};
	
	EventDatabase_UnpackCharInfo(pArray[6], vRSVP);
	
	return vRSVP;
end

function EventDatabase_PackEventRSVP(pEventRSVP)
	local	vEventRSVPString = ""..pEventRSVP.mDate..","..pEventRSVP.mTime..","..pEventRSVP.mStatus..","..EventDatabase_PackCharInfo(pEventRSVP)..",";
	
	if pEventRSVP.mComment then
		vEventRSVPString = vEventRSVPString..pEventRSVP.mComment;
	end
	
	return vEventRSVPString;
end

function EventDatabase_UnpackEventRSVP(pOrganizerName, pAttendeeName, pEventID, pEventRSVPString)
	local	vEventParameters = CalendarNetwork_ParseParameterString(pEventRSVPString);
	
	local	vRSVPFields =
	{
		mOrganizerName = pOrganizerName,
		mAttendeeName = pAttendeeName,
		mEventID = pEventID,
		mDate = tonumber(vEventParameters[1]),
		mTime = tonumber(vEventParameters[2]),
		mStatus = vEventParameters[3],
		mComment = vEventParameters[5],
	};
	
	EventDatabase_UnpackCharInfo(vEventParameters[4], vRSVPFields);
	
	return vRSVPFields;
end

function EventDatabase_PackCharInfo(pCharInfo)
	local	vRaceCode, vClassCode, vLevel;
	
	vRaceCode = pCharInfo.mRaceCode;
	
	if not vRaceCode then
		vRaceCode = "?";
	end
	
	vClassCode = pCharInfo.mClassCode;
	
	if not vClassCode then
		vClassCode = "?";
	end
	
	if pCharInfo.mLevel then
		vLevel = pCharInfo.mLevel;
	else
		vLevel = 0;
	end
	
	return vRaceCode..vClassCode..vLevel;
end

function EventDatabase_GetRaceCodeByRace(pRace)
	for vRaceCode, vRaceName in gGroupCalendar_RaceNamesByRaceCode do
		if pRace == vRaceName then
			return vRaceCode;
		end
	end
	
	return "?";
end

function EventDatabase_GetRaceByRaceCode(pRaceCode)
	return gGroupCalendar_RaceNamesByRaceCode[pRaceCode];
end

function EventDatabase_GetClassCodeByClass(pClass)
	for vClassCode, vClassInfo in gGroupCalendar_ClassInfoByClassCode do
		if pClass == vClassInfo.name then
			return vClassCode;
		end
	end
	
	return "?";
end

function EventDatabase_GetClassByClassCode(pClassCode)
	local	vClassInfo = gGroupCalendar_ClassInfoByClassCode[pClassCode];
	
	if not vClassInfo then
		return "Unknown";
	else
		return vClassInfo.name;
	end
end

function EventDatabase_UnpackCharInfo(pString, rCharInfo)
	if not pString then
		rCharInfo.mRaceCode = "?";
		rCharInfo.mClassCode = "?";
		rCharInfo.mLevel = 0;
	else
		rCharInfo.mRaceCode = string.sub(pString, 1, 1);
		rCharInfo.mClassCode = string.sub(pString, 2, 2);
		rCharInfo.mLevel = tonumber(string.sub(pString, 3));
	end
end
	
function EventDatabase_IsQuestingEventType(pEventType)
	return pEventType ~= "Meet" and pEventType ~= "Birth";
end

function CalendarChanges_New()
	return
	{
		ID = 1,
		Revision = 0,
		ProxyRevision = 0,
		ChangeList = {},
	};
end

function CalendarChanges_Rebuild(pChanges)
	if not pChanges.ID then
		pChanges.ID = 1;
	else
		pChanges.ID = pChanges.ID + 1;
	end
	
	pChanges.Revision = 0;
	pChanges.ProxyRevision = 0;
	pChanges.ChangeList = {};
end

function CalendarChanges_Compact(pChanges)
	local	vNewChanges = nil;
	
	for vRevision, vChanges in pChanges.ChangeList do
		for vIndex, vChange in vChanges do
			if not vNewChanges then
				vNewChanges = {};
			end
			
			if type(vIndex) == "number" then
				table.insert(vNewChanges, vChange);
			end
		end
	end
	
	pChanges.ID = pChanges.ID + 1;
	pChanges.ChangeList = {};
	
	if vNewChanges then
		pChanges.Revision = 1;
		pChanges.ProxyRevision = 1;
		pChanges.ChangeList[1] = vNewChanges;
	else
		pChanges.Revision = 0;
		pChanges.ProxyRevision = 0;
	end
end

function CalendarChanges_Open(pChanges, pRevision)
	local	vChangeList = pChanges.ChangeList[pRevision];
	
	if not vChangeList then
		vChangeList = {};
		pChanges.ChangeList[pRevision] = vChangeList;
		pChanges.Revision = pRevision;
	end
	
	vChangeList.IsOpen = true;
	
	return vChangeList;
end

function CalendarChanges_Close(pChanges, pRevision)
	local	vChangeList = pChanges.ChangeList[pRevision];
	
	if not vChangeList then
		return;
	end
	
	vChangeList.IsOpen = nil;
end

function CalendarChanges_GetCurrentChangeList(pChanges)
	local	vChangeList = nil;
	local	vRevisionChanged = false;
	
	if pChanges.Revision > 0 then
		vChangeList = pChanges.ChangeList[pChanges.Revision];
	end
	
	if vChangeList == nil
	or not vChangeList.IsOpen then
		pChanges.Revision = pChanges.Revision + 1;
		
		vChangeList = {};
		pChanges.ChangeList[pChanges.Revision] = vChangeList;
		
		vChangeList.IsOpen = true;
		
		vRevisionChanged = true;
	end
	
	return vChangeList, vRevisionChanged;
end

function CalendarChanges_LockdownCurrentChangeList(pChanges)
	-- Just return if there are no changes yet
	
	if pChanges.Revision == 0 then
		return;
	end
	
	-- See if the current list exists and is open
	
	local		vChangeList = pChanges.ChangeList[pChanges.Revision];
	
	if not vChangeList
	or not vChangeList.IsOpen then
		return;
	end
	
	-- Close the change list
	
	vChangeList.IsOpen = nil;
end

function CalendarChanges_GetRevisionPath(pLabel, pUserName, pDatabaseID, pRevision)
	if not pDatabaseID then
		return pLabel..":"..pUserName..",,"..pRevision.."/";
	else
		return pLabel..":"..pUserName..","..pDatabaseID..","..pRevision.."/";
	end
end

function CalendarChanges_IsEmpty(pChanges)
	if not pChanges
	or pChanges.Revision == 0 then
		return true;
	end
	
	for vRevision, vChanges in pChanges.ChangeList do
		return false;
	end
	
	return true;
end

function CalendarEvent_GetAttendanceCounts(pDatabase, pEvent)
	if not pEvent.mAttendance then
		return nil;
	end
	
	local	vSortByClass = EventDatabase_IsQuestingEventType(pEvent.mType);
	local	vAttendanceCounts = {NumCategories = 0, NumPlayers = 0, Categories = {}, SortedCategories = {}};
	
	for vAttendeeName, vRSVPString in pEvent.mAttendance do
		local	vRSVP = EventDatabase_UnpackEventRSVP(pDatabase.UserName, vAttendeeName, pEvent.mID, vRSVPString);
		local	vCountIndex;
		local	vStatus1 = string.sub(vRSVP.mStatus, 1, 1);
		local	vStatus2 = string.sub(vRSVP.mStatus, 2, 1);
		
		if vStatus1 == "N" then
			vCountIndex = "NO";
		elseif vSortByClass then
			vCountIndex = vRSVP.mClassCode;
			
			if not vCountIndex then
				vCountIndex = "?";
			end
		else
			vCountIndex = "YES";
		end
		
		local	vClassInfo = vAttendanceCounts.Categories[vCountIndex];
		
		if not vClassInfo then
			vClassInfo = {mCount = 1, mClassCode = vCountIndex, mAttendees = {}};
			vAttendanceCounts.Categories[vCountIndex] = vClassInfo;
			table.insert(vAttendanceCounts.SortedCategories, vCountIndex);
			
			vAttendanceCounts.NumCategories = vAttendanceCounts.NumCategories + 1;
		else
			vClassInfo.mCount = vClassInfo.mCount + 1;
		end
		
		vAttendanceCounts.NumPlayers = vAttendanceCounts.NumPlayers + 1;
		
		table.insert(vClassInfo.mAttendees, vRSVP);
	end
	
	-- Sort the categories
	
	table.sort(vAttendanceCounts.SortedCategories, EventDatabase_CompareClassCodes);
	
	-- Sort the attendance within each category
	
	for vCategory, vClassInfo in vAttendanceCounts.Categories do
--		if vSortByClass then
--			table.sort(vClassInfo.mAttendees, EventDatabase_CompareRSVPsByDate);
--		else
			table.sort(vClassInfo.mAttendees, EventDatabase_CompareRSVPsByName);
--		end
	end
	
	-- Done
	
	return vAttendanceCounts;
end

function EventDatabase_CompareRSVPsByDate(pRSVP1, pRSVP2)
	if pRSVP1.mDate < pRSVP2.mDate then
		return true;
	elseif pRSVP1.mDate > pRSVP2.mDate then
		return false;
	else
		return pRSVP1.mTime < pRSVP2.mTime;
	end
end

function EventDatabase_CompareRSVPsByName(pRSVP1, pRSVP2)
	return pRSVP1.mAttendeeName < pRSVP2.mAttendeeName;
end

function EventDatabase_CompareClassCodes(pClassCode1, pClassCode2)
	-- Not attending is always at the end of the list and
	-- pending is always at the top
	
	if pClassCode1 == "NO"
	or pClassCode2 == "PENDING" then
		return false;
	end
	
	if pClassCode2 == "NO"
	or pClassCode1 == "PENDING" then
		return true;
	end
	
	-- Is attending doesn't need sorted
	
	if pClassCode == "YES" then
		return true;
	end
	
	-- Sort class names alphabetically
	
	return EventDatabase_GetClassByClassCode(pClassCode1) < EventDatabase_GetClassByClassCode(pClassCode2);
end

function EventDatabase_CreateCurrentPlayerRSVP(pDatabase, pEvent, pStatus, pComment)
	local	vDate, vTime60 = EventDatabase_GetDateTime60Stamp();
	
	return
	{
		mAttendeeName = gGroupCalendar_PlayerName,
		mOrganizerName = pDatabase.UserName,
		mEventID = pEvent.mID,
		mDate = vDate,
		mTime = vTime60,
		mStatus = pStatus,
		mComment = pComment,
		mRaceCode = EventDatabase_GetRaceCodeByRace(UnitRace("PLAYER")),
		mClassCode = EventDatabase_GetClassCodeByClass(UnitClass("PLAYER")),
		mLevel = UnitLevel("PLAYER"),
	};
end

function EventDatabase_PlayerIsQualifiedForEvent(pEvent)
	if pEvent.mMinLevel and
	gGroupCalendar_PlayerLevel < pEvent.mMinLevel then
		return false;
	end
	
	if pEvent.mMaxLevel and
	gGroupCalendar_PlayerLevel > pEvent.mMaxLevel then
		return false;
	end
	
	return true;
end
