gCalendarEventEditor_ShowScheduleEditor = false;
gCalendarEventEditor_Active = false;

gCalendarEventEditor_Database = nil;
gCalendarEventEditor_Event = nil;
gCalendarEventEditor_IsNewEvent = false;

gCalendarEventEditor_PreviousEventType = nil;

gCalendarEventEditor_CurrentPanel = 1;

gGroupCalendar_cNumAttendanceItems = 12;
gGroupCalendar_cNumPlainAttendanceItems = 16;
gGroupCalendar_cAttendanceItemHeight = 16;

gCalendarEventEditor_PanelFrames =
{
	"CalendarEventEditorEventFrame",
	"CalendarEventEditorAttendanceFrame",
};

StaticPopupDialogs["CONFIRM_CALENDAR_EVENT_DELETE"] = {
	text = TEXT(CalendarEventEditor_cConfirmDeleteMsg),
	button1 = TEXT(CalendarEventEditor_cDelete),
	button2 = TEXT(CANCEL),
	OnAccept = function() CalendarEventEditor_DeleteEvent(); end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1
};


function CalendarEventEditor_EditEvent(pDatabase, pEvent, pIsNewEvent)
	gCalendarEventEditor_Database = pDatabase;
	gCalendarEventEditor_Event = pEvent;
	gCalendarEventEditor_IsNewEvent = pIsNewEvent;
	
	CalendarAttendanceList_SetEvent(CalendarEventEditorAttendance, pDatabase, pEvent);
	
	CalendarEventEditor_UpdateControlsFromEvent(pEvent);
	
	CalendarEventEditorFrameSubTitle:SetText(Calendar_GetLongDateString(pEvent.mDate));
	
	ShowUIPanel(CalendarEventEditorFrame);
	
	if pIsNewEvent then
		CalendarEventTitle:SetFocus();
		CalendarEventTitle:HighlightText();
	else
		CalendarEventTitle:HighlightText(0, 0);
	end
	
	gCalendarEventEditor_ShowScheduleEditor = false;
	gCalendarEventEditor_Active = true;
end

function CalendarEventEditor_EventChanged(pEvent)
	if pEvent == gCalendarEventEditor_Event then
		CalendarAttendanceList_EventChanged(CalendarEventEditorAttendance, gCalendarEventEditor_Database, pEvent);
		
		-- Don't update the control values since that'll overwrite whatever
		-- is currently being set up
		-- CalendarEventEditor_UpdateControlsFromEvent(pEvent);
	end
end

function CalendarEventEditor_OnLoad()
	-- Tabs
	
	PanelTemplates_SetNumTabs(this, table.getn(gCalendarEventEditor_PanelFrames));
	CalendarEventEditorFrame.selectedTab = gCalendarEventEditor_CurrentPanel;
	PanelTemplates_UpdateTabs(this);
end

function CalendarEventEditor_OnShow()
	PlaySound("igCharacterInfoOpen");
	
	CalendarEventTypeEventType.ChangedValueFunc = CalendarEventEditor_EventTypeChanged;
	
	CalendarEventEditor_ShowPanel(1); -- Always switch to the event view when showing the window
end

function CalendarEventEditor_OnHide()
	PlaySound("igCharacterInfoClose");
	
	if not gCalendarEventEditor_ShowScheduleEditor then
		CalendarEventEditor_SaveEvent();
		HideUIPanel(CalendarEditorFrame);
	end
	
	gCalendarEventEditor_Database = nil;
	gCalendarEventEditor_Event = nil;
	gCalendarEventEditor_Active = false;
end

function CalendarEventEditor_DoneEditing()
	if not gCalendarEventEditor_Active then
		return;
	end
	
	CalendarEventEditor_SaveEvent();
	CalendarEventEditor_CloseEditor(true);
end

function CalendarEventEditor_UpdateControlsFromEvent(pEvent, pExistingValuesOnly)
	if not pExistingValuesOnly
	or pEvent.mType then
		UIDropDownMenu_SetText("", CalendarEventTypeEventType); -- Set to empty in case the selected value isn't there
		CalendarDropDown_SetSelectedValue(CalendarEventTypeEventType, pEvent.mType);
	end

	if pEvent.mTitle and pEvent.mTitle ~= "" then
		CalendarEventTitle:SetText(Calendar_UnescapeString(pEvent.mTitle));
	elseif not pExistingValuesOnly
	or pEvent.mType then
		local	vEventName = EventDatabase_GetEventNameByID(pEvent.mType);
		
		if vEventName ~= nil then
			CalendarEventTitle:SetText(vEventName);
		else
			CalendarEventTitle:SetText("");
		end
	end
	
	if not pExistingValuesOnly
	or pEvent.mTime then
		CalendarEventEditor_SetTimeControlValue(CalendarEventEditorTime, pEvent.mTime);
	end
	
	if not pExistingValuesOnly
	or pEvent.mDuration then
		CalendarDropDown_SetSelectedValue(CalendarEventEditorDurationDuration, pEvent.mDuration);
	end

	if pEvent.mDescription then
		CalendarEventDescription:SetText(Calendar_UnescapeString(pEvent.mDescription));
	elseif not pExistingValuesOnly then
		CalendarEventDescription:SetText("");
	end

	if pEvent.mMinLevel then
		CalendarEventMinLevel:SetText(pEvent.mMinLevel);
	elseif not pExistingValuesOnly then
		CalendarEventMinLevel:SetText("");
	end

	if pEvent.mMaxLevel then
		CalendarEventMaxLevel:SetText(pEvent.mMaxLevel);
	elseif not pExistingValuesOnly then
		CalendarEventMaxLevel:SetText("");
	end
	
	if not pExistingValuesOnly
	or pEvent.mType then
		gCalendarEventEditor_PreviousEventType = pEvent.mType;
		CalendarEventEditor_SetEventType(pEvent.mType);
	end
end

function CalendarEventEditor_UpdateEventFromControls(rEvent, rChangedFields)
	local	vChanged = false;
	local	vValue;
	
	-- Type
	
	vValue = UIDropDownMenu_GetSelectedValue(CalendarEventTypeEventType);
	
	if rEvent.mType ~= vValue then
		rEvent.mType = vValue;
		rChangedFields.mType = {op = "UPD", val = vValue};
		vChanged = true;
	end
	
	-- Title
	
	vValue = CalendarEventTitle:GetText();
	
	if vValue == EventDatabase_GetEventNameByID(rEvent.mType)
	or vValue == "" then
		vValue = nil;
	else
		vValue = Calendar_EscapeString(vValue);
	end
	
	if rEvent.mTitle ~= vValue then
		rEvent.mTitle = vValue;
		rChangedFields.mTitle = "UPD";
		vChanged = true;
	end
	
	-- Time
	
	if rEvent.mType == "Birth" then
		vValue = nil;
	else
		vValue = CalendarEventEditor_GetTimeControlValue(CalendarEventEditorTime);
	end
	
	if rEvent.mTime ~= vValue then
		rEvent.mTime = vValue;
		rChangedFields.mTitle = "UPD";
		vChanged = true;
	end
	
	-- Duration
	
	if rEvent.mType == "Birth" then
		vValue = nil;
	else
		vValue = UIDropDownMenu_GetSelectedValue(CalendarEventEditorDurationDuration);
	end
	
	if vValue == 0 then
		vValue = nil;
	end
	
	if rEvent.mDuration ~= vValue then
		rEvent.mDuration = vValue;
		rChangedFields.mDuration = "UPD";
		vChanged = true;
	end
	
	-- Description
	
	vValue = CalendarEventDescription:GetText();
	
	if vValue == "" then
		vValue = nil;
	else
		vValue = Calendar_EscapeString(vValue);
	end
	
	if rEvent.mDescription ~= vValue then
		rEvent.mDescription = vValue;
		rChangedFields.mDescription = "UPD";
		vChanged = true;
	end
	
	-- MinLevel
	
	if rEvent.mType == "Birth"
	or rEvent.mType == "Meet" then
		vValue = nil;
	else
		vValue = CalendarEventMinLevel:GetText();
	end
	
	if vValue and vValue ~= "" then
		vValue = tonumber(vValue);
		
		if vValue == 0 then
			vValue = nil;
		end
	else
		vValue = nil;
	end
	
	if rEvent.mMinLevel ~= vValue then
		rEvent.mMinLevel = vValue;
		rChangedFields.mMinLevel = "UPD";
		vChanged = true;
	end
	
	-- MaxLevel
	
	if rEvent.mType == "Birth"
	or rEvent.mType == "Meet"  then
		vValue = nil;
	else
		vValue = CalendarEventMaxLevel:GetText();
	end
	
	if vValue and vValue ~= "" then
		vValue = tonumber(vValue);
		
		if vValue == 0 then
			vValue = nil;
		end
	else
		vValue = nil;
	end
	
	if rEvent.mMaxLevel ~= vValue then
		rEvent.mMaxLevel = vValue;
		rChangedFields.mMaxLevel = "UPD";
		vChanged = true;
	end
	
	-- Done
	
	return vChanged;
end

function CalendarEventEditor_SetEventType(pEventType)
	-- CalendarEventTypeEventType:Show();
	-- CalendarEventTitle:Show();
	-- CalendarEventDescription:Show();
	
	local	vTitleText = CalendarEventTitle:GetText();
	local	vEventName = EventDatabase_GetEventNameByID(gCalendarEventEditor_PreviousEventType);
	local	vTitleWasEventName = vTitleText == vEventName;
	
	if pEventType == "Meet" then
		CalendarEventEditorTime:Show();
		CalendarEventEditorDuration:Show();
		CalendarEventMinLevel:Hide();
		CalendarEventMaxLevel:Hide();
		CalendarEventEditorFrameTab2:Show();
	elseif pEventType == "Birth" then
		CalendarEventEditorTime:Hide();
		CalendarEventEditorDuration:Hide();
		CalendarEventMinLevel:Hide();
		CalendarEventMaxLevel:Hide();
		CalendarEventEditorFrameTab2:Hide();
	else
		CalendarEventEditorTime:Show();
		CalendarEventEditorDuration:Show();
		CalendarEventMinLevel:Show();
		CalendarEventMaxLevel:Show();
		CalendarEventEditorFrameTab2:Show();
	end
	
	if vTitleWasEventName then
		CalendarEventTitle:SetText(EventDatabase_GetEventNameByID(pEventType));
		CalendarEventTitle:SetFocus();
		CalendarEventTitle:HighlightText();
	end
	
	-- Update the sample icon
	
	if pEventType and pEventType ~= "" then
		CalendarEventEditorEventBackground:SetTexture("Interface\\AddOns\\GroupCalendar\\Textures\\Icon-"..pEventType);
		if pEventType == "Birth" then
			CalendarEventEditorEventBackground:SetVertexColor(1, 1, 1, 0.4);
		else
			CalendarEventEditorEventBackground:SetVertexColor(1, 1, 1, 0.18);
		end
	else
		CalendarEventEditorEventBackground:SetTexture("");
	end

	--
	
	gCalendarEventEditor_PreviousEventType = pEventType;
end

function CalendarEventEditor_DeleteEvent()
	if not gCalendarEventEditor_IsNewEvent then
		EventDatabase_DeleteEvent(gCalendarEventEditor_Database, gCalendarEventEditor_Event);
	end
	
	CalendarEventEditor_CloseEditor(true);
end

function CalendarEventEditor_CloseEditor(pShowScheduleEditor)
	gCalendarEventEditor_Active = false;
	
	gCalendarEventEditor_ShowScheduleEditor = pShowScheduleEditor;
	HideUIPanel(CalendarEventEditorFrame);
end

function CalendarEventEditor_SaveEvent()
	-- Update the event
	
	local	vChangedFields = {};
	
	if not CalendarEventEditor_UpdateEventFromControls(gCalendarEventEditor_Event, vChangedFields) then
		return;
	end
	
	-- Save the event if it's new
	
	if gCalendarEventEditor_IsNewEvent then
		if (gCalendarEventEditor_Event.mTitle ~= nil and gCalendarEventEditor_Event.mTitle ~= "")
		or gCalendarEventEditor_Event.mType ~= nil then
			EventDatabase_AddEvent(gCalendarEventEditor_Database, gCalendarEventEditor_Event);
		end
	else
		EventDatabase_EventChanged(gCalendarEventEditor_Database, gCalendarEventEditor_Event, vChangedFields);
	end
	
	-- Save a template for the event
	
	if gCalendarEventEditor_Event.mType
	and gCalendarEventEditor_Event.mType ~= "BIRTH" then
		if not gGroupCalendar_RealmSettings.EventTemplates then
			gGroupCalendar_RealmSettings.EventTemplates = {};
		end
		
		local	vEventTemplate = {};
		
		vEventTemplate.mTitle = gCalendarEventEditor_Event.mTitle;
		vEventTemplate.mDescription = gCalendarEventEditor_Event.mDescription;
		vEventTemplate.mTime = gCalendarEventEditor_Event.mTime;
		vEventTemplate.mDuration = gCalendarEventEditor_Event.mDuration;
		vEventTemplate.mMinLevel = gCalendarEventEditor_Event.mMinLevel;
		vEventTemplate.mMaxLevel = gCalendarEventEditor_Event.mMaxLevel;
		vEventTemplate.mSelfAttend = CalendarEventEditor_GetSelfAttend();
		
		gGroupCalendar_RealmSettings.EventTemplates[gCalendarEventEditor_Event.mType] = vEventTemplate;
	end
end

function CalendarEventEditor_SetTimeControlValue(pFrame, pTime)
	if pTime == nil then
		return;
	end
	
	local	vFrameName = pFrame:GetName();
	
	local	vHourFrame = getglobal(vFrameName.."Hour");
	local	vMinuteFrame = getglobal(vFrameName.."Minute");
	local	vAMPMFrame = getglobal(vFrameName.."AMPM");
	
	if TwentyFourHourTime then
		local	vHour, vMinute = Calendar_ConvertTimeToHM(pTime);
		
		CalendarDropDown_SetSelectedValue(vHourFrame, vHour);
		CalendarDropDown_SetSelectedValue(vMinuteFrame, vMinute);
		vAMPMFrame:Hide();
	else
		local	vHour, vMinute, vAMPM = Calendar_ConvertTimeToHMAMPM(pTime);
		
		CalendarDropDown_SetSelectedValue(vHourFrame, vHour);
		CalendarDropDown_SetSelectedValue(vMinuteFrame, vMinute);
		
		vAMPMFrame:Show();
		CalendarDropDown_SetSelectedValue(vAMPMFrame, vAMPM);
	end
end

function CalendarEventEditor_GetTimeControlValue(pFrame)
	local	vFrameName = pFrame:GetName();
	
	local	vHourFrame = getglobal(vFrameName.."Hour");
	local	vMinuteFrame = getglobal(vFrameName.."Minute");
	local	vAMPMFrame = getglobal(vFrameName.."AMPM");
	
	local	vHour = UIDropDownMenu_GetSelectedValue(vHourFrame);
	local	vMinute = UIDropDownMenu_GetSelectedValue(vMinuteFrame);
	
	if TwentyFourHourTime then
		return Calendar_ConvertHMToTime(vHour, vMinute);
	else
		local	vAMPM = UIDropDownMenu_GetSelectedValue(vAMPMFrame);
		
		return Calendar_ConvertHMAMPMToTime(vHour, vMinute, vAMPM);
	end
end

function CalendarEventEditor_EventTypeChanged(pMenuFrame, pValue)
	CalendarEventEditor_SetEventType(pValue);
	
	-- Set the templated field values if available
	
	if gGroupCalendar_RealmSettings.EventTemplates
	and gCalendarEventEditor_IsNewEvent then
		local	vEventTemplate = gGroupCalendar_RealmSettings.EventTemplates[pValue];
		
		if vEventTemplate then
			CalendarEventEditor_UpdateControlsFromEvent(vEventTemplate, true);
			
			if vEventTemplate.mSelfAttend then
				CalendarEventEditor_SetSelfAttend(vEventTemplate.mSelfAttend);
			end
		end
	end
end

function CalendarEventEditor_ShowPanel(pPanelIndex)
	if gCalendarEventEditor_CurrentPanel > 0
	and gCalendarEventEditor_CurrentPanel ~= pPanelIndex then
		CalendarEventEditor_HidePanel(gCalendarEventEditor_CurrentPanel);
	end
	
	-- NOTE: Don't check for redundant calls since this function
	-- will be called to reset the field values as well as to 
	-- actuall show the panel when it's hidden
	
	gCalendarEventEditor_CurrentPanel = pPanelIndex;
	
	-- Update the control values
	
	if pPanelIndex == 1 then
		-- Event panel
		
		EventEditorFrameTopTexture:SetTexture("Interface\\AddOns\\GroupCalendar\\Textures\\EditorFrame-Top");
		EventEditorFrameBottomTexture:SetTexture("Interface\\AddOns\\GroupCalendar\\Textures\\EventFrame-Bottom");
	elseif pPanelIndex == 2 then
		-- Attendance panel
		
		CalendarEventEditorSelfAttend:SetChecked(CalendarEventEditor_GetSelfAttend());
		CalendarEventEditorSelfAttendText:SetText(string.format(GroupCalendar_cSelfWillAttend, gGroupCalendar_PlayerName));
		
		if EventDatabase_IsQuestingEventType(gCalendarEventEditor_PreviousEventType) then
			EventEditorFrameTopTexture:SetTexture("Interface\\AddOns\\GroupCalendar\\Textures\\EventAttendanceFrame-Top");
			EventEditorFrameBottomTexture:SetTexture("Interface\\AddOns\\GroupCalendar\\Textures\\EventAttendanceFrame-Bottom");
			CalendarAttendanceList_SetClassTotalsVisible(CalendarEventEditorAttendance, true);
		else
			EventEditorFrameTopTexture:SetTexture("Interface\\AddOns\\GroupCalendar\\Textures\\EventSimpleAttendanceFrame-Top");
			EventEditorFrameBottomTexture:SetTexture("Interface\\AddOns\\GroupCalendar\\Textures\\EventSimpleAttendanceFrame-Bottom");
			CalendarAttendanceList_SetClassTotalsVisible(CalendarEventEditorAttendance, false);
		end
	else
		GroupCalendar_DebugMessage("CalendarEventEditor_ShowPanel: Unknown index ("..pPanelIndex..")");
	end
	
	getglobal(gCalendarEventEditor_PanelFrames[pPanelIndex]):Show();
	
	PanelTemplates_SetTab(CalendarEventEditorFrame, pPanelIndex);
	
	CalendarEventEditor_Update();
end

function CalendarEventEditor_HidePanel(pFrameIndex)
	if gCalendarEventEditor_CurrentPanel ~= pFrameIndex then
		return;
	end
	
	getglobal(gCalendarEventEditor_PanelFrames[pFrameIndex]):Hide();
	gCalendarEventEditor_CurrentPanel = 0;
end

function CalendarEventEditor_Update()
	-- Event panel
	
	if gCalendarEventEditor_CurrentPanel == 1 then
	
	-- Attendance panel
	
	elseif gCalendarEventEditor_CurrentPanel == 2 then
		CalendarAttendanceList_Update(CalendarEventEditorAttendance);
	end
end

function CalendarEventEditor_GetStatusString(pStatus)
	local	vStatus1 = string.sub(pStatus, 1, 1);
	
	if vStatus1 == "N" then
		return CalendarEventEditor_cNotAttending;
	elseif vStatus1 == "Y" then
		return CalendarEventEditor_cConfirmed;
	elseif vStatus1 == "D" then
		return CalendarEventEditor_cDeclined;
	elseif vStatus1 == "S" then
		return CalendarEventEditor_cStandby;
	else
		return format(CalendarEventEditor_cUnknownStatus, pStatus);
	end
end

gGroupCalendar_cAttendanceCategories =
{
	NO = GroupCalendar_cNotAttending,
	YES = GroupCalendar_cAttending,
	PENDING = GroupCalendar_cPendingApproval
};

function CalendarAttendanceList_SetItemToRSVP(pAttendanceList, pIndex, pRSVP, pIsQuestingEvent)
	local	vListName = pAttendanceList:GetName();
	local	vItemName = vListName.."Item"..pIndex;
	local	vItem = getglobal(vItemName);
	local	vItemHighlight = getglobal(vItemName.."Highlight");
	
	local	vItemCategory = getglobal(vItemName.."Category");
	local	vItemPlayerName = getglobal(vItemName.."Name");
	local	vItemPlayerStatus = getglobal(vItemName.."Status");
	
	vItemCategory:Hide();
	vItemPlayerName:Show();
	vItemPlayerStatus:Show();
	
	local	vAttendeeInfo =
			{
				name = pRSVP.mAttendeeName,
				class = EventDatabase_GetClassByClassCode(pRSVP.mClassCode),
				race = EventDatabase_GetRaceByRaceCode(pRSVP.mRaceCode),
				level = tostring(pRSVP.mLevel),
			};
	
	local	vAttendeeFormat;
	
	if pIsQuestingEvent then
		vAttendeeFormat = GroupCalendar_cQuestAttendanceNameFormat;
	else
		vAttendeeFormat = GroupCalendar_cMeetingAttendanceNameFormat;
	end
	
	vItemPlayerName:SetText(string.gsub(vAttendeeFormat, "%$(%w+)", function (pField) return vAttendeeInfo[pField]; end));
	vItemPlayerStatus:SetText(CalendarEventEditor_GetStatusString(pRSVP.mStatus));
	
	if pRSVP.mComment and pRSVP.mComment ~= "" then
		vItem:SetNormalTexture("Interface\\Addons\\GroupCalendar\\Textures\\AttendanceNoteIcon");
	else
		vItem:SetNormalTexture("");
	end
	
	vItemHighlight:SetTexture("");
	
	vItem.IsCategoryItem = false;
	vItem.RSVP = pRSVP;
	
	vItem:Show();
end

function CalendarAttendanceList_SetItemToCategory(pAttendanceList, pIndex, pCategory)
	local	vItemName = pAttendanceList:GetName().."Item"..pIndex;
	local	vItem = getglobal(vItemName);
	
	local	vItemHighlight = getglobal(vItemName.."Highlight");
	local	vItemCategory = getglobal(vItemName.."Category");
	local	vItemPlayerName = getglobal(vItemName.."Name");
	local	vItemPlayerStatus = getglobal(vItemName.."Status");
	
	local	vCategoryName = gGroupCalendar_cAttendanceCategories[pCategory];
	
	if not vCategoryName then
		vCategoryName = EventDatabase_GetClassByClassCode(pCategory);
	end
	
	vItemCategory:SetText(vCategoryName);
	vItemCategory:Show();
	vItemPlayerName:Hide();
	vItemPlayerStatus:Hide();
	
	if pAttendanceList.CollapsedCategories[pCategory] then
		vItem:SetNormalTexture("Interface\\Buttons\\UI-PlusButton-Up");
	else
		vItem:SetNormalTexture("Interface\\Buttons\\UI-MinusButton-Up"); 
	end
	
	vItemHighlight:SetTexture("Interface\\Buttons\\UI-PlusButton-Hilight");
	
	vItem.IsCategoryItem = true;
	vItem.Category = pCategory;
	
	vItem:Show();
end

function CalendarAttendanceList_HideUnusedAttendanceItems(pAttendanceList, pFirstItem)
	local	vListName = pAttendanceList:GetName();
	
	for vIndex = pFirstItem, gGroupCalendar_cNumPlainAttendanceItems - 1 do
		local	vItemName = vListName.."Item"..vIndex;
		getglobal(vItemName):Hide();
	end
end

function CalendarEventEditor_UpdateAttendanceItems(pAttendanceList)
	vFirstItemIndex = FauxScrollFrame_GetOffset(getglobal(pAttendanceList:GetName().."ScrollFrame"));
	
	vItemIndex = 0;
	
	local	vIsQuestingEvent = EventDatabase_IsQuestingEventType(pAttendanceList.Event.mType);
	
	if not pAttendanceList.AttendanceCounts then
		CalendarAttendanceList_HideUnusedAttendanceItems(pAttendanceList, vItemIndex);
		return;
	end
	
	for vIndex, vCategory in pAttendanceList.AttendanceCounts.SortedCategories do
		local	vClassAttendanceInfo = pAttendanceList.AttendanceCounts.Categories[vCategory];
		
		if vFirstItemIndex == 0 then
			CalendarAttendanceList_SetItemToCategory(pAttendanceList, vItemIndex, vCategory);
			vItemIndex = vItemIndex + 1;
			
			if vItemIndex >= pAttendanceList.NumItems then
				CalendarAttendanceList_HideUnusedAttendanceItems(pAttendanceList, vItemIndex);
				return;
			end
		else
			vFirstItemIndex = vFirstItemIndex - 1;
		end
		
		if not pAttendanceList.CollapsedCategories[vCategory] then
			if vFirstItemIndex >= vClassAttendanceInfo.mCount then
				vFirstItemIndex = vFirstItemIndex - vClassAttendanceInfo.mCount;
			else
				for vIndex, vRSVP in vClassAttendanceInfo.mAttendees do
					if vFirstItemIndex == 0 then
						CalendarAttendanceList_SetItemToRSVP(pAttendanceList, vItemIndex, vRSVP, vIsQuestingEvent);
						vItemIndex = vItemIndex + 1;
						
						if vItemIndex >= pAttendanceList.NumItems then
							CalendarAttendanceList_HideUnusedAttendanceItems(pAttendanceList, vItemIndex);
							return;
						end
					else
						vFirstItemIndex = vFirstItemIndex - 1;
					end
				end
			end
		end
	end
	
	CalendarAttendanceList_HideUnusedAttendanceItems(pAttendanceList, vItemIndex);
end

function CalendarAttendanceList_AnyItemsCollapsed(pAttendanceList)
	if not pAttendanceList.CollapsedCategories then
		return false;
	end
	
	for vCategory, vCollapsed in pAttendanceList.CollapsedCategories do
		return true;
	end
	
	return false;
end

gCalendarAttendanceList_VerticalScrollList = nil;

function CalendarAttendanceList_OnVerticalScroll()
	CalendarAttendanceList_Update(gCalendarAttendanceList_VerticalScrollList);
end

function CalendarAttendanceList_OnClick(pButton)
	local	vAttendanceList = this:GetParent();
	
	if this.IsCategoryItem then
		if not vAttendanceList.CollapsedCategories[this.Category] then
			vAttendanceList.CollapsedCategories[this.Category] = true;
		else
			vAttendanceList.CollapsedCategories[this.Category] = nil;
		end
		
		CalendarAttendanceList_Update(vAttendanceList);
	end
end

function CalendarAttendanceList_OnEnter()
	local	vAttendanceList = this:GetParent();
	
	if this.IsCategoryItem
	or not this.RSVP.mComment
	or this.RSVP.mComment == "" then
		return;
	end
	
	GameTooltip:SetOwner(this, "ANCHOR_LEFT");
	GameTooltip:AddLine(this.RSVP.mAttendeeName);
	GameTooltip:AddLine(Calendar_UnescapeString(this.RSVP.mComment), 1, 1, 1);
	GameTooltip:Show();
end

function CalendarAttendanceList_OnLeave()
	GameTooltip:Hide();
end

function CalendarAttendanceList_SetEvent(pAttendanceList, pDatabase, pEvent)
	pAttendanceList.Event = pEvent;
	pAttendanceList.AttendanceCounts = CalendarEvent_GetAttendanceCounts(pDatabase, pEvent);
end

function CalendarAttendanceList_EventChanged(pAttendanceList, pDatabase, pEvent)
	pAttendanceList.Event = pEvent;
	pAttendanceList.AttendanceCounts = CalendarEvent_GetAttendanceCounts(pDatabase, pEvent);
	CalendarAttendanceList_Update(pAttendanceList);
end

function CalendarAttendanceList_OnLoad()
	local	vListName = this:GetName();
	
	this.NumItems = gGroupCalendar_cNumAttendanceItems;
	this.CollapsedCategories = {};
	
	-- Set the text color for the class totals
	
	for vClassID, vClassInfo in gGroupCalendar_ClassInfoByClassCode do
		local	vLabel = getglobal(vListName.."Totals"..vClassInfo.element.."sLabel");
		local	vCount = getglobal(vListName.."Totals"..vClassInfo.element.."s");
		local	vColor = RAID_CLASS_COLORS[vClassInfo.color];
		
		vLabel:SetTextColor(vColor.r, vColor.g, vColor.b);
		vCount:SetTextColor(vColor.r, vColor.g, vColor.b);
	end
	
	CalendarAttendanceList_SetClassTotalsVisible(this, true);
end

function CalendarAttendanceList_OnShow()
	this.CollapsedCategories = {};
	CalendarAttendanceList_Update(this);
end

function CalendarAttendanceList_SetClassTotalsVisible(pAttendanceList, pVisible)
	local	vListName = pAttendanceList:GetName();
	local	vTotals = getglobal(vListName.."Totals");
	local	vScrollFrame = getglobal(vListName.."ScrollFrame");
	
	if pVisible then
		vTotals:Show();
		pAttendanceList.NumItems = gGroupCalendar_cNumAttendanceItems;
		vScrollFrame:SetHeight(195);
	else
		vTotals:Hide();
		pAttendanceList.NumItems = gGroupCalendar_cNumPlainAttendanceItems;
		vScrollFrame:SetHeight(254);
	end
end

function CalendarAttendanceList_Update(pAttendanceList)
	-- Update the scroll frame
	
	local		vTotalCount = 0;
	
	if pAttendanceList.AttendanceCounts then
		for vCategory, vClassAttendanceInfo in pAttendanceList.AttendanceCounts.Categories do
			vTotalCount = vTotalCount + 1;
			
			if not pAttendanceList.CollapsedCategories[vCategory] then
				vTotalCount = vTotalCount + vClassAttendanceInfo.mCount;
			end
		end
	end
	
	FauxScrollFrame_Update(
			getglobal(pAttendanceList:GetName().."ScrollFrame"),
			vTotalCount,
			pAttendanceList.NumItems,
			gGroupCalendar_cAttendanceItemHeight,
			nil, nil, nil,
			nil,
			293, 316);
	
	CalendarEventEditor_UpdateAttendanceItems(pAttendanceList);

	-- Update the expand/collapse all button
	
	local	vExpandAllButton = getglobal(pAttendanceList:GetName().."ExpandAllButton")
	
	if CalendarAttendanceList_AnyItemsCollapsed(pAttendanceList) then
		vExpandAllButton:SetNormalTexture("Interface\\Buttons\\UI-PlusButton-Up");
	else
		vExpandAllButton:SetNormalTexture("Interface\\Buttons\\UI-MinusButton-Up"); 
	end
	
	CalendarAttendanceList_UpdateClassTotals(pAttendanceList);
end

function CalendarAttendanceList_ToggleExpandAll(pAttendanceList)
	-- Just return if there are no attendance items to toggle
	
	if not pAttendanceList
	or not pAttendanceList.AttendanceCounts then
		return;
	end
	
	if CalendarAttendanceList_AnyItemsCollapsed(pAttendanceList) then
		-- Expand all
		
		pAttendanceList.CollapsedCategories = {};
	else
		-- Collapse all
		
		for vCategory, vClassAttendanceInfo in pAttendanceList.AttendanceCounts.Categories do
			pAttendanceList.CollapsedCategories[vCategory] = true;
		end
	end
	
	CalendarAttendanceList_Update(pAttendanceList);
end

function CalendarAttendanceList_UpdateClassTotals(pAttendanceList)
	local	vListName = pAttendanceList:GetName();
	
	for vClassCode, vClassInfo in gGroupCalendar_ClassInfoByClassCode do
		local	vClassAttendanceInfo;
		
		if pAttendanceList.AttendanceCounts then
			vClassAttendanceInfo = pAttendanceList.AttendanceCounts.Categories[vClassCode];
		else
			vClassAttendanceInfo = nil;
		end
		
		local	vCountTextItem = getglobal(vListName.."Totals"..vClassInfo.element.."s");
		
		if not vClassAttendanceInfo then
			vCountTextItem:SetText("0");
			vCountTextItem:SetTextColor(0.9, 0.2, 0.2);
		else
			local	vColor = RAID_CLASS_COLORS[vClassInfo.color];
			
			vCountTextItem:SetText(vClassAttendanceInfo.mCount);
			vCountTextItem:SetTextColor(vColor.r, vColor.g, vColor.b);
		end
	end
end

function CalendarEventEditor_GetSelfAttend()
	return EventDatabase_FindEventRSVPString(gCalendarEventEditor_Event, gGroupCalendar_PlayerName) ~= nil;
end

function CalendarEventEditor_SetSelfAttend(pWillAttend)
	if pWillAttend then
		local vRSVP = EventDatabase_CreateCurrentPlayerRSVP(
								gCalendarEventEditor_Database,
								gCalendarEventEditor_Event,
								"Y",
								nil);
		
		EventDatabase_AddEventRSVP(
				gCalendarEventEditor_Database,
				gCalendarEventEditor_Event,
				gGroupCalendar_PlayerName,
				vRSVP)
	else
		EventDatabase_RemoveEventRSVP(
				gCalendarEventEditor_Database,
				gCalendarEventEditor_Event,
				gGroupCalendar_PlayerName)
	end
end

function CalendarEventEditor_AskDeleteEvent()
	-- If it's new just kill it without asking
	
	if gCalendarEventEditor_IsNewEvent then
		CalendarEventEditor_DeleteEvent();
	else
		-- Update an event record so we can display a meaningful name
		
		local	vChangedFields = {};
		local	vEvent = {};
		
		CalendarEventEditor_UpdateEventFromControls(vEvent, vChangedFields);
		
		StaticPopup_Show("CONFIRM_CALENDAR_EVENT_DELETE", EventDatabase_GetEventDisplayName(vEvent));
	end
end
