gCalendarEventViewer_ShowScheduleEditor = false;
gCalendarEventViewer_Active = false;

gCalendarEventViewer_Database = nil;
gCalendarEventViewer_Event = nil;

gCalendarEventViewer_PanelFrames =
{
	"CalendarEventViewerEventFrame",
	"CalendarEventViewerAttendanceFrame",
};

gCalendarEventViewer_CurrentPanel = 1;

function CalendarEventViewer_ViewEvent(pDatabase, pEvent)
	gCalendarEventViewer_Database = pDatabase;
	gCalendarEventViewer_Event = pEvent;
	
	CalendarAttendanceList_SetEvent(CalendarEventViewerAttendance, pDatabase, pEvent);
	CalendarEventViewer_UpdateControlsFromEvent(gCalendarEventViewer_Event);
	
	ShowUIPanel(CalendarEventViewerFrame);
	
	CalendarEventTitle:SetFocus();
	CalendarEventTitle:HighlightText(0, 100);
	
	gCalendarEventViewer_ShowScheduleEditor = false;
	gCalendarEventViewer_Active = true;
end

function CalendarEventViewer_DoneViewing()
	if not gCalendarEventViewer_Active then
		return;
	end
	
	CalendarEventViewer_Close(true);
end

function CalendarEventViewer_ScheduleChanged(pDate)
	if gCalendarEventViewer_Active
	and gCalendarEventViewer_Event.mDate == pDate then
		CalendarAttendanceList_EventChanged(CalendarEventViewerAttendance, gCalendarEventViewer_Database, gCalendarEventViewer_Event);
		CalendarEventViewer_UpdateControlsFromEvent(gCalendarEventViewer_Event);
	end
end

function CalendarEventViewer_Save()
	-- Save attendance feedback
	
	if gCalendarEventViewer_Event.mType ~= "Birth"
	and EventDatabase_PlayerIsQualifiedForEvent(gCalendarEventViewer_Event) then
		local	vPendingRSVP = EventDatabase_FindRSVPRequestData(gGroupCalendar_UserDatabase, gCalendarEventViewer_Database.UserName, gCalendarEventViewer_Event.mID);
		local	vEventRSVP = EventDatabase_FindEventRSVP(gCalendarEventViewer_Event, gGroupCalendar_PlayerName);
		local	vChanged = false;
		local	vStatus;
		local	vComment;
		
		if not vPendingRSVP then
			if vEventRSVP then
				vStatus = vEventRSVP.mStatus;
				vComment = vEventRSVP.mComment;
			else
				vStatus = nil;
				vComment = "";
			end
		else
			vStatus = vPendingRSVP.mStatus;
			vComment = vPendingRSVP.mComment;
		end
		
		if not vComment then
			vComment = "";
		end
		
		-- Update the status
		
		if CalendarEventViewerYes:GetChecked()
		and vStatus ~= "Y" then
			vStatus = "Y";
			vChanged = true;
		elseif CalendarEventViewerNo:GetChecked()
		and vStatus ~= "N" then
			vStatus = "N";
			vChanged = true;
		end
		
		-- Update the comment
		
		local	vNewComment = Calendar_EscapeString(CalendarEventViewerComment:GetText());
		
		if vComment ~= vNewComment then
			vComment = vNewComment;
			vChanged = true;
		end
		
		-- Add a new RSVP if it's changed
		
		if vChanged then
			local	vRSVP = EventDatabase_CreateCurrentPlayerRSVP(
									gCalendarEventViewer_Database,
									gCalendarEventViewer_Event,
									vStatus,
									vComment);
			
			EventDatabase_AddRSVP(gGroupCalendar_UserDatabase, vRSVP);
		end
	end
end

function CalendarEventViewer_Close(pShowScheduleEditor)
	gCalendarEventViewer_ShowScheduleEditor = pShowScheduleEditor;
	HideUIPanel(CalendarEventViewerFrame);
end

function CalendarEventViewer_OnLoad()
	-- Tabs
	
	PanelTemplates_SetNumTabs(this, table.getn(gCalendarEventViewer_PanelFrames));
	CalendarEventViewerFrame.selectedTab = gCalendarEventViewer_CurrentPanel;
	PanelTemplates_UpdateTabs(this);
end

function CalendarEventViewer_OnShow()
	PlaySound("igCharacterInfoOpen");
	
	CalendarEventTypeEventType.ChangedValueFunc = CalendarEventViewer_EventTypeChanged;

	CalendarEventViewer_ShowPanel(1); -- Always switch to the event view when showing the window
end

function CalendarEventViewer_OnHide()
	PlaySound("igCharacterInfoClose");
	
	CalendarEventViewer_Save();
	
	if not gCalendarEventViewer_ShowScheduleEditor then
		HideUIPanel(CalendarEditorFrame);
	end
	
	gCalendarEventViewer_Database = nil;
	gCalendarEventViewer_Event = nil;
	
	gCalendarEventViewer_Active = false;
end

function CalendarEventViewer_UpdateControlsFromEvent(pEvent)
	CalendarEventViewerEventFrameEventTitle:SetText(EventDatabase_GetEventDisplayName(pEvent));
	
	if pEvent.mTime ~= nil then
		local		vTime;
		local		vDate = pEvent.mDate;
		
		if gGroupCalendar_Settings.ShowEventsInLocalTime then
			vTime = pEvent.mTime + gGroupCalendar_ServerTimeZoneOffset;
			
			if vTime < 0 then
				vTime = vTime + 1440; -- +24 hours
				vDate = vDate - 1;
			elseif vTime >= 1440 then
				vTime = vTime - 1440;
				vDate = vDate + 1;
			end
		else
			vTime = pEvent.mTime;
		end
		
		-- Set the date
		
		CalendarEventViewerEventFrameDate:SetText(Calendar_GetLongDateString(vDate));
		
		--
		
		local		vTimeString;
		
		if pEvent.mDuration ~= nil then

			vTimeString = string.format(
								GroupCalendar_cTimeRangeFormat,
								Calendar_GetShortTimeString(vTime),
								Calendar_GetShortTimeString(vTime + pEvent.mDuration));
			
		else
			vTimeString = Calendar_GetShortTimeString(vTime);
		end

		CalendarEventViewerEventFrameTime:SetText(vTimeString);
		CalendarEventViewerEventFrameTime:Show();
	else
		CalendarEventViewerEventFrameDate:SetText(Calendar_GetLongDateString(pEvent.mDate));
		CalendarEventViewerEventFrameTime:Hide();
	end
	
	if pEvent.mType ~= "Birth"
	and pEvent.mType ~= "Meet" then
		if pEvent.mMinLevel ~= nil then
			if pEvent.mMaxLevel ~= nil then
				if pEvent.mMinLevel == pEvent.mMaxLevel then
					CalendarEventViewerEventFrameLevels:SetText(string.format(CalendarEventViewer_cSingleLevel, pEvent.mMinLevel));
				else
					CalendarEventViewerEventFrameLevels:SetText(string.format(CalendarEventViewer_cLevelRangeFormat, pEvent.mMinLevel, pEvent.mMaxLevel));
				end
			else
				if pEvent.mMinLevel == 60 then
					CalendarEventViewerEventFrameLevels:SetText(string.format(CalendarEventViewer_cSingleLevel, pEvent.mMinLevel));
				else
					CalendarEventViewerEventFrameLevels:SetText(string.format(CalendarEventViewer_cMinLevelFormat, pEvent.mMinLevel));
				end
			end
			
			CalendarEventViewerEventFrameLevels:Show();
		else
			if pEvent.mMaxLevel ~= nil then
				CalendarEventViewerEventFrameLevels:SetText(string.format(CalendarEventViewer_cMaxLevelFormat, pEvent.mMaxLevel));
			else
				CalendarEventViewerEventFrameLevels:SetText(CalendarEventViewer_cAllLevels);
			end
			
			CalendarEventViewerEventFrameLevels:Show();
		end
		
		if EventDatabase_PlayerIsQualifiedForEvent(gCalendarEventViewer_Event) then
			CalendarEventViewerEventFrameLevels:SetTextColor(1.0, 0.82, 0);
		else
			CalendarEventViewerEventFrameLevels:SetTextColor(1.0, 0.2, 0.2);
		end
	else
		CalendarEventViewerEventFrameLevels:Hide();
	end
	
	if pEvent.mDescription then
		CalendarEventViewerDescText:SetText(Calendar_UnescapeString(pEvent.mDescription));
		CalendarEventViewerDescText:Show();
	else
		CalendarEventViewerDescText:Hide();
	end
	
	if pEvent.mType ~= "Birth" then
		CalendarEventViewer_SetAttendanceVisible(true);
		
		if EventDatabase_PlayerIsQualifiedForEvent(gCalendarEventViewer_Event) then
			CalendarEventViewer_SetAttendanceEnabled(true);
			
			local	vPendingRSVP = EventDatabase_FindRSVPRequestData(gGroupCalendar_UserDatabase, gCalendarEventViewer_Database.UserName, gCalendarEventViewer_Event.mID);
			local	vEventRSVP = EventDatabase_FindEventRSVP(gCalendarEventViewer_Event, gGroupCalendar_PlayerName);
			
			if not vPendingRSVP then
				if vEventRSVP then
					CalendarEventViewerYes:SetChecked(vEventRSVP.mStatus == "Y");
					CalendarEventViewerNo:SetChecked(vEventRSVP.mStatus == "N");
					if vEventRSVP.mComment then
						CalendarEventViewerComment:SetText(Calendar_UnescapeString(vEventRSVP.mComment));
					else
						CalendarEventViewerComment:SetText("");
					end
				else
					CalendarEventViewerYes:SetChecked(false);
					CalendarEventViewerNo:SetChecked(false);
					CalendarEventViewerComment:SetText("");
				end
			else
				CalendarEventViewerYes:SetChecked(vPendingRSVP.mStatus == "Y");
				CalendarEventViewerNo:SetChecked(vPendingRSVP.mStatus == "N");
				if vPendingRSVP.mComment then
					CalendarEventViewerComment:SetText(Calendar_UnescapeString(vPendingRSVP.mComment));
				else
					CalendarEventViewerComment:SetText("");
				end
			end
			
			CalendarEventViewer_SetResponseStatus(vPendingRSVP, vEventRSVP);
		else
			CalendarEventViewerYes:SetChecked(false);
			CalendarEventViewerNo:SetChecked(false);
			CalendarEventViewerComment:SetText("");
			
			CalendarEventViewer_SetAttendanceEnabled(false);
		end
	else
		CalendarEventViewer_SetAttendanceVisible(false);
	end
	
	if pEvent.mType ~= nil then
		CalendarEventViewerEventBackground:SetTexture("Interface\\AddOns\\GroupCalendar\\Textures\\Icon-"..pEvent.mType);
		if pEvent.mType == "Birth" then
			CalendarEventViewerEventBackground:SetVertexColor(1, 1, 1, 0.8);
		else
			CalendarEventViewerEventBackground:SetVertexColor(1, 1, 1, 0.15);
		end
	else
		CalendarEventViewerEventBackground:SetTexture("");
	end
end

function CalendarEventViewer_SetAttendanceEnabled(pEnable)
	Calendar_SetCheckButtonEnable(CalendarEventViewerYes, pEnable);
	Calendar_SetCheckButtonEnable(CalendarEventViewerNo, pEnable);
	Calendar_SetEditBoxEnable(CalendarEventViewerComment, pEnable);
	
	if pEnable then
		CalendarEventViewerEventFrameStatus:Show();
	else
		CalendarEventViewerEventFrameStatus:Hide();
	end
end

function CalendarEventViewer_SetAttendanceVisible(pVisible)
	if pVisible then
		CalendarEventViewerYes:Show();
		CalendarEventViewerNo:Show();
		CalendarEventViewerComment:Show();
		CalendarEventViewerEventFrameStatus:Show();
		CalendarEventViewerFrameTab2:Show();
	else
		CalendarEventViewerYes:Hide();
		CalendarEventViewerNo:Hide();
		CalendarEventViewerComment:Hide();
		CalendarEventViewerEventFrameStatus:Hide();
		CalendarEventViewerFrameTab2:Hide();
	end
end

function CalendarEventViewer_CalculateResponseStatus(pPendingRSVP, pEventRSVP)
	if pPendingRSVP then
		return 2;
	elseif pEventRSVP then
		return 3;
	else
		return 1;
	end
end

function CalendarEventViewer_SetResponseStatus(pPendingRSVP, pEventRSVP)
	local	vStatus = CalendarEventViewer_CalculateResponseStatus(pPendingRSVP, pEventRSVP);
	CalendarEventViewerEventFrameStatus:SetText(CalendarEventViewer_cResponseMessage[vStatus]);
end

function CalendarEventViewer_ShowPanel(pPanelIndex)
	if gCalendarEventViewer_CurrentPanel > 0
	and gCalendarEventViewer_CurrentPanel ~= pPanelIndex then
		CalendarEventViewer_HidePanel(gCalendarEventViewer_CurrentPanel);
	end
	
	-- NOTE: Don't check for redundant calls since this function
	-- will be called to reset the field values as well as to 
	-- actuall show the panel when it's hidden
	
	gCalendarEventViewer_CurrentPanel = pPanelIndex;
	
	-- Update the control values
	
	if pPanelIndex == 1 then
		-- Event panel

		EventViewerFrameTopTexture:SetTexture("Interface\\AddOns\\GroupCalendar\\Textures\\ViewerFrame-Top");
		EventViewerFrameBottomTexture:SetTexture("Interface\\AddOns\\GroupCalendar\\Textures\\ViewerFrame-Bottom");
		
	elseif pPanelIndex == 2 then
		-- Attendance panel

		if EventDatabase_IsQuestingEventType(gCalendarEventViewer_Event.mType) then
			EventViewerFrameTopTexture:SetTexture("Interface\\AddOns\\GroupCalendar\\Textures\\EventAttendanceFrame-Top");
			EventViewerFrameBottomTexture:SetTexture("Interface\\AddOns\\GroupCalendar\\Textures\\EventAttendanceFrame-Bottom");
			CalendarAttendanceList_SetClassTotalsVisible(CalendarEventViewerAttendance, true);
		else
			EventViewerFrameTopTexture:SetTexture("Interface\\AddOns\\GroupCalendar\\Textures\\EventSimpleAttendanceFrame-Top");
			EventViewerFrameBottomTexture:SetTexture("Interface\\AddOns\\GroupCalendar\\Textures\\EventSimpleAttendanceFrame-Bottom");
			CalendarAttendanceList_SetClassTotalsVisible(CalendarEventViewerAttendance, false);
		end
	else
		GroupCalendar_DebugMessage("CalendarEventViewer_ShowPanel: Unknown index ("..pPanelIndex..")");
	end
	
	getglobal(gCalendarEventViewer_PanelFrames[pPanelIndex]):Show();
	
	PanelTemplates_SetTab(CalendarEventViewerFrame, pPanelIndex);
end

function CalendarEventViewer_HidePanel(pFrameIndex)
	if gCalendarEventViewer_CurrentPanel ~= pFrameIndex then
		return;
	end
	
	CalendarEventViewer_Save();
	
	getglobal(gCalendarEventViewer_PanelFrames[pFrameIndex]):Hide();
	gCalendarEventViewer_CurrentPanel = 0;
end

