GroupCalendar_cTitle = string.format(GroupCalendar_cTitle, gGroupCalendar_VersionString);

gGroupCalendar_Settings =
{
	Debug = false,
	ShowEventsInLocalTime = false,
};

gGroupCalendar_RealmSettings = nil;

gGroupCalendar_PlayerName = nil;
gGroupCalendar_PlayerGuild = nil;
gGroupCalendar_PlayerLevel = nil;
gGroupCalendar_PlayerGuildRank = nil;
gGroupCalendar_RealmName = GetRealmName();

gGroupCalendar_ServerTimeZoneOffset = 0; -- Offset that, when added to the server time yields the local time

gGroupCalendar_CurrentPanel = 1;

gGroupCalendar_PanelFrames =
{
	"GroupCalendarCalendarFrame",
	"GroupCalendarChannelFrame",
	"GroupCalendarTrustFrame",
	"GroupCalendarAboutFrame",
};

function GroupCalendar_OnLoad()
	GroupCalendar_CalculateTimeZoneOffset();
	
	SlashCmdList["CALENDAR"] = function(pMessage)
		ShowUIPanel(GroupCalendarFrame);
	end
	
	tinsert(UISpecialFrames, "GroupCalendarFrame");
	
	UIPanelWindows["GroupCalendarFrame"] =
	{
		area = "left",
		pushable = 5
	};	
	
	SLASH_CALENDAR1 = "/calendar";
	
	gGroupCalendar_PlayerName = UnitName("player");
	gGroupCalendar_PlayerLevel = UnitLevel("player");
	
	-- Register events
	
	this:RegisterEvent("VARIABLES_LOADED");
	this:RegisterEvent("CHAT_MSG_CHANNEL");
	this:RegisterEvent("GUILD_ROSTER_UPDATE");
	this:RegisterEvent("PLAYER_GUILD_UPDATE");
	this:RegisterEvent("PLAYER_LEVEL_UP");
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	
	this:RegisterEvent("PLAYER_CAMPING");
	this:RegisterEvent("PLAYER_QUITING");
	this:RegisterEvent("LOGOUT_CANCEL");
	
	this:RegisterForDrag("LeftButton");
	
	-- Tabs
	
	PanelTemplates_SetNumTabs(this, table.getn(gGroupCalendar_PanelFrames));
	GroupCalendarFrame.selectedTab = gGroupCalendar_CurrentPanel;
	PanelTemplates_UpdateTabs(this);
	
	-- Done initializing
	
	if DEFAULT_CHAT_FRAME then
		DEFAULT_CHAT_FRAME:AddMessage(GroupCalendar_cLoadMessage, 0.8, 0.8, 0.2);
	end
end

function GroupCalendar_VariablesLoaded()
	GroupCalendar_CalculateTimeZoneOffset();
	
	EventDatabase_SetUserName(gGroupCalendar_PlayerName);
	
	EventDatabase_CheckDatabases();
	
	CalendarNetwork_CalendarLoaded();
end

function GroupCalendar_OnShow()
	PlaySound("igCharacterInfoOpen");
	
	local	vYear, vMonth, vDay, vHour, vMinute = Calendar_GetCurrentYearMonthDayHourMinute();
	local	vMonthStartDate = Calendar_ConvertMDYToDate(vMonth, 1, vYear);
	
	-- Update the guild roster
	
	if IsInGuild() and GetNumGuildMembers() == 0 then
		GuildRoster();
	end
	
	Calendar_SetDisplayDate(vMonthStartDate);
	Calendar_SetActualDate(vMonthStartDate + vDay - 1);
	GroupCalendar_ShowPanel(1); -- Always switch  back to the Calendar view when showing the window
end

function GroupCalendar_OnHide()
	PlaySound("igCharacterInfoClose");
	CalendarEventEditor_DoneEditing();
	CalendarEventViewer_DoneViewing();
	CalendarEditor_Close();
end

function GroupCalendar_OnEvent(pEvent)
	if 	pEvent == "CHAT_MSG_CHANNEL" then
		
		if arg9
		and strlower(arg9) == gGroupCalendar_Channel.NameLower
		and arg1
		and type(arg1) == "string"
		then
			if arg2 == gGroupCalendar_PlayerName then
				-- Ignore messages from ourselves
				
				if not gGroupCalendar_Settings.Debug then
					return;
				end
				
				-- Special debugging case: allow self-send of messages starting with '!'
				
				if strsub(arg1, 1, 1) ~= "!" then
					return;
				end
				
				arg1 = strsub(arg1, 2);
			end
						
			CalendarNetwork_ChannelMessageReceived(arg2, arg1);
		end
	elseif pEvent == "VARIABLES_LOADED" then
		GroupCalendar_VariablesLoaded();
	elseif pEvent == "GUILD_ROSTER_UPDATE" then
		-- Ignore the update if we're not initialized yet
		
		if not gGroupCalendar_RealmSettings then
			return;
		end
		
		CalendarNetwork_GuildRosterChanged();

	elseif pEvent == "PLAYER_GUILD_UPDATE" then
		-- Ignore the update if we're not initialized yet
		
		if not gGroupCalendar_RealmSettings then
			return;
		end
		
		CalendarNetwork_CheckPlayerGuild();

		GroupCalendar_UpdateEnabledControls();
		
	elseif pEvent == "PLAYER_LEVEL_UP" then
		gGroupCalendar_PlayerLevel = tonumber(arg1);
		GroupCalendar_MajorDatabaseChange(nil);
	elseif pEvent == "PLAYER_ENTERING_WORLD" then
		gGroupCalendar_PlayerLevel = UnitLevel("player");
		GroupCalendar_MajorDatabaseChange(nil);
	elseif pEvent == "PLAYER_CAMPING"
	or pEvent == "PLAYER_QUITING" then
		CalendarNetwork_SuspendChannel();
	elseif pEvent == "LOGOUT_CANCEL" then
		CalendarNetwork_ResumeChannel();
	end
end

function GroupCalendar_UpdateEnabledControls()
	if GroupCalendarFrame.selectedTab == 1 then
		-- Update the calendar display
		
	elseif GroupCalendarFrame.selectedTab == 2 then
		-- Update the channel frame
		
		Calendar_SetCheckButtonEnable(GroupCalendarAutoChannelConfig, IsInGuild());
		
		Calendar_SetEditBoxEnable(GroupCalendarChannelName, GroupCalendarManualChannelConfig:GetChecked());
		Calendar_SetEditBoxEnable(GroupCalendarChannelPassword, GroupCalendarManualChannelConfig:GetChecked());
		
		if IsInGuild()
		and CanEditPublicNote()
		and GroupCalendarManualChannelConfig:GetChecked() then
			Calendar_SetCheckButtonEnable(GroupCalendarStoreAutoConfig, true);

			if GroupCalendarStoreAutoConfig:GetChecked() then
				Calendar_SetEditBoxEnable(GroupCalendarAutoConfigPlayer, true);
			else
				Calendar_SetEditBoxEnable(GroupCalendarAutoConfigPlayer, false);
			end
		else
			Calendar_SetCheckButtonEnable(GroupCalendarStoreAutoConfig, false);
			Calendar_SetEditBoxEnable(GroupCalendarAutoConfigPlayer, false);
		end
		
		Calendar_SetButtonEnable(GroupCalendarApplyChannelButton, GroupCalendar_ChannelPanelHasChanges());
		
	elseif GroupCalendarFrame.selectedTab == 3 then
		-- Update the trust frame
		
		if gGroupCalendar_RealmSettings.Channel.AutoConfig then
			Calendar_SetDropDownEnable(GroupCalendarTrustGroup, false);
			Calendar_SetDropDownEnable(GroupCalendarTrustMinRank, false);
		else
			GroupCalendar_SaveTrustGroup();
			
			Calendar_SetDropDownEnable(GroupCalendarTrustGroup, true);
			Calendar_SetDropDownEnable(GroupCalendarTrustMinRank, UIDropDownMenu_GetSelectedValue(GroupCalendarTrustGroup) == 2);
		end
		
	elseif GroupCalendarFrame.selectedTab == 4 then
		-- Update the ignore frame
	
	end
end

function GroupCalendar_SetAutoChannelConfig(pEnableAutoConfig)
	GroupCalendarAutoChannelConfig:SetChecked(pEnableAutoConfig);
	GroupCalendarManualChannelConfig:SetChecked(not pEnableAutoConfig);
	
	GroupCalendar_UpdateEnabledControls();
end

function GroupCalendar_EnableAutoConfigPlayer(pEnableStoreAutoConfig)
	GroupCalendarStoreAutoConfig:SetChecked(pEnableStoreAutoConfig);
	GroupCalendar_UpdateEnabledControls();
end

function GroupCalendar_SavePanel(pIndex)
	if pIndex == 2 then
		-- Channel panel
		
		gGroupCalendar_RealmSettings.Channel.AutoConfig = GroupCalendarAutoChannelConfig:GetChecked() == 1;
		
		if gGroupCalendar_RealmSettings.Channel.AutoConfig then
			gGroupCalendar_RealmSettings.Channel.Name = nil;
			gGroupCalendar_RealmSettings.Channel.Password = nil;
			gGroupCalendar_RealmSettings.Channel.AutoConfigPlayer = nil;
			
			CalendarNetwork_ScheduleAutoConfig(0.5);
			
			GroupCalendar_ShowPanel(pIndex); -- Refresh the controls
		else
			gGroupCalendar_RealmSettings.Channel.Name = GroupCalendarChannelName:GetText();

			if gGroupCalendar_RealmSettings.Channel.Name == "" then
				gGroupCalendar_RealmSettings.Channel.Name = nil;
			end
			
			gGroupCalendar_RealmSettings.Channel.Password = GroupCalendarChannelPassword:GetText();
			
			if gGroupCalendar_RealmSettings.Channel.Password == "" then
				gGroupCalendar_RealmSettings.Channel.Password = nil;
			end
			
			CalendarNetwork_SetChannel(gGroupCalendar_RealmSettings.Channel.Name, gGroupCalendar_RealmSettings.Channel.Password);
			
			if GroupCalendarStoreAutoConfig:GetChecked() then
				gGroupCalendar_RealmSettings.Channel.AutoConfigPlayer = GroupCalendarAutoConfigPlayer:GetText();
				CalendarNetwork_SetAutoConfigData(gGroupCalendar_RealmSettings.Channel.AutoConfigPlayer);
			elseif CanEditPublicNote() then
				CalendarNetwork_RemoveAllAutoConfigData();
			end
		end
	elseif pIndex == 3 then
		-- Trust panel
		
		if not gGroupCalendar_RealmSettings.Channel.AutoConfig then
			GroupCalendar_SaveTrustGroup();
		end
	end
	
	GroupCalendar_UpdateEnabledControls();
end

function GroupCalendar_ChannelPanelHasChanges()
	if (GroupCalendarAutoChannelConfig:GetChecked() == 1)
	~= gGroupCalendar_RealmSettings.Channel.AutoConfig then
		return true;
	end
	
	if GroupCalendarManualChannelConfig:GetChecked() then
		local	vChannelName = gGroupCalendar_RealmSettings.Channel.Name;
		
		if not vChannelName then
			vChannelName = "";
		end
		
		local	vChannelPassword = gGroupCalendar_RealmSettings.Channel.Password;
		
		if not vChannelPassword then
			vChannelPassword = "";
		end
		
		if GroupCalendarChannelName:GetText() ~= vChannelName
		or GroupCalendarChannelPassword:GetText() ~= vChannelPassword then
			return true;
		end
	end
	
	local	vStoreAutoConfigIsEnabled = (not gGroupCalendar_RealmSettings.Channel.AutoConfig)
	                               and (gGroupCalendar_RealmSettings.Channel.AutoConfigPlayer ~= nil);

	if (GroupCalendarStoreAutoConfig:GetChecked() == 1)
	~= vStoreAutoConfigIsEnabled then
		return true;
	end
	
	if GroupCalendarStoreAutoConfig:GetChecked()
	and GroupCalendarAutoConfigPlayer:GetText() ~= gGroupCalendar_RealmSettings.Channel.AutoConfigPlayer then
		return true;
	end
	
	return false;
end

function GroupCalendar_SaveTrustGroup()
	local	vTrustGroup = UIDropDownMenu_GetSelectedValue(GroupCalendarTrustGroup);
	local	vChanged = false;
	
	if vTrustGroup == 1
	and not gGroupCalendar_RealmSettings.Security.TrustAnyone then
		gGroupCalendar_RealmSettings.Security.TrustAnyone = true;
		gGroupCalendar_RealmSettings.Security.TrustGuildies = false;
		vChanged = true;
	elseif vTrustGroup == 2 then
		if not gGroupCalendar_RealmSettings.Security.TrustGuildies then
			gGroupCalendar_RealmSettings.Security.TrustAnyone = false;
			gGroupCalendar_RealmSettings.Security.TrustGuildies = true;
			vChanged = true;
		end
		
		local	vMinTrustedRank = UIDropDownMenu_GetSelectedValue(GroupCalendarTrustMinRank);
		
		if vMinTrustedRank ~= gGroupCalendar_RealmSettings.Security.MinTrustedRank then
			gGroupCalendar_RealmSettings.Security.MinTrustedRank = vMinTrustedRank;
			vChanged = true;
		end
	elseif vTrustGroup == 3
	and (gGroupCalendar_RealmSettings.Security.TrustAnyone
	or gGroupCalendar_RealmSettings.Security.TrustGuildies) then
		gGroupCalendar_RealmSettings.Security.TrustAnyone = false;
		gGroupCalendar_RealmSettings.Security.TrustGuildies = false;
		vChanged = true;
	end
	
	if vChanged then
		-- Update auto-config string with the trust config
		
		if CanEditPublicNote()
		and gGroupCalendar_RealmSettings.Channel.AutoConfigPlayer then
			CalendarNetwork_SetAutoConfigData(gGroupCalendar_RealmSettings.Channel.AutoConfigPlayer);
		end
		
		CalendarTrust_TrustSettingsChanged();
	end
end

function GroupCalendar_ShowPanel(pPanelIndex)
	if gGroupCalendar_CurrentPanel > 0
	and gGroupCalendar_CurrentPanel ~= pPanelIndex then
		GroupCalendar_HidePanel(gGroupCalendar_CurrentPanel);
	end
	
	-- NOTE: Don't check for redundant calls since this function
	-- will be called to reset the field values as well as to 
	-- actuall show the panel when it's hidden
	
	gGroupCalendar_CurrentPanel = pPanelIndex;
	
	-- Hide the event editor/viewer if the calendar panel is being hidden
	
	if pPanelIndex ~= 1 then
		CalendarEventEditor_DoneEditing();
		CalendarEventViewer_DoneViewing();
		CalendarEditor_Close();
	end
	
	-- Update the control values
	
	if pPanelIndex == 1 then
	elseif pPanelIndex == 2 then
		-- Channel panel
		
		if gGroupCalendar_RealmSettings.Channel.AutoConfig
		and not IsInGuild() then
			gGroupCalendar_RealmSettings.Channel.AutoConfig = false;
		end
		
		GroupCalendarAutoChannelConfig:SetChecked(gGroupCalendar_RealmSettings.Channel.AutoConfig);
		GroupCalendarManualChannelConfig:SetChecked(not gGroupCalendar_RealmSettings.Channel.AutoConfig);
		
		local	vChannelName;
		local	vAutoConfigPlayer;
		
		if gGroupCalendar_RealmSettings.Channel.AutoConfig then
			vChannelName = gGroupCalendar_Channel.Name;
			if gGroupCalendar_Channel.Password then
				GroupCalendarChannelPassword:SetText("******");
			else
				GroupCalendarChannelPassword:SetText("");
			end
			
			vAutoConfigPlayer = gGroupCalendar_Channel.AutoPlayer;
		else
			vChannelName = gGroupCalendar_RealmSettings.Channel.Name;
			vAutoConfigPlayer = gGroupCalendar_RealmSettings.Channel.AutoConfigPlayer;
			
			if gGroupCalendar_RealmSettings.Channel.Password then
				GroupCalendarChannelPassword:SetText(gGroupCalendar_RealmSettings.Channel.Password);
			else
				GroupCalendarChannelPassword:SetText("");
			end
		end
		
		if vChannelName ~= nil then
			GroupCalendarChannelName:SetText(vChannelName);
		else
			GroupCalendarChannelName:SetText("");
		end
		
		if vAutoConfigPlayer then
			GroupCalendarAutoConfigPlayer:SetText(vAutoConfigPlayer);
		else
			GroupCalendarAutoConfigPlayer:SetText("");
		end
		
		if not gGroupCalendar_RealmSettings.Channel.AutoConfig
		and gGroupCalendar_RealmSettings.Channel.AutoConfigPlayer then
			GroupCalendarStoreAutoConfig:SetChecked(true);
		else
			GroupCalendarStoreAutoConfig:SetChecked(false);
		end
		
		GroupCalendar_UpdateChannelStatus();
		
	elseif pPanelIndex == 3 then
		-- Trust panel
		
		GroupCalendarTrustGroup.ChangedValueFunc = GroupCalendar_UpdateEnabledControls;
		
		if gGroupCalendar_RealmSettings.Security.TrustGuildies
		and not IsInGuild() then
			gGroupCalendar_RealmSettings.Security.TrustGuildies = false;
		end
		
		if gGroupCalendar_RealmSettings.Security.TrustAnyone then
			UIDropDownMenu_SetSelectedValue(GroupCalendarTrustGroup, 1);
		elseif gGroupCalendar_RealmSettings.Security.TrustGuildies then
			UIDropDownMenu_SetSelectedValue(GroupCalendarTrustGroup, 2);
		else
			UIDropDownMenu_SetSelectedValue(GroupCalendarTrustGroup, 3);
		end
		
		if gGroupCalendar_RealmSettings.Security.MinTrustedRank ~= nil then
			UIDropDownMenu_SetSelectedValue(GroupCalendarTrustMinRank, gGroupCalendar_RealmSettings.Security.MinTrustedRank);
		end
		
		CalendarPlayerList_SetItemFunction(CalendarTrustedPlayersList, GroupCalendar_GetIndexedTrustedPlayer);
		CalendarPlayerList_SetSelectionChangedFunction(CalendarTrustedPlayersList, GroupCalendar_TrustedPlayerSelected);
		
		CalendarPlayerList_SetItemFunction(CalendarExcludedPlayersList, GroupCalendar_GetIndexedExcludedPlayer);
		CalendarPlayerList_SetSelectionChangedFunction(CalendarExcludedPlayersList, GroupCalendar_ExcludedPlayerSelected);
	end
	
	GroupCalendar_UpdateEnabledControls();
	
	getglobal(gGroupCalendar_PanelFrames[pPanelIndex]):Show();
	
	PanelTemplates_SetTab(GroupCalendarFrame, pPanelIndex);
end

function GroupCalendar_HidePanel(pFrameIndex)
	if gGroupCalendar_CurrentPanel ~= pFrameIndex then
		return;
	end
	
	GroupCalendar_SavePanel(pFrameIndex);
	
	getglobal(gGroupCalendar_PanelFrames[pFrameIndex]):Hide();
	gGroupCalendar_CurrentPanel = 0;
end

function GroupCalendar_UpdateChannelStatus()
	local	vChannelStatus = CalendarNetwork_GetChannelStatus();
	local	vStatusText = GroupCalendar_cChannelStatus[vChannelStatus];
	
	GroupCalendarChannelStatus:SetText(vStatusText.mText);
	GroupCalendarChannelStatus:SetTextColor(vStatusText.mColor.r, vStatusText.mColor.g, vStatusText.mColor.b);
	
	if vChannelStatus == "Connected" then
		Calendar_SetButtonEnable(GroupCalendarConnectChannelButton, true);
		GroupCalendarConnectChannelButton:SetText(GroupCalendar_cDisconnectChannel);
	elseif vChannelStatus == "Disconnected" then
		Calendar_SetButtonEnable(GroupCalendarConnectChannelButton, true);
		GroupCalendarConnectChannelButton:SetText(GroupCalendar_cConnectChannel);
	else
		Calendar_SetButtonEnable(GroupCalendarConnectChannelButton, false);
	end
end

function GroupCalendar_FixPlayerName(pName)
	if pName == nil
	or pName == "" then
		return nil;
	end
	
	return string.upper(string.sub(pName, 1, 1))..string.lower(string.sub(pName, 2));
end

function GroupCalendar_AddTrustedPlayer(pPlayerName)
	local	vPlayerName = GroupCalendar_FixPlayerName(pPlayerName);
	
	if vPlayerName == nil then
		return;
	end
	
	gGroupCalendar_RealmSettings.Security.Player[vPlayerName] = 1;
	GroupCalendar_UpdateTrustedPlayerList();
	CalendarTrust_TrustSettingsChanged();
end

function GroupCalendar_AddExcludedPlayer(pPlayerName)
	local	vPlayerName = GroupCalendar_FixPlayerName(pPlayerName);

	if vPlayerName == nil then
		return;
	end
	
	gGroupCalendar_RealmSettings.Security.Player[vPlayerName] = 2;
	GroupCalendar_UpdateTrustedPlayerList();
	CalendarTrust_TrustSettingsChanged();
end

function GroupCalendar_RemoveTrustedPlayer(pPlayerName)
	local	vPlayerName = GroupCalendar_FixPlayerName(pPlayerName);

	if vPlayerName == nil then
		return;
	end
	
	gGroupCalendar_RealmSettings.Security.Player[vPlayerName] = nil;
	
	GroupCalendar_UpdateTrustedPlayerList();
	
	CalendarPlayerList_SelectIndexedPlayer(CalendarTrustedPlayersList, 0);
	CalendarPlayerList_SelectIndexedPlayer(CalendarExcludedPlayersList, 0);
	
	CalendarTrust_TrustSettingsChanged();
end

function GroupCalendar_UpdateTrustedPlayerList()
	CalendarPlayerList_Update(CalendarTrustedPlayersList);
	CalendarPlayerList_Update(CalendarExcludedPlayersList);
end

function GroupCalendar_GetIndexedTrustedPlayer(pIndex)
	if pIndex == 0 then
		return CalendarTrust_GetNumTrustedPlayers(1);
	end
	
	return
	{
		Text = CalendarTrust_GetIndexedTrustedPlayers(1, pIndex);
	};
end

function GroupCalendar_GetIndexedExcludedPlayer(pIndex)
	if pIndex == 0 then
		return CalendarTrust_GetNumTrustedPlayers(2);
	end
	
	return
	{
		Text = CalendarTrust_GetIndexedTrustedPlayers(2, pIndex);
	};
end

function GroupCalendar_TrustedPlayerSelected(pIndex)
	if pIndex == 0 then
		return;
	end
	
	CalendarPlayerList_SelectIndexedPlayer(CalendarExcludedPlayersList, 0);
	
	local	vName = CalendarTrust_GetIndexedTrustedPlayers(1, pIndex);
	
	if vName then
		CalendarTrustedPlayerName:SetText(vName);
		CalendarTrustedPlayerName:HighlightText();
		CalendarTrustedPlayerName:SetFocus();
	end
end

function GroupCalendar_ExcludedPlayerSelected(pIndex)
	if pIndex == 0 then
		return;
	end
	
	CalendarPlayerList_SelectIndexedPlayer(CalendarTrustedPlayersList, 0);
	
	local	vName = CalendarTrust_GetIndexedTrustedPlayers(2, pIndex);
	
	if vName then
		CalendarTrustedPlayerName:SetText(vName);
		CalendarTrustedPlayerName:HighlightText();
		CalendarTrustedPlayerName:SetFocus();
	end
end

function GroupCalendar_SelectDate(pDate)
	Calendar_SetSelectedDate(pDate);
	
	local	vCompiledSchedule = EventDatabase_GetCompiledSchedule(pDate);
	
	CalendarEditor_ShowCompiledSchedule(pDate, vCompiledSchedule);
end

function GroupCalendar_EditorClosed()
	Calendar_ClearSelectedDate();
end

function GroupCalendar_EventChanged(pDatabase, pEvent, pChangedFields)
	local	vSchedule = pDatabase.Events[pEvent.mDate];
	
	GroupCalendar_ScheduleChanged(pDatabase, pEvent.mDate, vSchedule);
	CalendarEventEditor_EventChanged(pEvent);
end

function GroupCalendar_ScheduleChanged(pDatabase, pDate, pSchedule)
	CalendarEditor_ScheduleChanged(pDate, pSchedule);
	Calendar_ScheduleChanged(pDate, pSchedule);
	CalendarEventViewer_ScheduleChanged(pDate);
end

function GroupCalendar_MajorDatabaseChange(pDatabase)
	CalendarEditor_MajorDatabaseChange();
	Calendar_MajorDatabaseChange();
end

function GroupCalendar_DebugMessage(pMessage)
	DEFAULT_CHAT_FRAME:AddMessage(pMessage, 0.7, 0.3, 1.0);
end

gGroupCalendar_QueueElapsedTime = 0;

function GroupCalendar_Update(pElapsed)
	-- Process the event queues if it's time
	
	gGroupCalendar_QueueElapsedTime = gGroupCalendar_QueueElapsedTime + pElapsed;
	
	if gGroupCalendar_QueueElapsedTime > 0.5 then
		CalendarNetwork_ProcessQueues(gGroupCalendar_QueueElapsedTime);
		
		gGroupCalendar_QueueElapsedTime = 0;
		
		if not CalendarNetwork_NeedsUpdateTimer() then
			if gGroupCalendar_Settings.DebugQueues then
				GroupCalendar_DebugMessage("GroupCalendar_Update: Stopping timer");
			end
			
			GroupCalendarUpdateFrame:Hide();
		end
	end
end

function GroupCalendar_StartUpdateTimer()
	if GroupCalendarUpdateFrame:IsVisible() then
		return;
	end
	
	if gGroupCalendar_Settings.DebugQueues then
		GroupCalendar_DebugMessage("GroupCalendar_StartUpdateTimer()");
	end
	
	GroupCalendarUpdateFrame:Show();
end

function GroupCalendar_StartMoving()
	if not gGroupCalendar_RealmSettings.UI.LockWindow then
		GroupCalendarFrame:StartMoving();
	end
end

function GroupCalendar_GetSettings(pPlayerName, pRealmName)
	-- Wipe the settings if they're the alpha version
	
	if not gGroupCalendar_Settings.Format then
		gGroupCalendar_Settings =
		{
			Debug = false,
			Format = 1,
		};
	end
	
	local	vSettings = gGroupCalendar_Settings[pRealmName.."_"..pPlayerName];
	
	if vSettings == nil then
		if IsInGuild() then
			GroupCalendar_DebugMessage(GroupCalendar_cInitializingGuilded);
		else
			GroupCalendar_DebugMessage(GroupCalendar_cInitializingUnguilded);
		end
		
		vSettings =
		{
			Security =
			{
				TrustAnyone = false,
				TrustGuildies = IsInGuild(),
				MinTrustedRank = 1,
				Player = {},
			},
			
			Channel =
			{
				AutoConfig = IsInGuild(),
				AutoConfigPlayer = nil,
				Name = nil,
				Password = nil,
			},
			
			UI =
			{
				LockWindow = false,
			},
		};
		
		gGroupCalendar_Settings[pRealmName.."_"..pPlayerName] = vSettings;
	end
	
	return vSettings;
end

function GroupCalendar_RoundTimeOffsetToNearest30(pOffset)
	local	vNegativeOffset;
	local	vOffset;
	
	if pOffset < 0 then
		vNegativeOffset = true;
		vOffset = -pOffset;
	else
		vNegativeOffset = false;
		vOffset = pOffset;
	end
	
	vOffset = vOffset - (math.mod(vOffset + 15, 30) - 15);
	
	if vNegativeOffset then
		return -vOffset;
	else
		return vOffset;
	end
end

function GroupCalendar_CalculateTimeZoneOffset()
	local	vServerTime = Calendar_ConvertHMToTime(GetGameTime());
	local	vLocalTime = Calendar_GetCurrentLocalTime();
	local	vTimeDelta = vLocalTime - vServerTime;
	
	gGroupCalendar_ServerTimeZoneOffset = GroupCalendar_RoundTimeOffsetToNearest30(vTimeDelta);
	
	if gGroupCalendar_ServerTimeZoneOffset > 690 then -- 11.5 hours
		gGroupCalendar_ServerTimeZoneOffset = gGroupCalendar_ServerTimeZoneOffset - 1440;
	elseif gGroupCalendar_ServerTimeZoneOffset < -720 then -- 12 hours
		gGroupCalendar_ServerTimeZoneOffset = gGroupCalendar_ServerTimeZoneOffset + 1440;
	end
end

function GroupCalendar_UpdateTimeTooltip()
	local	vServerTime = Calendar_ConvertHMToTime(GetGameTime());
	local	vLocalDate = Calendar_GetCurrentLocalDate();
	
	local	vServerTimeString = Calendar_GetShortTimeString(vServerTime);
	local	vLocalDateString = Calendar_GetLongDateString(vLocalDate, true);

	GameTooltip:AddLine(vLocalDateString);
	GameTooltip:AddLine(vServerTimeString);
	
	if gGroupCalendar_ServerTimeZoneOffset ~= 0 then
		local	vLocalTime = vServerTime + gGroupCalendar_ServerTimeZoneOffset;
		local	vLocalTimeString = Calendar_GetShortTimeString(vLocalTime);
		
		GameTooltip:AddLine(string.format(GroupCalendar_cLocalTimeNote, vLocalTimeString));
	end
	
	GameTooltip:Show();
end

function GroupCalendar_ChannelChanged()
	if GroupCalendarFrame:IsVisible()
	and gGroupCalendar_CurrentPanel == 2 then
		if GroupCalendarAutoChannelConfig:GetChecked() then
			GroupCalendar_ShowPanel(gGroupCalendar_CurrentPanel); -- Refresh the channel and auto-player names
		end
		
		GroupCalendar_UpdateChannelStatus();
	end
end

function GroupCalendar_GetLocalizedStrings(pLocale)
	local	vStrings = {};
	
	-- Initialize the strings with copies form the enUS set
	
	for vIndex, vString in gCalendarLocalizedStrings.enUS do
		vStrings[vIndex] = vString;
	end
	
	-- Select a set to use for overwriting
	
	local	vLocalizedStrings = gCalendarLocalizedStrings[pLocale];
	
	if not vLocalizedStrings then
		-- There's not a fit for the exact language/country specified, so just match the language
		
		local	vLanguageCode = string.sub(pLocale, 1, 2);
		
		vLocalizedStrings = GroupCalendar_SelectLanguage(vLanguageCode);
		
		if not vLocalizedStrings then
			return vStrings;
		end
	end
	
	-- Overwrise the english strings with translated strings
	
	for vIndex, vString in vLocalizedStrings do
		vStrings[vIndex] = vString;
	end
	
	return vStrings;
end

function GroupCalendar_SelectLanguage(pLanguageCode)
	-- There's not a fit for the exact language/country specified, so just match the language

	local	vLanguageCode = string.sub(pLocale, 1, 2);

	for vLocale, vLocalizedStrings in gCalendarLocalizedStrings do
		if pLanguageCode == string.sub(vLocale, 1, 2) then
			return vLocalizedStrings;
		end
	end
	
	-- No luck, hope they know english!
	
	return nil;
end

function GroupCalendar_ToggleChannelConnection()
	local	vChannelStatus = CalendarNetwork_GetChannelStatus();
	
	if vChannelStatus == "Initializing" then
		return;
	end
	
	if vChannelStatus == "Connected" then
		gGroupCalendar_Channel.Disconnected = true;
		
		CalendarNetwork_LeaveChannel();
	else
		gGroupCalendar_Channel.Disconnected = false;
		
		if gGroupCalendar_RealmSettings.Channel.AutoConfig then
			CalendarNetwork_ScheduleAutoConfig(0.5);
		elseif gGroupCalendar_RealmSettings.Channel.Name then
			CalendarNetwork_SetChannel(
					gGroupCalendar_RealmSettings.Channel.Name,
					gGroupCalendar_RealmSettings.Channel.Password);
		end
	end
end
